%% 1.2. Sucesin de Fibonacci
%
% Este problema muestra la construccin de los 
% nmeros de Fibonacci a partir de la frmula recurrente
% y comprueba que el cociente de los trminos consecutivos
% tiende a la razn urea
%
%
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson


%% 
%
% a) Sucesin de Fibonacci
%
% Cada trmino de la sucesin es la suma de los dos anteriores. 
% los dos primeros trminos se definen arbitrariamente. Por ejemplo, 1 y 1. 
% Los siguientes se calculan mediante la frmula F(k) = F(k-1) + F(k-2)
%
% Por ejemplo:

F(1)=1; F(2)=1;
for k=3:8
    F(k) = F(k-1) + F(k-2);
end

bar(F,'r')


%% 
%
% b) Programa la sucesin de Fibonacci
%
% La siguiente funcin obtiene n trminos de la sucesin de Fibonacci 
% que comienza en 1, 1.

type fibonacci


%%
%
% c) Estabilizacin de los coeficientes
%
% El cociente de cada trmino de la sucesin de Fibonacci por el anterior
% se estabiliza rpidamente

C = F(2:8)./F(1:7)

plot(C)


%%
%
% El siguiente programa va calculando trminos de la sucesin de Fibonacci
% y sus cocientes sucesivos hasta que stos se estabilizan.
% 
% function [fi,C] = aureo(tol,maxiter)
%
% % Calcula el limite del cociente de los trminos
% % de la sucesin de Fibonacci, con error < tol
%
% F(1) = 1; 
% F(2) = 1;
% C(1) = F(2)/F(1);
% k = 3;
% incr = tol+1;
% while incr > tol & k < maxiter
%     F(k) = F(k-1) + F(k-2);
%     C(k-1) = F(k)/F(k-1);
%     incr = abs(C(k-1)-C(k-2));
%     k = k+1;
% end
% fi = C(end);
%
% % Aviso de no convergencia
% if incr > tol
%     disp('Insuficientes iteraciones')
% end


%%
%
% Modificando un par de instrucciones, el inicio de la sucesin de 
% Fibonacci puede elegirse arbitrariamente.

type aureo


%% 
% 
% Nmero ureo
% 
% Calculamos el nmero ureo con precisin de milmillonsimas mediante

[fi,C] = aureo(1,1,1e-9,50);

format long, C',format short


%%
%
% e) Los cocientes tienden a la razn urea independientemente
% de los posibles inicios de la sucesin

Au = [];
for k = 1:0.2:3
    [fj,C] = aureo(1,k,0,15);
    Au=[Au;C];
end
plot(Au'), axis([1 15 1 3])


%%
%
% Expresin matricial de la recurrencia
% 
% La matriz A = [0 1; 1 1] avanza un paso en la sucesin de Fibonacci al
% al aplicarla al vector columna formado por un par de trminos consecutivos

A = [0 1; 1 1]; 
f = [1; 1];
for k = 1:10
    f = A*f
end


%%
%
% El nmero ureo es el mayor valor propio de la matriz de recurrencia

format long
eig(A)
fi
format short


%%
%
% FIN del Problema 1.2
