%% 1.3. Polinomios de Taylor de la funcin logaritmo
%
% Este problema muestra la aproximacin de la funcin log(1+x) por
% polinomios de Taylor en un entorno del origen y estima el valor log(1.5)
% con un margen de error prefijado.
%
%
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson


%% 
%
% a) Polinomio de Taylor de log(1+x): x - x^2/2 +x^3/3 - x^4/4 +  
%
% Tomamos un vector columna x con componentes equiespaciadas de -0.99 a 1, 
% dejando aparte el -1 para evitar la singularidad del logaritmo.

x = (-0.99:0.01:1)';
plot(x,log(1+x),'.-')
title('y = log(1+x)')
axis([-1 1 -2 1])
grid


%% 
%
% La aproximacin de grado 1 es:

y = x;

plot(x,log(1+x),'.-',x,y)
axis([-1 1 -2 1])
title('Polinomios de Taylor del logaritmo')
legend('Logaritmo', 'Aproximacin lineal', 'Location', 'SouthEast')
grid


%%
%
% Almacenamos en las columnas de la matriz y los valores de las sucesivas
% aproximaciones y las representamos grficamente.

for k = 2:7
    y(:,k) = y(:,k-1) + (-1)^(k+1)*x.^k/k;
    plot(x, log(1+x), '.-', x, y)
    h = get(gca, 'Children');
    legend(h, str2mat('log(1+x)', [repmat('Grado ', k-1, 1) num2str((k-1:-1:1)')]), ...
    'Location', 'SouthEast')
    title('Polinomios de Taylor del logaritmo')
    grid
    axis([-1 1 -2 1])
    pause(1)
end
hold on


%%
%
% b) Evaluamos el desarrollo en x = 1.5, aadiendo trminos hasta que su
% magnitud sea menor que 10^-3.

a = 1.5; 
x = a - 1; loga = x; incr = x; k = 2;

disp(' ')
disp('Aproximacin de log(1.5) por polinomios de Taylor')
disp(' ')
disp('Grado  Aproximacin')
disp([1 loga])
axis([0.4 0.6 0.3 0.55])
hold on
while abs(incr) > 1e-3 & k < 10
    plot(x,loga,'r.')
    incr = (-1)^(k+1)*x^k/k;
    loga = loga + incr;
    disp([k loga])
    k = k+1;
end
hold off
disp(' ')
disp(['La aproximacin de grado ', num2str(k-1),...
' de log(', num2str(a),') es ', num2str(loga)])
disp(' ')
disp(['El logaritmo neperiano de  ', num2str(a),' es ', num2str(log(a))])


%%
%
% El error cometido es menor que la tolerancia impuesta.

abs(loga-log(a))


%%
%
% FIN del Problema 1.3