%% 1.4. Aproximacin del nmero PI
%
% Este problema compara distintas series que aproximan
% el numero PI, hallando el nmero de trminos necesarios
% para estimarlo con una precisin determinada.
%
%
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson


%% 
%
% a) Serie de trmino general (-1)^k*4/(2*k+1)
% 

s = 4;
for k = 1:99
    s(k+1) = s(k) +(-1)^k*4/(2*k+1);
end
s'

plot([1 99], [pi pi])
hold on
stairs(s(2:end),'r')
hold off
title('Serie (-1)^k*4/(2*k+1)')
xlabel('Posicin')
legend('Valor de \pi', 'Suma parcial')
grid


%%
% 
% Como los trminos de la serie alternan signo positivo y negativo, 
% el valor absoluto del ultimo trmino sumado, 4/(2n + 1), es una cota 
% del error de la aproximacin. La convergencia es desesperadamente lenta.  

4/201
abs(s(100)-pi)


%%
%
% Sustituimos el desarrollo del arco tangente:
%        atan(x) = x - x^3/3 + x^5/5 - 
% en la expresin 
%        pi = 4*(atan(1/2)+atan(1/3))
% 
% Iteramos hasta que la variacin sea menor que 1e-6.

tol = 1e-6; incr = tol+1;
s = 0; k = 0;
maxiter = 50;
while incr>tol & k<maxiter
    j = 2*k+1;
    incr = 4*(1/2^j/j + 1/3^j/j);
    s(k+2) = s(k+1) + (-1)^k*incr;
    k = k+1;
end
format long, s(2:end)',pi,format short

plot([1 10], [pi pi])
hold on
stairs(s(2:end),'r')
hold off
title('Desarrollo de 4(atan(1/2) + atan(1/3))')
xlabel('Posicin')
legend('Valor de  \pi', 'Suma parcial')
grid


%%
% 
% La precisin del resultado es del orden de 10^-7, un orden mayor que el estimado, 
% debido a la rapidez de la convergencia.  
abs(s(end)-pi)


%%
%
% c) Utilizamos ahora la expresin
%         pi =16*atan(1/5)-4*atan(1/239)
  
tol = 1e-12; incr = tol+1;
s = 0; k = 0;
maxiter = 50;
while incr>tol & k<maxiter
    j = 2*k+1;
    incr = 16*1/5^j/j -4/239^j/j;
    s(k+2) = s(k+1) + (-1)^k*incr;
    k = k+1;
end
format long, s(2:end)',pi,format short

plot([1 10], [pi pi])
hold on
stairs(s(2:end),'r')
hold off
title('Desarrollo de 16*atan(1/5) - 4*atan(1/239)')
xlabel('Posicin')
legend('Valor de  \pi', 'Suma parcial')
grid


%%
% 
% La precisin del resultado es del orden de 10^-15, con el mismo
% nmero de pasos que la iteracin del apartado anterior.
abs(s(end)-pi)


%%
%
% FIN del Problema 1.4