%% 1.5. Divisin de Polinomios. Teorema del resto
%
% Este problema usa la orden deconv para dividir un polinomio por x-a y la
% orden polyval para evaluarlo y comprobar el teorema del resto.
%
%
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson


%% 
%
% Los polinomios en MATLAB se almacenan como vectores cuyas componentes son
% los coeficientes del polinomio ordenados de mayor a menos grado,
% incluyendo ceros en los trminos que faltan
%
% Representamos grficamente el polinomio p(x) = x^3 - 2x - 5
% 

p = [1 0 -2 -5];
x = linspace(-3,3);
y = polyval(p,x);
plot(x, y, 'm')
title('p(x) = x^3 - 2x - 5')
grid
set(gca, 'YMinorTick', 'on')


%%
% 
% La divisin de polinomios se efecta mediante deconv
%

divisor = [1 -2];
[cociente, resto] = deconv(p, divisor)

yd = polyval(divisor, x);
yc = polyval(cociente, x);
yr = polyval(resto, x);
hold on
plot(x, yd, x, yc, x, yr)
legend('Dividendo', 'Divisor', 'Cociente', 'Resto', 'Location', 'SouthEast')


%%
%
% La prueba de la divisin,
%     dividendo = divisor x cociente + resto
% con MATLAB es

p % dividendo
conv(divisor, cociente) +  resto


%%
% 
% El Teorema del Resto establece que el resto de la divisin de un
% polinomio entre x - a coincide con el valor del polinomio en x = a
%
% Evaluamos el polinomio p(x) en x = 2. El resultado coincide con el resto.

y2 = polyval(p,2)

plot(2, y2, 'g*')

%%
%
% FIN del Problema 1.5