%% 1.6. Cambio de base de polinomios
%
% En este problema se muestran varios procedimientos para expresar un
% polinomio en x como combinacin lineal de potencias de x - a.
%
%
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson


%% 
%
% El polinomio p(x) = 5x^3 - x^2 + 3x + 1 es combinacin lineal de
% potencias de x. La figura muestra el polinomio en el intervalo [-1, 1] y
% cada uno de los monomios que lo componen.

x = linspace(-1,2);
p = [5 -1 3 1];
y = polyval(p,x);
plot(x, y, '.-', x, [ones(size(x)); 3*x; -x.^2; 5*x.^3])
title('p(x) = 5x^3 - x^2 + 3x + 1')
legend('Total', 'Grado 0', 'Grado 1', 'Grado 2', 'Grado 3', ...
    'Location', 'NorthWest')
grid


%% 
%
% Para expresar el polinomio p(x) como combinacin lineal de potencias de 
% x - 1:
%      p(x) = a(x - 1)^3 + b(x - 1)^2 + c(x - 1) + d
% resolvemos el sistema lineal que se obtiene al identificar los
% coeficientes de las potencias del mismo grado.

A = [1 0 0 0; -3 1 0 0; 3 -2 1 0; -1 1 -1 1]
q = A\p'


%% 
%
% El polinomio p(x) = 5x^3 - x^2 + 3x + 1 expresado en la nueva base es 
%      p(x) = 5(x - 1)^3 + 14(x - 1)^2 + 16(x - 1) + 8
% La grafica del polinomio es la misma que antes, pero las de los monomios
% que lo componen son distintas:

y = polyval(q,x-1);

plot(x, y, '.-', x, [q(4)*ones(size(x)); q(3)*(x-1); q(2)*(x-1).^2; q(1)*(x-1).^3])
title('p(x) = 5(x - 1)^3 + 14(x - 1)^2 + 16(x - 1) + 8')
legend('Total', 'Grado 0', 'Grado 1', 'Grado 2', 'Grado 3')
grid


%%
%
% Alternativamente, los coeficientes obtenidos son los del desarrollo de
% Taylor de p(x) en x = 1:

d = polyval(p,1);
Dp = polyder(p);    c = polyval(Dp,1);
D2p = polyder(Dp);  b = polyval(D2p,1)/2;
D3p = polyder(D2p); a = polyval(D3p,1)/6;
disp([a b c d])


%%
%
% Tambin pueden obtenerse los coeficientes como restos de divisiones 
% sucesivas de p(x) por x - 1:

p = [5 -1 3 1];
g = [1 -1]; % x-1
[p1,d] = deconv(p,g);
[p2,c] = deconv(p1,g);
[p3,b] = deconv(p2,g);
[p4,a] = deconv(p3,g);
disp([a(end) b(end) c(end) d(end)])


%%
%
% FIN del Problema 1.6