%% 1.7. Campo elctrico creado por un dipolo
%
% En este problema representamos grficamente el potencial  y el campo
% elctrico generados en el plano por un dipolo.
%
%
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson


%% 
%
% Representamos el campo en el cuadrado [-5, 5][-4, 4]

q = 2e-6; k = 9e9;
a = 1.5; b = -1.5;
x = -5 : 1/3 : 5;
y = -4 : 1/3 : 4;
[X,Y] = meshgrid(x,y);
rp = sqrt((X-a).^2+(Y-b).^2);
rn = sqrt((X+a).^2+(Y+b).^2);
V = q*k*(1./rp-1./rn);

surf(X,Y,V)
title('Potencial del campo elctrico creado por un dipolo')


%% 
%
% Dibujamos las curvas equipotenciales mediante contour, fijando
% manualmente los niveles a representar en P:

N = 10.^(2.5:.5:4.5);
P = [-fliplr(N) 0 N];
contour(X,Y,V,P)
text([a -a], [b -b], ['+'; '-'], 'Color', 'r',  ...
    'FontSize', 16, 'HorizontalAlignment', 'Center')
title('Curvas equipotenciales')
axis equal


%% 
%
% El campo elctrico es menos el gradiente del potencial. Se representa 
% mediante quiver.

[Ex,Ey] = gradient(-V);
hold on
quiver(X,Y,Ex,Ey)
title('Intensidad del campo elctrico creado por un dipolo')
hold off


%%
%
% Para visualizar las lineas de campo, normalizamos  los vectores del campo
% elctrico y volvemos a aplicar quiver.

E = sqrt(Ex.^2+Ey.^2);
quiver(X,Y,Ex./E,Ey./E)
title('Lneas de campo')
text([a -a], [b -b], ['+'; '-'], 'Color', 'r',  ...
    'FontSize', 16, 'HorizontalAlignment', 'Center')
axis equal


%%
%
% FIN del Problema 1.7