%% 1.8. Generacin de superficies
%
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson
%
% En este problema representamos graficamente superficies generadas por
% curvas planas que se mueven en el espacio.


%% 
%
% a) Superficie trica
% Sea C(r, 0, z) un punto genrico de una circunferencia situada en el
% plano XZ con centro en (2, 0, 0) y radio b = 1. 

a = 2;
b = 1;
axis([-3 3 -3 3 -1 1]), axis equal
hold on
u = linspace(0,2*pi,100);
r = a + b*cos(u);
y = zeros(size(u));
z = b*sin(u);
plot3(r,y,z,'r')
title('Circunferencia generatriz del toro')
grid


%%
%
% Al girar entorno a Z un angulo s, el punto (r, 0, z) recorre las
% posiciones (x, y, z).

title('Generacin del toro')
for v = pi/10 : pi/10 : 1.9*pi
    x = r*cos(v);
    y = r*sin(v);
    plot3(x, y, z)
    pause(0.1)
end
hold off


%%
%
% Traducimos a MATLAB la parametrizacin de la superficie y la
% representamos con surf.

s = 0 : pi/10 : 2*pi;
t = 0 : pi/10 : 2*pi;
[T,S] = meshgrid(t,s);
R = a + b*cos(T);
Z = b*sin(T);
X = R.*cos(S);
Y = R.*sin(S);
surf(X, Y, Z), axis equal off
title('Superficie trica')


%%
%
% b) Una recta R situada sobre el eje OX se desplaza en la direccin del
% eje OZ al tiempo que gira en torno al mismo un angulo t igual a la altura
% z, generando la superficie helicoidal.

Rx = [-2, 2]; Ry = [0, 0]; Rz = [0, 0];
plot3(Rx, Ry, Rz, 'r')
title('Recta generatriz de la superficie helicoidal')
axis ([-2 2 -2 2 0 pi])
xlabel('Eje x')
ylabel('Eje y')
grid


%%
%
% b) Una recta R situada sobre el eje OX se desplaza en la direccin del
% eje OZ al tiempo que gira en torno al mismo un angulo t igual a la altura
% z, generando la superficie helicoidal.

hold on
title('Generacin de la superficie helicoidal')
for t = pi/20 : pi/20 : pi 
    plot3(Rx*cos(t), -Rx*sin(t), [t, t]),
    pause(0.1)
end
hold off


%%
%
% Para representar graficamente la superficie generada, tomamos como 
% parametros la distancia s al eje Z y el angulo girado t.

s = -2 : 0.2 : 2;
t = 0 : pi/20 : 2*pi;
[T,S] = meshgrid(t,s);
X =  S.*cos(T);
Y = -S.*sin(T);
Z = T;
surf(X, Y, Z)
title('Superficie helicoidal')
axis off


%%
%
%  c) Una recta gira entorno a otra a la que no corta.

a = pi/6; 
Rx = [1 1]; Ry = [-sin(a) sin(a)]; Rz = [-1 1];
plot3(Rx, Ry, Rz, 'r')
axis([-1.5 1.5 -1.5 1.5 -1 1])
title('Recta generatriz del hiperboliode de una hoja')
xlabel('Eje x')
ylabel('Eje y')
grid


%%
%
%  c) Una recta gira entorno a otra a la que no corta.

hold on
title('Generacin del hiperboliode de una hoja')
for t = 0.1:0.1:6.2
    plot3(Rx*cos(t) - Ry*sin(t), Rx*sin(t) + Ry*cos(t), Rz)
    pause(0.1)
end
hold off


%%
%
% Representamos la superficie mediante esta parametrizacin.

s = linspace(-1, 1, 10);
t = linspace(0, 2*pi, 25);
a = pi/6; d = 1;
[T,S] = meshgrid(t,s);
X = d*cos(T) - S*cos(a).*sin(T);
Y = d*sin(T) + S*cos(a).*cos(T);
Z = S*sin(a);
surf(X, Y, Z)
title('Hiperboloide de una hoja')
axis off


%%
%
% FIN del Problema 1.8