%% 1.10. La distribucin normal
%
% En este problema se aproxima la distribucin normal mediante numeros
% aleatorios y se sinula el movimiento browniano.
%
%
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson


%% 
%
% a) Grafica de la funcin densidad de la distribucin normal
t = -4:0.1:4;
y = 1/sqrt(2*pi)*exp(-t.^2/2);
plot(t,y)
title('Densidad de la distribucin normal')

%%
%
% Aproximacin mediante numeros aleatorios. Generamos numeros aleatorios
% normalmente distribuidos con randn. La media y la desviacin tipica se
% aproximan a los valores de la distribucin teorica.

x = randn(1,1000);
mean(x)
std(x)


%%
%
% Histograma de los valores obtenidos. La orden hist agrupa los datos en
% intervalos y cuenta las frecuencias. Tomamos intervalos de longitud
% h=0.4.

h = 0.4;
c = -4:h:4;
frec = hist(x,c)


%%
%
% Representaremos las frecuencias relativas en un diagrama de barras. Para
% compararlo con la funcin densidad de probabilidad, dividimos las
% frecuencias relativas por la longitud del subintervalo.

frel = frec/1000;
hold on
bar(c,frel/h,'y')
hold off


%%
%
% Calculamos las frecuencias relativas acumuladas con cumsum y las
% representamos junto con las probabilidades dadas por la funcin de
% distribucion. 

z = (1+erf(t/sqrt(2)))/2;
plot(t,z)
title('Funcin de distribucin normal')
hold on
frelac = cumsum(frel);
bar(c+h/2,frelac,'y')
hold off


%%
%
% c) Simulacin del movimiento browniano.

x = randn(1,1000); y = randn(1,1000);
p = cumsum(x); q = cumsum(y);
plot(p,q,'r')
axis equal off


%%
%
% FIN del Problema 1.10