function [sol,x,incr,iter] = biseccion(f,a,b,tol,maxiter)

% [sol,x,incr,iter]=biseccion(f,a,b,tol,maxiter)
% 
% Aplica biseccin a la ecuacin f(x)=0 en el intervalo [a,b]
%
% tol: precisin deseada
% maxiter: tope de iteraciones
%
% sol: solucin aproximada
% x: vector de iterados
% incr: distancia entre los dos ltimos
% iter: numero de iteraciones
%
% Ejemplo: biseccion('cos',1,2,0.001,10)


% Problemas resueltos de Mtodos Numricos
% Cordero, Hueso, Martnez, Torregrosa
% Ed. Thomson
%
% jlh, jun-2005

% Comprueba que hay cambio de signo en el intervalo

fa = feval(f,a);
fb = feval(f,b);
if fa*fb>0
   disp('La funcin debe tener signo distinto en los extremos del intervalo')
   return
end

% Inicializar variables
iter = 0;
x = [];
incr = b-a;

% Biseccin
while incr>tol & iter<maxiter
   c = (a+b)/2;
   x = [x,c];						
   fc = feval(f,c);
   if fc==0							% Por casualidad, c es la solucin
      a = c; b = c;
   elseif fa*fc<0					% cambio de signo en [a,c]
      b = c; fb = fc;
   else  %   fc*fb<0				% cambio de signo en [c,b]
      a = c; fa = fc;
   end
   incr = b-a;
   iter = iter+1;					% Contador de iteraciones
end

% Salida
sol = c;
if incr>tol
   disp('Diverge o insuficientes iteraciones')
end
