%% 2.1. Races de un polinomio por bisecin
% 
% Este problema muestra paso a paso el funcionamiento del mtodo
% de biseccin para calcular una raz de un polinomio.
%
% 
% Tema 2, Problema 1
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson


%% 
%
% Representamos grficamente el polinomio p(x) = 2x^3 - 3x - 1 en el intervalo [-2,2]:

x = linspace(-2,2);
p = [2 0 -3 -1];
y = polyval(p,x);
plot(x,y)
title('p(x) = 2x^3 - 3x - 1')
grid


%%
%
% Observamos dos races negativas y una positiva. 
% Partimos del intervalo [a, b] = [0, 2] para hallar la raz positiva.
% Verificamos que el polinomio tiene distinto signo en los extremos del
% intervalo.

a = 0; fa = polyval(p,a)
b = 2; fb = polyval(p,b)
x = linspace(a,b);
y = polyval(p,x);
plot(x, y, [a - 0.1, b + 0.1], [0, 0], 'k', [a b], [0 0], 'g*', ...
    [a a], [0 fa], 'r:', [b b], [0 fb], 'r:')
txa = text(a, -0.5*sign(fa), 'a');
txb = text(b, -0.5*sign(fb), 'b');
title('Mtodo de biseccin')


%%
%
% Calculamos el punto medio c del intervalo [a, b], evaluamos el polinomio
% en c y determinamos en qu mitad de [a, b] se produce el cambio de signo.

c = (a+b)/2
fc = polyval(p,c)
hold on
plot(c, 0, 'g*', [c c], [0 fc], 'r:')
txc = text(c, -0.5*sign(fc), 'c');


%%
%
% Comprobamos que el cambio de signo tiene lugar en el semiintervalo 
% [c, b] = [1, 2].
% Lo seleccionamos dando al extremo izquierdo a el valor c = 1, es decir,
% desplazamos el extremo izquierdo del intervalo al punto medio.

a = c
set(txc,'Visible','off')
set(txa,'Position',[c, -0.5*sign(fc)])


%%
%
% Repetimos el proceso hallando el punto medio de [1, 2] y seleccionando el
% semiintervalo en el que el polinomio cambia de signo. 

c = (a+b)/2
fc = polyval(p,c)
plot(c, 0, 'g*', [c c], [0 fc], 'r:')
set(txc, 'Position', [c, -0.5*sign(fc)], 'Visible', 'on')


%%
%
% Ahora el cambio de signo se produce en el semiintervalo izquierdo, por lo
% que movemos el extremo derecho a c = 1.5

b = c;
set(txc, 'Visible', 'off')
set(txb, 'Position', [c, -0.5*sign(fc)])


%%
%
% Bisectamos ahora el intervalo [1, 1.5] viendo que el polinomio cambia de
% signo entre c = 1.25 y b = 1.5.

c = (a+b)/2
fc = polyval(p,c)
plot(c, 0, 'g*', [c c], [0 fc], 'r:')
set(txc, 'Position', [c, -0.5*sign(fc)], 'Visible', 'on')


%%
%
% Elegimos este intervalo y continuamos el proceso hasta que la longitud
% del intervalo [a, b] sea tan pequea como queramos.

a = c;
set(txc, 'Visible', 'off')
set(txa, 'Position', [c, -0.5*sign(fc)])


%%
%
% FIN del Problema 2.1
