%% 2.2. Mtodo de Regula-Falsi 
% 
% Este problema muestra paso a paso el funcionamiento del mtodo
% de Regula-Falsi para calcular una raz de la ecuacin exp(-x^2) = cos(x).
%
%
% Tema 2, Problema 2
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson

%% 
%
% Representamos cada miembro de la ecuacin mediante curvas distintas.
% Observamos que se cortan en varios puntos.

x = linspace(-10,10);
y1 = exp(-x.^2);
y2 = cos(x);
plot(x, y1, x, y2)
title('Interseccin de dos curvas')
legend('exp(-x^2)','cos x')

%%
%
% Transformamos la ecuacin a forma normal f(x)=0.
% Representamos grficamente la funcin y = f(x).
% Las races son los cortes de la grfica con el eje OX.

f = inline('exp(-x.^2)-cos(x)');
y = feval(f,x);
plot(x, y, [-10 10], [0 0], 'k')
title('y = exp(-x^2) - cos(x)')


%%
%
% Nos acercamos un poco para localizar la primera solucin positiva.
% En este intervalo hay tres soluciones. La buscada est entre 1 y 2.

axis([-3 3 -0.2 1])


%%
%
% El intervalo [a, b] = [1, 2] contiene la raz pedida y la funcin toma
% distinto signo en sus extremos.

a = 1; fa = feval(f,a)
b = 2; fb = feval(f,b)
x = linspace(a,b);
y = feval(f,x);
plot(x, y, [a-0.1 b+0.1], [0 0], 'k', [a b], [0 0], 'g*', [a a], [0 fa], 'r:', [b b], [0 fb], 'r:')
title('y = exp(-x^2) - cos(x)')
txa = text(a, 0.03, 'a');
txb = text(b, -0.03, 'b');
%%
%
% Dibujamos el segmento que une los puntos (a, f(a)) y (b, f(b)).
% Determinamos el punto c de interseccin de este segmento con el eje OX.

c = (a*fb - b*fa)/(fb - fa)
hold on 
plot([a b], [fa fb], 'm:')
plot(c, 0, 'g*')
txc = text(c, 0.03, 'c');
title('Mtodo de Regula-Falsi')

%%
%
% Evaluamos la funcin en c. Comprobamos que el cambio de signo tiene lugar
% en el semiintervalo [c, b] = [1.28, 2]. 

fc = feval(f,c)
plot([c c], [0 fc], 'r:')

%%
%
% Elegimos el seemiintervalo derecho, dando al extremo
% izquierdo a el valor c = 1.28, es decir, desplazamos el extremo izquierdo
% del intervalo al punto de corte del segmento con el eje OX.
% Actualizamos el valor de f(a) para determinar el siguiente corte.
a = c;
fa = fc;
set(txc,'Visible','off')
set(txa,'Position',[c, 0.03])

%%
%
% Repetimos el proceso con el intervalo [1.28, 2].
% Determinamos el punto de corte del segmento por (a, f(a)) y (b, f(b)) con
% el eje OX.

c = (a*fb - b*fa)/(fb - fa)
plot([a b], [fa fb], 'm:')
plot(c, 0, 'g*')
set(txc,'Position',[c, 0.03],'Visible','on')

%%
%
% Evaluamos la funcin en c. 
fc = feval(f,c)
plot([c c], [0 fc], 'r:')

%%
%
% El cambio de signo se produce de nuevo en el semiintervalo derecho, por lo que 
% movemos el extremo izquierdo a c = 1.44.

a = c;
fa = fc;
set(txc,'Visible','off')
set(txa,'Position',[c, 0.03])

%%
% Repetimos ahora con el intervalo [1.44, 2].

c = (a*fb - b*fa)/(fb - fa)
fc = feval(f,c)
plot([a b], [fa fb], 'm:')
plot(c, 0, 'g*')
set(txc,'Position',[c, 0.03],'Visible','on')
plot([c c], [0 fc], 'r:')


%% 
%
% Una caracterstica tpica de regula-falsi es que el cambio de signo se
% produce siempre al mismo lado, con lo que un extremo del intervalo queda
% fijo y la longitud del intervalo no tiende a 0.


a = c;
fa = fc;
set(txc,'Visible','off')
set(txa,'Position',[c, 0.03])

%%
%
% El proceso se detiene cuando la diferencia entre dos estimaciones
% consecutivas de c es tan pequea como se quiere, aunque el intervalo que
% contiene la raz, [a, b], no sea pequeo.


%%
%
% FIN del Problema 2.2
