%% 2.3. Mtodo de Punto Fijo 
% 
% Este problema muestra la relacin entre la convergencia del mtodo y el
% valor en el punto fijo de la derivada de la funcin de iteracin.
%
%
% Tema 2, Problema 3
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson

%% 
%
% Representamos la funcin g1(x) = 2cos(x) junto a la bisectriz del primer
% cuadrante para obtener la estimacin inicial.

g1 = inline('2*cos(x)');
xg = linspace(-3,3);
yg = g1(xg);
plot(xg,yg,xg,xg)
axis equal
axis([-3 3 -3 3])
title('x = 2*cos(x)')


%%
%
% La interseccin de las dos curvas es un punto fijo. Como los valores de
% g1 estn acotados entre -2 y 2, no hay otras intersecciones fuera del
% intervalo representado, por lo que el punto fijo es nico.


%%
%
% Centramos la grfica en torno a la posicin aproximada el punto fijo.
% Comprobaremos que las iteraciones se alejan del punto fijo, en lugar de
% acercarse.

xg = linspace(-0.5,2.5);
yg = g1(xg);
plot(xg,yg,xg,xg)
axis equal
axis([-0.5 2.5 -0.5 2.5])
hold on
plot([-0.5 2.5], [0 0], 'k', [0 0], [-0.5 2.5],'k')
title('x = 2*cos(x)')


%%
% En efecto, tomamos como estimacin inicial un valor prximo a la
% interseccin, x1=1, e iteramos para hallar x2:

x(1) = 1;
x(2) = g1(x(1))
plot(x(1), 0, 'g*'), text(x(1), -0.1, 'x_1')
plot([x(1) x(1)], [x(1) x(2)], 'r')
plot([x(1) x(1)], [0 x(1)], 'r:')
title('Punto fijo')


%%
%
% Para representar las iteraciones, llevamos la ordenada x2
% horizontalmente hasta la bisectriz para convertirla en abscisa, a la que
% aplicaremos la funcin de iteracin

plot(x(2), 0, 'g*')
text(x(2), 0.1, 'x_2')
plot([x(1) x(2)], [x(2) x(2)], 'r')
plot([x(2) x(2)], [0 x(2)], 'r:')


%%
%
% La siguiente iteracin se efectua aplicando g1 a x2, que es el valor
% obtenido en la iteracin anterior. Grficamente, llevamos x2
% verticalmente hasta la curva que representa g1. El resultado obtenido es
% x3, que se obtiene grficamente llevando horizontalmente el punto de la
% curva a la bisectriz.

x(3) = g1(x(2))
plot([x(2) x(2) x(3)], [x(2) x(3) x(3)], 'r')
plot([x(3) x(3)], [0 x(3)], 'r:')
plot(x(3), 0, 'g*')
text(x(3), 0.1, 'x_3')


%%
%
% Iteramos otra vez: 

x(4) = g1(x(3))
plot([x(3) x(3) x(4)], [x(3) x(4) x(4)], 'r')
plot([x(4) x(4)], [0 x(4)], 'r:')
plot(x(4), 0, 'g*')
text(x(4), -0.1, 'x_4')

%%
%
% Y otra vez: 

x(5) = g1(x(4))
plot([x(4) x(4) x(5)], [x(4) x(5) x(5)], 'r')
plot([x(5) x(5)], [0 x(5)], 'r:')
plot(x(5), 0, 'g*')
text(x(5), -0.1, 'x_5')

%%
%
% Una vez ms: 

x(6) = g1(x(5))
plot([x(5) x(5) x(6)], [x(5) x(6) x(6)], 'r')
plot([x(6) x(6)], [0 x(6)], 'r:')
plot(x(6), 0, 'g*')
text(x(6), 0.1, 'x_6')

%%
%
% Observamos que las iteraciones divergen. La sucesin de iterados oscila
% alejndose cada vez ms del punto fijo. El motivo es que la derivada de
% g1 cerca del punto fijo es de mdulo mayor que 1.



%%
%
% Buscamos otra funcin para resolver el problema:
%               g2(x) = arc cos(x/2)
% Como antes, la representamos junto a la bisectriz del primer cuadrante en
% un entorno del punto fijo.

hold off
g2 = inline('acos(x/2)');
xg = linspace(0,2);
yg = g2(xg);
plot(xg,yg,xg,xg)
axis equal
axis([-0.1 2.1 -0.1 2.1])
title('x = arccos(x/2)')
hold on


%%
%
% Como g2 es la funcin inversa de g2, su derivada en el punto fijo tiene
% valor absoluto menor que 1. Las iteraciones de punto fijo van a
% converger, aunque partamos de un punto ms alejado.

plot([-0.5 2.5], [0 0], 'k', [0 0], [-0.5 2.5],'k')
x = x(end)
plot(x(1), 0, 'g*'), text(x(1), -0.1, 'x_1')

%%
%
% Iteramos como antes para hallar x2 = g2(x1). Llevamos la ordenada x2 al
% eje de abscisas para continuar iterando.

x(2) = g2(x(1))
plot([x(1) x(1)], [0 x(2)], 'r:')
plot(x(2), 0, 'g*')
text(x(2), 0.1, 'x_2')
plot([x(1) x(2)], [x(2) x(2)], 'r')
plot([x(2) x(2)], [0 x(2)], 'r:')


%%
%
% Iteramos varias veces.

x(3) = g2(x(2))
plot([x(2) x(2) x(3)], [x(2) x(3) x(3)], 'r')
plot([x(3) x(3)], [0 x(3)], 'r:')
plot(x(3), 0, 'g*')
text(x(3), 0.1, 'x_3')


%%
%
% Iteramos otra vez: 

x(4) = g2(x(3))
plot([x(3) x(3) x(4)], [x(3) x(4) x(4)], 'r')
plot([x(4) x(4)], [0 x(4)], 'r:')
plot(x(4), 0, 'g*')
text(x(4), -0.1, 'x_4')

%%
%
% Y otra vez: 

x(5) = g2(x(4))
plot([x(4) x(4) x(5)], [x(4) x(5) x(5)], 'r')
plot([x(5) x(5)], [0 x(5)], 'r:')
plot(x(5), 0, 'g*')
text(x(5), -0.1, 'x_5')

%%
%
% Una vez ms: 

x(6) = g2(x(5))
plot([x(5) x(5) x(6)], [x(5) x(6) x(6)], 'r')
plot([x(6) x(6)], [0 x(6)], 'r:')
plot(x(6), 0, 'g*')
text(x(6), 0.1, 'x_6')

%%
%
% Observamos que las iteraciones convergen. La sucesin de iterados oscila
% acerndose cada vez ms del punto fijo. El motivo es que la derivada de
% g2 cerca del punto fijo es de mdulo menor que 1.

axis([0.5 1.5 0.5 1.5])
axis square


%%
%
% Iterando unas cuantas veces ms, se obtiene el punto fijo con la
% precisin deseada.

for k = 6:20
    x(k+1) = g2(x(k));
    plot([x(k) x(k) x(k+1)], [x(k) x(k+1) x(k+1)], 'r')
end
punto_fijo = x(end)

%%
%
% El comportamiento de  las iteraciones de punto fijo no es siempre tan
% sencillo como en los ejemplos presentados. Observa, por ejemplo, lo que
% ocurre al realizar ms iteraciones con la funcin g1.
%

%%
%
% FIN del Problema 2.3