%% 2.4. Mtodo de Newton 
% 
% Este problema muestra grficamente el funcionamiento del mtodo de Newton
% y comprueba que converge cuadrticamente calculando la raz quinta de 2. 
%
%
% Tema 2, Problema 4
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson

%% 
%
% Representamos la funcin y = x^5 -2 en el intervalo de 1 a 1.5.
% Elegimos este intervalo porque 1^5 < 2 < 1.5^5 y el mtodo de Newton
% necesita una estimacin inicial cercana a la solucin.


xg = linspace(1,1.5);
yg = raiz5(xg);
plot(xg,yg)
title('y = x^5 - 2')
grid on


%%
%
% El mtodo de Newton utiliza la funcin y su derivada, que calculamos en
% el fichero raiz5.m

type raiz5

%%
%
% La raz buscada es el corte de la curva con el eje OX.
% Como estimacin inicial tomamos x1 = 1.

x = 1
grid off, hold on
axis([0.99 1.25 -1 1])
plot(x(1), 0, 'g*'), text(x(1), 0.05, 'x_1')
plot([0.99 1.25], [0 0], 'k')


%%
%
% El mtodo de Newton toma la estimacin inicial de la solucin x1, traza
% la tangente a la curva por (x1,f(x1)) y la intersecta con el eje OX. Esta
% interseccin es la nueva estimacin x2 de la solucin.

[y(1),dy] = raiz5(x(1));
x(2) = x(1) - y(1)/dy
plot([x(1) x(1) x(2)], [0 y(1) 0], 'r:')
plot(x(2), 0, 'g*'), text(x(2), -0.05, 'x_2')
title('Mtodo de Newton')


%%
%
% En el siguiente paso se toma x2 como estimacin actual y se vuelve a
% aproximar localmente la funcin por la tangente. La interseccin de la
% tangente con OX es la nueva estimacin x3.

[y(2),dy] = raiz5(x(2));
x(3) = x(2) - y(2)/dy
plot([x(2) x(2) x(3)], [0 y(2) 0], 'r:')
plot(x(3), 0, 'g*'), text(x(3), -0.05, 'x_3')


%%
%
% Anlogamente, obtenemos x4:

[y(3),dy] = raiz5(x(3));
x(4) = x(3) - y(3)/dy
plot([x(3) x(3) x(4)], [0 y(3) 0], 'r:')
plot(x(4), 0, 'g*'), text(x(4), 0.05, 'x_4')


%%
%
% Las siguientes iteraciones son difciles de ver en esta grfica, debido a
% la rapidez de convergencia del mtodo de Newton.

[y(4),dy] = raiz5(x(4));
x(5) = x(4) - y(4)/dy;
[y(5),dy] = raiz5(x(5));
x(6) = x(5) - y(5)/dy

%%
%
% Para medir la velocidad de convergencia de las iteraciones de Newton
% calculamos los cocientes de las diferencias de los iterados en valor
% absoluto.

e = abs(diff(x))
tcl = e(2:end)./e(1:end-1) % Tasa de convergencia lineal


%%
%
% No slo las diferencias tienden a cero, sino tambin los cocientes de
% diferencias sucesivas (lo que llamamos tasa de convergencia lineal). 

hold off
plot(tcl)
title('Tasa de convergencia lineal')
grid

%%
%
% Medimos la velocidad de convergencia con unos cocientes ms exigentes:
% dividimos el incremento en cada paso por el cuadrado del incremento
% anterior. Estos cocientes se estabilizan, por lo que decimos que hay
% convergencia cuadrtica. El valor lmite se denomina tasa de convergencia
% cuadrtica.

tcc = e(2:end)./e(1:end-1).^2 % Tasa de convergencia cuadrtica
plot(tcc)
title('Tasa de convergencia cuadrtica')
grid

%%
%
% El mtodo de Newton presenta convergencia cuadrtica si converge a una
% raz donde la derivada no se anula. Un ejemplo en el que esto no se
% cumple es la ecuacin x^5 = 0.


%%
%
% FIN del Problema 2.4