%% 2.5.a) Problema del billar circular. Mtodo de biseccin. 
% 
% Este problema aplica el mtodo de biseccin, para determinar la direccin
% en la que hay que golpear una bola en una mesa de billar circular para
% hacer impacto en otra despus de rebotar en la pared.
%
%
% Tema 2, Problema 5, Apartado a)
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson

%% 
%
% La figura muestra la trayectoria de la bola que se lanza desde el punto
% P, choca con la pared de la mesa circular en I y rebota hasta alcanzar el
% punto Q. 
% Para resolver el problema matemticamente, tratamos de determinar el
% ngulo t que define la trayectoria de la bola.

billar


%%
% Para que se produzca el impacto, el ngulo t debe verificar la ecuacin
%
%   (xP*sen(t) - yP*cos(t))/sqrt((R*cos(t) - xP)^2 + (R*sen(t) - yP)^2) +
%   (xQ*sen(t) - yQ*cos(t))/sqrt((R*cos(t) - xQ)^2 + (R*sen(t) - yQ)^2) = 0
%
% donde P = (xP, yP) y Q = (xQ, yQ) y R es el radio de la mesa.
% 
% La funcin fbillar calcula los valores del primer miembro en funcin de
% t, para las posiciones P = (0.6, 0.3) y Q = (-0.6, -0.3).

type fbillar


%%
%
% Representamos la funcin fbillar en el intervalo [0, 2*pi]. Se observa
% que que existe una solucin en cada uno de los siguientes intervalos
% [1, 2], [3, 4], [4, 5] y [5, 6].

t = linspace(0,2*pi);
y = fbillar(t);
area(t,y,'FaceColor','y')
xlabel('ngulo')
title('Funcin fbillar')
grid

%%
%
% En primer lugar, determinamos por el mtodo de biseccin la raz del
% intervalo [1, 2].

[sol, x, incr, iter] = biseccion('fbillar', 1, 2, 1e-4, 100)
plot(x,'.-')
xlabel('Iteracin'), ylabel('Valor del iterado')
title('Mtodo de biseccin en [1, 2]')
grid


%%
%
% Sin ser expertos en el juego, observamos en la figura anterior que una
% solucin exacta de la ecuacin es t = pi/2 = 90, por lo que podemos
% calcular el error cometido en nuestra aproximacin:

error = abs(sol-pi/2)


%%
%
% Si consideramos los puntos medios obtenidos en cada biseccin como
% sucesivas aproximaciones de la solucin, observamos que stas no se
% acercan uniformemente al resultado final. Por ejemplo, el primer punto
% medio obtenido, 1.5, est ms cerca de la solucin que el siguiente. Esto
% es una carcterstica tpica del mtodo de biseccin.



%%
%
% La solucin en el intervalo [3, 4] se obtiene anlogamente. Ntese que el
% nmero de pasos requeridos es el mismo, puesto que partimos de un
% intervalo de la misma amplitud y requerimos la misma precisin.

[sol2, x2, incr, iter] = biseccion('fbillar', 3, 4, 1e-4, 100)
plot(x2,'g.-')
xlabel('Iteracin'), ylabel('Valor del iterado')
title('Mtodo de biseccin en [3, 4]')
grid


%%
%
% Obtenemos las races en los otros intervalos y mostramos la evolucin de
% las iteraciones en cada intervalo.

[sol(3), x3, incr, iter] = biseccion('fbillar', 4, 5, 1e-4, 100);
[sol(4), x4, incr, iter] = biseccion('fbillar', 5, 6, 1e-4, 100);

plot(x, 'b.-'), hold on
plot(x2,'g.-'), plot(x3,'r.-')
plot(x4,'y.-'), hold off
xlabel('Iteraciones'), ylabel('Valor de los iterados')
grid


%%
%
% Los ngulos obtenidos tienen simetra en la circunferencia.

sol(2) = sol2;
plot(cos(t), sin(t), 'k', cos(sol), sin(sol), 'm*')
P = [0.6 -0.3]; Q = [-0.6 -0.3]; S = [cos(sol)' sin(sol)'];
hold on
plot([P(1) Q(1)],[P(2) Q(2)],'m.','MarkerSize',20)
plot([P(1) S(1,1) Q(1)], [P(2) S(1,2) Q(2)], 'b:')
plot([P(1) S(2,1) Q(1)], [P(2) S(2,2) Q(2)], 'g:')
plot([P(1) S(3,1) Q(1)], [P(2) S(3,2) Q(2)], 'r:')
plot([P(1) S(4,1) Q(1)], [P(2) S(4,2) Q(2)], 'y:')
title('Rebotes para carambola')
axis([-1.05 1.05 -1.05 1.05]), axis square 
set(gca, 'XTickLabel', '', 'YTickLabel', '', ...
    'XColor', 'w', 'YColor', 'w', 'ZColor', 'w')
hold off


%%
%
% FIN del Problema 2.5.a)