%% 2.5.b) Problema del billar circular. Mtodo de regula-falsi. 
% 
% Este problema aplica el mtodo de regula-falsi, para determinar la direccin
% en la que hay que golpear una bola en una mesa de billar circular para
% hacer impacto en otra despus de rebotar en la pared.
%
%
% Tema 2, Problema 5, Apartado b)
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson

%% 
%
% La figura muestra la trayectoria de la bola que se lanza desde el punto
% P, choca con la pared de la mesa circular en I y rebota hasta alcanzar el
% punto Q. 
% Para resolver el problema matemticamente, tratamos de determinar el
% ngulo t que define la trayectoria de la bola.

billar


%%
% Para que se produzca el impacto, el ngulo t debe verificar la ecuacin
%
%   (xP*sen(t) - yP*cos(t))/sqrt((R*cos(t) - xP)^2 + (R*sen(t) - yP)^2) +
%   (xQ*sen(t) - yQ*cos(t))/sqrt((R*cos(t) - xQ)^2 + (R*sen(t) - yQ)^2) = 0
%
% donde P = (xP, yP) y Q = (xQ, yQ) y R es el radio de la mesa.
% 
% La funcin fbillar calcula los valores del primer miembro en funcin de
% t, para las posiciones P = (0.6, 0.3) y Q = (-0.6, -0.3).

type fbillar


%%
%
% Representamos la funcin fbillar en el intervalo [0, 2*pi]. Se observa
% que que existe una solucin en cada uno de los siguientes intervalos
% [1, 2], [3, 4], [4, 5] y [5, 6].

t = linspace(0,2*pi);
y = fbillar(t);
area(t,y,'FaceColor','y')
xlabel('ngulo')
title('Funcin fbillar')
grid


%%
%
% En primer lugar, determinamos por el mtodo de regula-falsi la raz del
% intervalo [1, 2].

[sol, x, incr, iter] = regulafalsi('fbillar', 1, 2, 1e-12, 50)
plot(x,'.-')
title('Regula-falsi partiendo de [1, 2]')
xlabel('Iteracin'), ylabel('Valor del iterado')
grid


%%
%
% A pesar de exigir tolerancia 10^(-12), el resultado se alcanza en muy
% pocas iteraciones. 
% Hallando las diferencias entre iterados consecutivos vemos que, en este
% caso, la convergencia no es montona como suele ocurrir en regula-falsi,
% pues los signos van alternando.
% Aqu regula-falsi converge ms rpidamente de lo esperado.

diff(x)

%%
%
% La tasa de convergencia lineal tiende a 0, por lo que la convergencia
% lineal esperada del mtodo de regula falsi se convierte en este caso en
% convergencia superlineal. 

e = abs(diff(x));
format long
r = e(2:end)./e(1:end-1)
format short
plot(r,'.-')
title('Tasa de convergencia lineal')
xlabel('Iteracin')
set(gca, 'XTick', 1:4)
grid


%%
%
% En general, el mtodo de regula-falsi converge linealmente. Este es el
% caso de la raz situada en el intervalo [3, 4]. 

[sol2, x2, incr, iter] = regulafalsi('fbillar', 3, 4, 1e-12, 50)
plot(x2, 'g.-')
title('Regula-falsi partiendo de [3, 4]')
xlabel('Iteracin'), ylabel('Valor del iterado')
grid


%%
%
% La tasa de convergencia lineal obtenida es 0.088, notablemente inferior a
% 1, lo que proporciona una convergenica muy rpida, dentro de ser de tipo
% lineal.
% Observamos que, tras el paso inicial, los incrementos entre los iterados
% siempre tienen el mismo signo.

diff(x2)
e = abs(diff(x2));
r = e(2:end)./e(1:end-1)
plot(r, 'g.-')
title('Tasa de convergencia lineal')
xlabel('Iteracin')
grid


%%
%
% El gfico compara la tendencia a cero de las diferencias de los iterados
% de regula-falsi en los dos intervalos anteriores.

plot(diff(x(2:end)), '.-')
hold on
plot(diff(x2(2:end)), 'g.-')
title('Incremento entre los iterados de regula-falsi ')
legend('Intervalo [1, 2]', 'Intervalo [3, 4]')
xlabel('Iteracin'), ylabel('Incremento')
grid
hold off


%%
%
% En el itervalo [4, 5], regula-falsi se comporta como en [1, 2].

[sol3, x3, incr, iter] = regulafalsi('fbillar', 4, 5, 1e-12, 50)
plot(x3, 'r.-')
title('Regula-falsi partiendo de [4, 5]')
xlabel('Iteracin'), ylabel('Valor del iterado')
grid


%%
%
% Las diferencias cambian de signo y la convergencia es superlineal.

diff(x3)
e = abs(diff(x3));
format long
r = e(2:end)./e(1:end-1)
format short
plot(r, 'r.-')
title('Tasa de convergencia lineal')
xlabel('Iteracin')
set(gca, 'XTick', 1:5)
grid


%%
%
% En el intervalo [5, 6], regula-falsi es muy lento, debido a que la
% funcin no se parece a una recta en este intervalo. 

[sol4, x4, incr, iter] = regulafalsi('fbillar', 5, 6, 1e-12, 50)
plot(x4, 'y.-')
title('Regula-falsi partiendo de [4, 5]')
xlabel('Iteracin'), ylabel('Valor del iterado')
grid


%%
%
% Presenta convergencia lineal, pero la tasa es cercana a 0.5, que es
% la tasa del mtodo de biseccin.

diff(x4)
e = abs(diff(x4));
r = e(2:end)./e(1:end-1)
plot(r, 'y.-')
title('Tasa de convergencia lineal')
xlabel('Iteracin')
grid


%%
%
% En el grfico se aprecian los incrementos de los iterados en los cuatro
% intervalos considarados. En los cuatro casos hay convergencia, pero el
% nmero de iteraciones y la velocidad de convergencia es diferente en cada
% caso.

plot(diff(x(2:end)), '.-')
title('Incremento entre los iterados de regula-falsi ')
hold on
plot(diff(x2(2:end)), 'g.-')
plot(diff(x3(2:end)), 'r.-')
plot(diff(x4(2:end)), 'y.-')
legend('Intervalo [1, 2]', 'Intervalo [3, 4]', 'Intervalo [4, 5]', 'Intervalo [5, 6]')
xlabel('Iteraciones'), ylabel('Incrementos')
grid
hold off


%%
%
% Los ngulos obtenidos tienen simetra en la circunferencia.

sol = [sol, sol2, sol3, sol4];
plot(cos(t), sin(t), 'k', cos(sol), sin(sol), 'm*')
P = [0.6 -0.3]; Q = [-0.6 -0.3]; S = [cos(sol)' sin(sol)'];
hold on
plot([P(1) Q(1)],[P(2) Q(2)],'m.','MarkerSize',20)
plot([P(1) S(1,1) Q(1)], [P(2) S(1,2) Q(2)], 'b:')
plot([P(1) S(2,1) Q(1)], [P(2) S(2,2) Q(2)], 'g:')
plot([P(1) S(3,1) Q(1)], [P(2) S(3,2) Q(2)], 'r:')
plot([P(1) S(4,1) Q(1)], [P(2) S(4,2) Q(2)], 'y:')
title('Rebotes para carambola')
axis([-1.05 1.05 -1.05 1.05]), axis square 
set(gca, 'XTickLabel', '', 'YTickLabel', '', ...
    'XColor', 'w', 'YColor', 'w', 'ZColor', 'w')
hold off


%%
%
% FIN del Problema 2.5.b)