%% 2.5.c) Problema del billar circular. Mtodo de Newton
% 
% Este problema utiliza el mtodo de Newton en la determinacin de la
% direccin en la que hay que golpear una bola en una mesa de billar
% circular para hacer impacto en otra despus de rebotar en la pared.
%
%
% Tema 2, Problema 5 c)
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson

%% 
%
% La figura muestra la trayectoria de la bola que se lanza desde el punto
% P, choca con la pared de la mesa circular en I y rebota hasta alcanzar el
% punto Q. 
% Para resolver el problema matemticamente, tratamos de determinar el
% ngulo t que define la trayectoria de la bola.

billar


%%
% Para que se produzca el impacto, el ngulo t debe verificar la ecuacin
%
%   (xP*sen(t) - yP*cos(t))/sqrt((R*cos(t) - xP)^2 + (R*sen(t) - yP)^2) +
%   (xQ*sen(t) - yQ*cos(t))/sqrt((R*cos(t) - xQ)^2 + (R*sen(t) - yQ)^2) = 0
%
% donde P = (xP, yP) y Q = (xQ, yQ) y R es el radio de la mesa.


%%
%
% Para resolver el problema mediante el mtodo de Newton, hemos de
% calcular la derivada, que incluimos junto con la funcin en nwbillar.

type nwbillar
t = linspace(0,2*pi);
[y, dy] = nwbillar(t);
plot(t,y,t,dy)
xlabel('ngulo')
title('Funcin nwbillar')
legend('Funcin', 'Derivada')
grid

%%
%
% En la grfica se observa que que existe una solucin en cada uno de los
% siguientes intervalos [1, 2], [3, 4], [4, 5] y [5, 6].


plot(t, y, [0 7], [0 0], 'k')
xlabel('ngulo')
title('Funcin nwbillar')
grid


%%
%
% En primer lugar, iteramos por el mtodo de Newton, partiendo de la
% estimacin inicial x1 = 1. Exigimos mucha precisin para obtener
% suficientes iterados, ya que Newton converge muy rpidamente.

[sol, x, incr, iter] = newton('nwbillar', 1, 1e-12, 50)
plot(x,'.-')
xlabel('Iteracin'), ylabel('Valor del iterado')
title('Mtodo de Newton con x_1 = 1')
set(gca, 'XTick', 1:5)
grid


%%
%
% La tasa de convergencia cuadrtica tiende a 0, por lo que la convergencia
% en este ejemplo es de mayor orden. Esto se debe a que la raz es un punto
% de inflexin.

e = abs(diff(x));
format long
r2 = e(2:end)./e(1:end-1).^2
format short
plot(r2,'.-')
title('Tasa de convergencia cuadrtica')
xlabel('Iteracin')
set(gca, 'XTick', 1:3)
grid


%%
%
% Observamos que los cocientes de r2 decrecen, mientras que los dos
% primeros valores de r3 son practicamente iguales. El cociente finalmente
% sube debido al error de redondeo.

format long
r3 = e(2:end)./e(1:end-1).^3
format short
plot(r3,'.-')
title('Tasa de convergencia cbica')
xlabel('Iteracin')
set(gca, 'XTick', 1:3)
grid


%%
%
% En general, el mtodo de Newton converge cuadrticamente. As ocurre con
% las iteraciones que convergen a la raz entre 3 y 4.
% Sin embargo, hay que partir de una estimacin inicial de la raz
% suficientemente buena, pues de lo contrario, nos podemos encontrar con
% sorpresas, como en este caso en que la raz hallada est muy lejos de la
% buscada.

[sol2, x2, incr, iter] = newton('nwbillar', 3, 1e-12, 50)
plot(x2, 'g.-')
title('Mtodo de Newton con x_1 = 3')
xlabel('Iteracin'), ylabel('Valor del iterado')
set(gca, 'XTick', 1:6)
grid


%%
%
% Probamos con x1 = 4. Ahora s se localiza la raz entre 3 y 4.

[sol2, x2, incr, iter] = newton('nwbillar', 4, 1e-12, 50)
plot(x2, 'g.-')
title('Mtodo de Newton con x_1 = 4')
xlabel('Iteracin'), ylabel('Valor del iterado')
grid


%%
%
% La convergencia es cuadrtica, como se deduce de la estabilizacin de los
% valores de la tasa correspondiente.

e = abs(diff(x2));
r2 = e(2:end)./e(1:end-1).^2
plot(r2, 'g.-')
title('Tasa de convergencia cuadrtica')
xlabel('Iteracin')
set(gca, 'XTick', 1:5)
grid


%%
%
% La tercera solucin se obtiene por Newton partiendo de x1 = 5.

[sol3, x3, incr, iter] = newton('nwbillar', 5, 1e-12, 50)
plot(x3, 'r.-')
title('Mtodo de Newton con x_1 = 5')
xlabel('Iteracin'), ylabel('Valor del iterado')
grid


%%
%
% La tasa de convergencia cuadrtica tiende a 0, como en el primer caso.
% Los primeros cocientes de la tasa cbica se mantienen relativamente
% estables.

e = abs(diff(x3));
format long
r2 = e(2:end)./e(1:end-1).^2
r3 = e(2:end)./e(1:end-1).^3
format short
plot(r3(1:end-1),'.-')
title('Tasa de convergencia cbica')
xlabel('Iteracin')
set(gca, 'XTick', 1:3)
grid


%%
%
% Aproximamos la ltima raz partiendo de x1 = 6.

[sol4, x4, incr, iter] = newton('nwbillar', 6, 1e-12, 50)
plot(x4, 'y.-')
title('Mtodo de Newton con x_1 = 6')
xlabel('Iteracin'), ylabel('Valor del iterado')
grid


%%
%
% Presenta convergencia cuadrtica, a pesar de que el ltimo cociente se
% dispara. 

e = abs(diff(x4));
r2 = e(2:end)./e(1:end-1).^2
plot(r2, 'y.-')
title('Tasa de convergencia cuadrtica')
xlabel('Iteracin')
grid


%%
%
% En el grfico se muestran iterados de Newton que convergen a cada una de
% las soluciones del problema del billar.

plot(x, '.-')
title('Iterados de Newton para el problema del billar')
hold on
plot(x2, 'g.-')
plot(x3, 'r.-')
plot(x4, 'y.-')
xlabel('Iteraciones')
hold off
grid


%%
%
% Los ngulos obtenidos tienen simetra en la circunferencia.

sol = [sol, sol2, sol3, sol4];
plot(cos(t), sin(t), 'k', cos(sol), sin(sol), 'm*')
P = [0.6 -0.3]; Q = [-0.6 -0.3]; S = [cos(sol)' sin(sol)'];
hold on
plot([P(1) Q(1)],[P(2) Q(2)],'m.','MarkerSize',20)
plot([P(1) S(1,1) Q(1)], [P(2) S(1,2) Q(2)], 'b:')
plot([P(1) S(2,1) Q(1)], [P(2) S(2,2) Q(2)], 'g:')
plot([P(1) S(3,1) Q(1)], [P(2) S(3,2) Q(2)], 'r:')
plot([P(1) S(4,1) Q(1)], [P(2) S(4,2) Q(2)], 'y:')
title('Rebotes para carambola')
axis([-1.05 1.05 -1.05 1.05]), axis square 
set(gca, 'XTickLabel', '', 'YTickLabel', '', ...
    'XColor', 'w', 'YColor', 'w', 'ZColor', 'w')
hold off



%%
%
% FIN del Problema 2.5.c)