%% 2.5. Problema del billar circular. Mtodo de la secante
% 
% Este problema utiliza el mtodo de la secante para determinar la
% direccin en la que hay que golpear una bola en una mesa de billar
% circular para hacer impacto en otra despus de rebotar en la pared.
% En el primera iteracin se muestran detaladamente los pasos del mtodo y
% su interpretacin grfica.
%
%
% Tema 2, Problema 5 d)
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson


%% 
%
% La figura muestra la trayectoria de la bola que se lanza desde el punto
% P, choca con la pared de la mesa circular en I y rebota hasta alcanzar el
% punto Q. 
% Para resolver el problema matemticamente, tratamos de determinar el
% ngulo t que define la trayectoria de la bola.

billar


%%
% Para que se produzca el impacto, el ngulo t debe verificar la ecuacin
%
%   (xP*sen(t) - yP*cos(t))/sqrt((R*cos(t) - xP)^2 + (R*sen(t) - yP)^2) +
%   (xQ*sen(t) - yQ*cos(t))/sqrt((R*cos(t) - xQ)^2 + (R*sen(t) - yQ)^2) = 0
%
% donde P = (xP, yP) y Q = (xQ, yQ) y R es el radio de la mesa.
% 
% La funcin fbillar calcula los valores del primer miembro en funcin de
% t, para las posiciones P = (0.6, 0.3) y Q = (-0.6, -0.3).

type fbillar


%%
%
% Representamos la funcin fbillar en el intervalo [0, 2*pi]. Se observa
% que que existe una solucin en cada uno de los siguientes intervalos
% [1, 2], [3, 4], [4, 5] y [5, 6].

t = linspace(0,2*pi);
y = fbillar(t);
area(t,y,'FaceColor','y')
xlabel('ngulo')
title('Funcin fbillar')
grid


%%
%
% Realizaremos detenidamente los pasos del mtodo de la secante para hallar
% la raz situada entre 3 y 4. 
% El mtodo necesita dos estimaciones iniciales. Tomamos x1 = 3 y x2 = 4
% como aproximaciones de la raz comprendida entre ambas, aunque sto no es
% necesario; las dos estimaciones podran estar al mismo lado de la solucin.

x(1) = 3; y(1) = fbillar(x(1));
x(2) = 4; y(2) = fbillar(x(2));
u = linspace(x(1), x(2));
v = fbillar(u);
plot(u, v, [x(1)-0.1 x(2)+0.1], [0 0], 'k', [x(1) x(2)], [0 0], 'g*', ...
    [x(1) x(1)], [0 y(1)], 'r:', [x(2) x(2)], [0 y(2)], 'r:')
title('Funcin fbillar en [3, 4]')
text(x(1), 0.03, 'x_1');
tx2 = text(x(2), -0.03, 'x_2');
hold on 


%%
%
% Dibujamos la recta que une los puntos (x1, y1) y (x2, y2).
% El punto de interseccin de esta recta con el eje OX es el iterado
% siguiente, x3.

m = (y(2)-y(1))/(x(2)-x(1));    % Pendiente de la secante
x(3) = x(2) - y(2)/m            % Corte con OX
y(3) = fbillar(x(3));
plot([x(1:3) x(3)], [y(1:2) 0 y(3)], 'm:')
plot(x(3), 0, 'g*')
tx3 = text(x(3), -0.03, 'x_3');
title('Mtodo de la secante')

%%
%
% En el siguiente paso, trazamos la recta que pasa por (x2, y2) y (x3, y3)
% y hallamos su interseccin con OX, que ser la prxima estimacin, x4.

m = (y(3)-y(2))/(x(3)-x(2));    % Pendiente de la secante
x(4) = x(3) - y(3)/m            % Corte con OX
y(4) = fbillar(x(4));
plot([x(2:4) x(4)], [y(2:3) 0 y(4)], 'm:')
plot(x(4), 0, 'g*')
tx4 = text(x(4), 0.03, 'x_4');


%%
%
% Vemos el grfico ms de cerca, para apreciar los pasos siguientes.

axis([3.82 4.02 -0.1 0.1])
set(tx2, 'Position', [x(2), -0.01])
set(tx3, 'Position', [x(3), -0.01])
set(tx4, 'Position', [x(4), 0.01])


%%
%
% Anlogamente, obtenemos x5:

m = (y(4)-y(3))/(x(4)-x(3));    % Pendiente de la secante
x(5) = x(4) - y(4)/m            % Corte con OX
y(5) = fbillar(x(5));
plot([x(3:5) x(5)], [y(3:4) 0 y(5)], 'm:')
plot(x(5), 0, 'g*')
text(x(5), -0.01, 'x_5');


%%
%
% Anlogamente, obtenemos x6:

m = (y(5)-y(4))/(x(5)-x(4));    
x(6) = x(5) - y(5)/m;     
y(6) = fbillar(x(6));
plot(x(6), 0, 'g*'), text(x(6)-0.002, -0.01, 'x_6');


%%
%
% Las siguientes iteraciones son difciles de ver en la grfica, debido a
% la rapidez de convergencia del mtodo de la secante.

m = (y(6)-y(5))/(x(6)-x(5));    
x(7) = x(6) - y(6)/m 
plot(x(7), 0, 'g*'), text(x(7),  0.01, 'x_7');
hold off


%%
%
% Estas iteraciones pueden hacerse automticamente con el programa secante.
% Para ello se ejecuta

[sol2, x2, incr, iter] = secante('fbillar', 3, 4, 1e-12, 50)
plot(x2,'g.-')
xlabel('Iteracin'), ylabel('Valor del iterado')
title('Mtodo de la secante con x_1 = 3 y x_2 = 4')
grid


%%
%
% El mtodo de la secante converge superlinealmente, es decir, la
% convergencia es ms rpida que lineal, pero menos que cuadrtica.
% Numricamente, se comprueba que la tasa de convergencia lineal tiende a
% 0, mientras que la tasa de convergencia cuadrtica se dispara.


e = abs(diff(x2));
r = e(2:end)./e(1:end-1)
plot(r,'g.-')
title('Tasa de convergencia lineal')
xlabel('Iteracin')
grid


%%
%
% El mtodo de la secante converge superlinealmente, es decir, la
% convergencia es ms rpida que lineal, pero menos que cuadrtica.
% Numricamente se comprueba que la tasa de convergencia lineal tiende a 0,
% mientras que la tasa de convergencia cuadrtica se dispara.

r2 = e(2:end)./e(1:end-1).^2
plot(r2,'g.-')
title('Tasa de convergencia cuadrtica')
xlabel('Iteracin')
grid


%%
%
% Buscamos ahora a la raz situada entre 1 y 2. Tomamos estos valores como
% iteraciones iniciales. 

[sol, x, incr, iter] = secante('fbillar', 1, 2, 1e-12, 50)
plot(x,'.-')
title('Secante partiendo de x_1 = 1 y x_2 = 2')
xlabel('Iteracin'), ylabel('Valor del iterado')
grid


%%
%
% La tasa de convergencia lineal tiende a 0, por lo que la convergencia
% es superlineal. 
% En este caso, en las primeras iteraciones, la tasa de convergencia
% cuadrtica toma valores pequeos, por lo que la aproximacin es muy
% rpida al principio.

e = abs(diff(x));
r = e(2:end)./e(1:end-1)
r2 = e(2:end)./e(1:end-1).^2;
r2(1:end-1)
plot(r,'.-')
title('Tasa de convergencia lineal')
xlabel('Iteracin')
set(gca,'XTick',1:5)
grid


%%
%
% En general, el mtodo de la secante presenta convergencia superlineal. As ocurre con
% las iteraciones que convergen a la raz entre 4 y 5.

[sol3, x3, incr, iter] = secante('fbillar',4, 5, 1e-12, 50)
plot(x3, 'r.-')
title('Mtodo de la secante con x_1 = 4 y x_2 = 5')
xlabel('Iteracin'), ylabel('Valor del iterado')
grid


%%
%
% La tasa de convergencia lineal tiende a 0.
% Los primeros cocientes de la tasa cuadrtica se mantienen relativamente
% estables. La convergencia es "casi" cuadrtica.

e = abs(diff(x3));
format long
r = e(2:end)./e(1:end-1)
r2 = e(2:end)./e(1:end-1).^2
format short
plot(r,'r.-')
title('Tasa de convergencia lineal')
xlabel('Iteracin')
grid


%%
%
% Aproximamos la ltima raz partiendo de x1 = 5 y x2 = 6.
% Hay que partir de una estimacin inicial de la raz
% suficientemente buena, pues de lo contrario, nos podemos encontrar con
% sorpresas, como en este caso en que la raz hallada no es la buscada.

[sol4, x4, incr, iter] = secante('fbillar', 5, 6, 1e-12, 50)
plot(x4, 'y.-')
title('Mtodo de la secante con x_1 = 5 y x_2 = 6')
xlabel('Iteracin'), ylabel('Valor del iterado')
grid


%%
%
% Probamos con una estiamcin inicial mejor. De este modo s que converge a la
% solucin deseada.

[sol4, x4, incr, iter] = secante('fbillar', 5, 5.5, 1e-12, 50)
plot(x4, 'y.-')
title('Mtodo de la secante con x_1 = 5 y x_2 = 6')
xlabel('Iteracin'), ylabel('Valor del iterado')
grid


%%
%
% Presenta convergencia superlineal una vez que se estabiliza.

e = abs(diff(x4));
r = e(2:end)./e(1:end-1)
plot(r, 'y.-')
title('Tasa de convergencia lineal')
xlabel('Iteracin')
grid


%%
%
% En el grfico se muestran iterados del mtodo de la secante que convergen
% a cada una de las soluciones del problema del billar.

plot(x, '.-')
title('Iterados de la secante para el problema del billar')
hold on
plot(x2, 'g.-')
plot(x3, 'r.-')
plot(x4, 'y.-')
xlabel('Iteraciones')
grid
hold off


%%
%
% Los ngulos obtenidos tienen simetra en la circunferencia.

sol = [sol, sol2, sol3, sol4];
plot(cos(t), sin(t), 'k', cos(sol), sin(sol), 'm*')
P = [0.6 -0.3]; Q = [-0.6 -0.3]; S = [cos(sol)' sin(sol)'];
hold on
plot([P(1) Q(1)],[P(2) Q(2)],'b.','MarkerSize',20)
plot([P(1) S(1,1) Q(1)], [P(2) S(1,2) Q(2)], 'b:')
plot([P(1) S(2,1) Q(1)], [P(2) S(2,2) Q(2)], 'g:')
plot([P(1) S(3,1) Q(1)], [P(2) S(3,2) Q(2)], 'r:')
plot([P(1) S(4,1) Q(1)], [P(2) S(4,2) Q(2)], 'y:')
title('Rebotes para carambola')
axis([-1.05 1.05 -1.05 1.05]), axis square 
set(gca, 'XTickLabel', '', 'YTickLabel', '', ...
    'XColor', 'w', 'YColor', 'w', 'ZColor', 'w')
hold off



%%
%
% FIN del Problema 2.5.d)