%% 2.6. La catenaria
% 
% En este problema se determina la forma de una catenaria iterando por el
% mtodo de punto fijo con diversas funciones de iteracin, y se comprueba
% la relacin de la derivada de dicha funcin con la convergencia o
% divergencia de las iteraciones.
%
%
% Tema 2, Problema 6
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson


%% 
%
% La curva representada muestra un esquema de la cadena pendiente. Se trata
% de determinar lambda en funcin de d.

x = linspace(-1,1);
y = cosh(x);
plot(x, y, [0 0 1 1], [0 1 1 cosh(1)],'r:')
axis equal, axis([-1.2 1.2 0 1.8])
title('Catenaria')
text([0.1 1.1], [0.5 1.2], str2mat('Lambda','d'))


%%
%
% Apartado a)
% Consideramos la funcin de punto fijo g1(x) = x*cosh(1/x) - d, con 
% d = 0.1. 
% Nota: x juega el papel de lambda, por abreviar la notacin.

g1 = inline('x.*cosh(1./x) - 0.1');


%%
%
% Representamos la funcin junto con la bisectriz del primer
% cuadrante y = x. La interseccin de ambas es el punto fijo buscado. 
% Como ambas curvas tienen casi la misma pendiente, la convergencia del
% mtodo de punto fijo ser muy lenta.

x = linspace(1, 6);
plot(x, g1(x), x ,x)
axis equal, grid
title('Punto fijo de x = x*cosh(1/x) - 0.1')


%%
%
% Para apreciar mejor la interseccin, representamos la diferencia entre g1
% y la bisectriz. La solucin est cerca de 5. 

plot(x, g1(x)-x, [1 6], [0 0], 'k')
title('y = x*cosh(1/x) - 0.1 - x')
grid


%%
%
% Tomando como aproximacin inicial x = 4, se necesitan 255 iteraciones
% para alcanzar el punto fijo con una tolerancia de 10^-4.
% La grfica muestra el lento progreso de las iteraciones.

[sol, x1, incr, k] = puntofijo(g1, 4, 1e-4, 500)
plot(x1,'.'), grid
title('Punto fijo de x = x*cosh(1/x) - 0.1')
xlabel('Iteracin')
ylabel('Valor del iterado')


%%
% Apartado b)
% Consideramos ahora la funcin de iteracin g2(x) = 1/acosh((x+d)/x).
% En la grfica se observa claramente el corte con la funcin y = x. El
% mtodo de punto fijo converger ms rpidamente.

g2 = inline('1./acosh((x+0.1)./x)');
plot(x, g2(x), 'r', x ,x)
axis equal, grid
title('Punto fijo de x = 1/acosh((x+d)/x)')



%%
%
% Con la misma estimacin inicial x = 4, se obtiene la solucin en 15
% iteraciones.
% La grfica muestra la evolucin de los iterados.

[sol2, x2, incr, k] = puntofijo(g2, 4, 1e-4, 500)
plot(x2, 'r.-'), grid
title('Punto fijo de x = 1/acosh((x+d)/x)')
xlabel('Iteracin')
ylabel('Valor del iterado')


%%
%
% Apartado c)
% En el mtodo del punto fijo, la tasa de convergencia lineal coincide con
% el valor absoluto de la derivada de la funcin de iteracin en la
% solucin.
% Para la funcin g1(x) = x*cosh(1/x) - 0.1  tenemos:

e = abs(diff(x1));
r = e(2:end)./e(1:end-1);
r(end)                 % Tasa de convergencia lineal

dg1 = cosh(1/sol) - sol*sinh(1/sol)/sol^2  % Derivada

plot(r,'.-'), grid
title('Tasa de convergencia de los iterados de x = x*cosh(1/x) - 0.1')


%%
%
% Para la funcin g2(x) = g2(x) = 1/acosh((x+d)/x)  se cumple:

e = abs(diff(x2));
r = e(2:end)./e(1:end-1);
r(end);                 % Tasa de convergencia lineal
% Dejemos que MATLAB calcule g2'(sol2):
subs(diff('1/acosh((x+0.1)/x)'), sol2)

plot(r,'r.-'), grid
title('Tasa de convergencia de los iterados de x = 1/acosh((x+0.1)/x)')


%%
%
% El grfico muestra conjuntamente los iterados de las dos funciones de
% punto fijo consideradas. Se aprecia la diferente velocidad de
% convergencia de cada una de ellas.

plot(x1, '.-')
title('Iterados de punto fijo para el problema de la catenaria')
hold on, grid
plot(x2, 'r.-')
xlabel('Iteraciones')
ylabel('Valor del iterado')
legend('g_1(x) = x*cosh(1/x) - 0.1', 'g_2(x) = 1/acosh((x + 0.1)/x)') 
hold off


%%
%
% FIN del Problema 2.6