%% 2.8. La ecuacin logstica
% 
% Este problema muestra la variedad de comportamientos del mtodo de punto
% fijo aplicado a la ecuacin logstica
%                   x = ax(1-x)
% al variar el prmetro a y la estimacin inicial.
% Ademas de los casos de convergencia, es interesante observar oscilaciones
% peridicas de distintos periodos y el comportamiento catico. 
%
%
% Tema 2, Problema 8
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson


%%
%
% a) Representamos como una ecuacin de punto fijo la ecuacin logstica, para
% el caso de a = 4. 
% Las intersecciones con la bisectriz del primer cuadrante son los puntos
% fijos. En este caso son los puntos x = 0 y x = 1 - 1/4 = 3/4.

x = linspace(-0.1, 1.1);
y = 4*x.*(1-x);
plot(x, x, x, y, [0 3/4], [0 3/4], 'r*', 'clipping', 'off')
axis equal
axis([0 1 0 1])
title('Puntos fijos de la ecuacin logstica')
    
    
%%
%
% b) Un punto fijo es atractor si la derivada de la funcin de iteracin en el
% punto fijo es menor que 1 en valor absoluto.
% El punto fijo 0 es atractor si |g?(0)| = |a| < 1; en nuestro caso, si a
% est entre 0 y 1.

y = [0.2 : 0.2 : 0.8]'*(x.*(1-x));
plot(x, x, x, y, 0, 0, 'r*', 'clipping','off')
axis equal
axis([0 1 0 1])
title('Casos con  x = 0 atractor')


%%
%
% El otro punto fijo, x = 1-1/a, es atractor si
% |g'(1-1/a)| = |2-a| < 1
% o sea, para valores de a entre 1 y 3.

a = [1.2 : 0.2 : 2.8]';
y = a*(x.*(1-x));
p = 1 -1./a;
plot(x, x, x, y, p, p, 'r*', 'clipping','off')
axis equal
axis([0 1 0 1])
title('Casos con  x = 1 - 1/a  atractor')


%%
%
% c) Las iteraciones convergen cuadrticamente cuando la derivada se anula
% en el punto fijo. Esto ocurre en el origen, en el caso trivial de a = 0 y
% en el otro punto fijo, 1 - 1/a, para a = 2, en cuyo caso el punto fijo es 1/2.

a = [0 2]';
y = a*(x.*(1-x));
p = [0 1/2];
plot(x, x, x, y, p, p, 'r*', [-0.1 0.1], [0 0], ':', ...
    [0.4 0.6], [0.5 0.5], ':', 'clipping','off')
axis equal
axis([0 1 0 1])
title('Casos con convergencia cuadrtica')


%%
%
% d) Para valores de a entre 0 y 4, la grfica est en [0, 1]  [0, 1].

a = [0.5 : 0.5 : 4];
y = a'*(x.*(1-x));
plot(x, x, x, y, 'clipping','off')
text([0.75:-0.05:0.4],a/4-0.05,[repmat('a = ',8,1),num2str(a')])
axis equal, axis([0 1 0 1])
title('Casos en que [0, 1] va a parar a [0, 1]')


%%
%
% e) Para a = 1/2, los iterados convergen a 0.

tol = 1e-12;
g = inline('1/2*x.*(1-x)');
[sol, x1, incr, k] = puntofijo(g, 0.1, tol, 100)
plot(x, x, x, g(x))
axis equal
axis([-0.05 0.15 -0.05 0.15])
title('Iteraciones de punto fijo de x = 1/2*x*(1-x)')
xlabel('a = 1/2; x_0 = 0.1')
hold on, telaranya(x1), hold off

%%
%
% Los iterados convergen linealmente y la tasa de convergencia es 1/2, que
% coincide con la derivada en 0, g'(0) = a = 1/2.

e = abs(diff(x1));
r = e(2:end)./e(1:end-1);
plot(r,'.-'), grid
title('Tasa de convergencia de los iterados de x = 1/2*x*(1-x)')


%%
%
% f) Para a = 3/2, partiendo de 0.1, los iterados convergen a 1/3.

g = inline('3/2*x.*(1-x)');
[sol, x1, incr, k] = puntofijo(g, 0.1, tol, 100)
plot(x, x, x, g(x))
axis equal
axis([0.05 0.4 0.05 0.4])
title('Iteraciones de punto fijo de x = 3/2*x*(1-x)')
xlabel('a = 3/2; x_0 = 0.1')
hold on, telaranya(x1), hold off

%%
%
% Partiendo de 0.375, los iterados convergen decreciendo a 1/3.

[sol, x2, incr, k] = puntofijo(g, 0.375, tol, 100)
plot(x, x, x, g(x))
axis equal
axis([0.3 0.4 0.3 0.4])
title('Iteraciones de punto fijo de x = 3/2*x*(1-x)')
xlabel('a = 3/2; x_0 = 0.375')
hold on, telaranya(x2), hold off


%%
%
% En ambos casos, los iterados convergen linealmente, aunque en el caso de
% x0 = 0.1, al principio la convergencia es ms lenta. 
% La tasa de convergencia es g'(0) = 2 - a = 1/2.

e1 = abs(diff(x1));
r1 = e1(2:end)./e1(1:end-1);
e2 = abs(diff(x2));
r2 = e2(2:end)./e2(1:end-1);
plot(r1, '.-')
hold on, grid
plot(r2, 'r.-')
title('Tasa de convergencia de los iterados de x = 3/2*x*(1-x)')
legend('x_0 = 0.1','x_0 = 0.375')
hold off

%%
%
% g) Para a = 2, partiemos de una estimacin bastante alejada del punto
% fijo, 1/2. Al principio los iterados se acercan lentamente, pero luego
% caen rpidamente en el punto fijo.

g = inline('2*x.*(1-x)');
[sol, x3, incr, k] = puntofijo(g, 0.95, tol, 10)
plot(x, x, x, g(x))
axis equal
axis([0 1 0 1])
title('Iteraciones de punto fijo de x = 2*x*(1-x)')
xlabel('a = 2; x_0 = 0.95')
hold on, telaranya(x3), hold off


%%
%
% La tasa de convergencia lineal tiende a 0, mientras que la tasa
% cuadrtica se estabiliza. Por tanto hay convergencia cuadrtica.

e = abs(diff(x3));
r = e(2:end)./e(1:end-1)        % Tasa de convergencia lineal
r2 = e(2:end)./e(1:end-1).^2    % Tasa de convergencia cuadrtica
plot(r2,'.-'), grid
title('Tasa de convergencia cuadrtica de los iterados de x = 2*x*(1-x)')

%%
%
% h) Para a = 3.5, los iterados no convergen a 0 ni al otro punto fijo,
% 5/7, pues en ambos la derivada de la funcin de iteracin es mayor que 1
% en valor absoluto. Los iterados mostrados en el grfico parten
% de 0.7, que es un punto muy prximo al punto fijo 5/7.

g = inline('3.5*x.*(1-x)');
[sol, x4, incr, k] = puntofijo(g, 0.7, tol, 99)
plot(x, x, x, g(x))
axis equal
axis([0 1 0 1])
title('Iteraciones de punto fijo de x = 3.5*x*(1-x)')
xlabel('a = 3.5; x_0 = 0.7')
hold on, telaranya(x4), hold off

%%
%
% Sin embargo, a la larga los iterados presentan un comportamiento
% peridico, tomando sucesivamente 4 valores distintos que se repiten
% indefinidamente. La figura muestra la evolucin de los iterados
% posteriores al 20.

plot(x, x, x, g(x))
axis equal
axis([0 1 0 1])
title('Iteraciones de periodo 4 de x = 3.5*x*(1-x)')
xlabel('a = 3.5')
hold on, telaranya(x4(20:end)), hold off


%%
%
% La grfica muestra los valores de los iterados. En ella se aprecia la
% oscilacin de periodo 4.

plot(x4,'+')
title('Iterados de punto fijo de x = 3.5*x*(1-x)')
xlabel('Iteracin')
ylabel('Valor del iterado')
grid

%%
%
% En cambio, para a = 3,75, los iterados presentan un comportamiento
% catico. La figura  muestra 100 iterados obtenidos a partir de x0 = 0.5.

g = inline('3.75*x.*(1-x)');
[sol, x5, incr, k] = puntofijo(g, 0.5, tol, 999)
plot(x, x, x, g(x))
axis equal
axis([0 1 0 1])
title('Iteraciones de punto fijo de x = 3.75*x*(1-x)')
xlabel('a = 3.75; x_0 = 0.5')
hold on, telaranya(x5(1:100)), hold off

%%
%
% A diferencia del caso anterior, en esta figura no se aprecia regularidad
% en los iterados.

plot(x5,'+')
title('Iterados de punto fijo de x = 3.75*x*(1-x)')
xlabel('Iteracin')
ylabel('Valor del iterado')
grid


%%
% i) El fractal de Feigenbaum se obtiene representando los valores
% estacionarios de las iteraciones de la ecuacin de punto fijo
% x = a*x*(1-x) como ordenadas de puntos de abscisa a, para valores de a
% entre 0 y 4. 
% Para cada valor de a, el programa feigenbaum.m efecta 400 iteraciones a
% partir de un punto arbitrario y representa las 200 ltimas, suponiendo
% que el proceso ya se ha estabilizado.

type feigenbaum
feigenbaum
axis([0 4 -0.01 1])
xlabel('Valor del parmetro ''a''')
ylabel('Iterados estacionarios')
title('Fractal de Feigenbaum')


%%
%
% Cambiando las variables p, q, r y s de la funcin feigenbaum.m , puede
% obtenerse una zona del fractal con ms detalle.

feigenbaum(3.5, 3.75, 0.70, 0.95)
title('Detalle del fractal de Feigenbaum')


%%
%
% FIN del Problema 2.8