%% 2.9. Diodo de capacidad variable
% 
% En este problema se estudia el funcionamiento de un diodo de capacidad
% variable.
% Para obtener la tensin del diodo en funcin de la carga se utilizan
% mtodos de resolucin de ecuaciones no lineales.
%
%
% Tema 2, Problema 9
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson


%%
%
% Representamos grficamente la capacidad C de un diodo en funcin de la
% tensin V entre sus extremos.

C0 = 81e-12; %F capacidad nominal del diodo
fi = 0.6;    %V 
gamma = 0.44;
V = linspace(0,5);
C = C0./(1+abs(V)/fi).^gamma;
plot(V, C)
title('Diodo de capacidad variable')
xlabel('Tensin')
ylabel('Capacidad')
grid


%%
%
% a) La carga del diodo est relacionada con la capacidad y la tensin segn
% la ley de Faraday, Q = V*C. Calculamos la carga cuando la tensin es 1.

v = 1;
c = C0./(1+abs(v)/fi).^gamma;
q = v*c


%%
%
% b) Representamos grficamente la carga en funcin de V para valores de la
% tensin entre 0 y 5V.
% Marcamos en la grfica el valor obtenido en a).
Q = V.*C;
plot(V, Q, v, q, 'r*')
title('Diodo de capacidad variable')
xlabel('Tensin')
ylabel('Carga')
grid


%%
%
% Para analizar el comportamiento del diodo, se necesita conocer la tensin
% V en funcin de la carga Q, lo que lleva a la ecuacin 
% C0*V - Q*(1+abs(V)/fi)^gamma = 0.
% Codificamos esta ecuacin y su derivada en un fichero .m.

type diodo2


%%
%
% c) Resolvemos la ecuacin por el mtodo de Newton y dibujamos la solucin en
% la grfica previa.

[v, xn, incr, k] = newton('diodo2', 3, 1e-6, 30)
hold on
plot(v, 1e-10, 'g*')
hold off


%%
%
% d) Expresamos la relacin entre carga y tension como ecuacin de punto
% fijo. Representamos la ecuacin de punto fijo como la interseccin entre
% la funcin de iteracin y la bisectriz del primer cuadrante.

type diodopf

plot(V, diodopf(V), V, V)
axis equal square
title('Ecuacin de punto fijo para el problema del diodo')
legend('diodof(V)','V')
grid


%%
%
% Resolvemos por el mtodo de punto fijo. Observamos que se requieren ms
% iteraciones que con el mtodo de Newton.

[sol,xf,incr,k] = puntofijo('diodopf',3,1e-6,30)
axis([1.5 3.5 1.5 3.5])
hold on, telaranya(xf), hold off

%%
%
% En el grfico se aprecia la distinta velocidad de convergencia de los
% iterados de Newton y de punto fijo.

plot(xn, 'x-')
title('Iterados de Newton para el problema del diodo')
hold on
plot(xf, 'r.-')
hold off
xlabel('Iteraciones')
legend('Newton','Punto fijo')
grid


%%
%
% e) Los cocientes entre diferencias de iterados consecutivos del mtodo de
% punto fijo se aproximan cada vez ms a la derivada de la funcin de
% iteracin.

XR = xf(2:end);
XL = xf(1:end-1);
dg = diff(XR)./diff(XL)     % Cocientes de diferencias

dy = diodopfd(xf(2:end-1))          % Derivada
n = 2:length(XR);
plot(n, dg, '.-', n, dy, '.-')
title('Punto fijo para el problema del diodo')
xlabel('Iteracin')
legend('Cociente de diferencias','Valor de la derivada','Location','Best')
grid

%%
%
% FIN del Problema 2.9