%% 2.10. Amortizacin de un prstamo
% 
% El problema trata de la determinacin del inters de un prstamo en
% funcin del nmero e importe de los plazos.
% Se aplica el mtodo de Newton y se examinan distintas funciones de
% iteracin con el mtodo de punto fijo.
%
%
% Tema 2, Problema 10
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson


%%
%
% Nos ofrecen un crdito de 6.000 euros a devolver en 50 mensualidades. En
% la grfica se muestra la cantidad a satisfacer mensualmente en funcin
% del tipo de inters.

C = 6000; n = 50;
r = linspace(1.001, 1.1);
a = C*r.^n.*(r-1)./(r.^n-1);
plot(r, a)
axis([1 1.1 100 700])
title('Amortizacin de 6000 en 50 mensualidades')
xlabel('r = 1 + i(tipo de inters mensual)')
ylabel('Importe del pago mensual ()')
grid


%%
%
% a) Para obtener por el mtodo de Newton el inters de la ecuacin de las anualidades, la
% codificamos junto con su derivada en un fichero de funcin.

type anualidadn


%%
%
% Aplicamos el mtodo de Newton para obtener el inters del crdito
% partiendo de la estimacin inicial r = 1.1 hasta alcanzar una precisin
% del orden de 10^-6.
% Representamos el resultado sobre el grfico anterior.

[rn, xn, incr, k] = newton('anualidadn', 1.1, 1e-6, 30)
hold on
plot(rn, 150, 'r*')
hold off
title('Mtodo de Newton para el problema de la anualidad')

%%
%
% Se ha obtenido el resultado con 4 iteraciones. La convergencia es cuadrtica.

e = abs(diff(xn));
tcc = e(2:end)./e(1:end-1).^2
plot(tcc, '.-')
title('Tasa de convergencia cuadrtica')
xlabel('Iteracin')
grid

%%
%
% b) Para aplicar el mtodo del punto fijo podemos despejar r de la
% ecuacin de las anualidades de varias formas.
% I) Consideramos la iteracin de punto fijo dada por la funcin
% anualidadpf1, que representamos junto a la identidad para interpretar
% grficamente las iteraciones.

type anualidadpf1

plot(r, anualidadpf1(r), r, r)
axis equal square
title('Ecuacin de punto fijo I para el problema de la  anualidad')
legend('anualidadpf1(r)','r')
grid


%%
%
% Resolvemos por el mtodo de punto fijo. Observamos que se requieren ms
% iteraciones que con el mtodo de Newton.

[sol, r1, incr, k] = puntofijo('anualidadpf1', 1.1, 1e-6, 50)
hold on
telaranya(r1)
hold off
%axis([1.5 3.5 1.5 3.5])


%%
%
% La convergencia de punto fijo es generalmente lineal. En este caso, la
% tasa de convergencia es bastante elevada, por lo que la convergencia es
% lenta.

e = abs(diff(r1));
tcl = e(2:end)./e(1:end-1);
tcl(end)                 % Tasa de convergencia lineal
plot(tcl,'.-'), grid
title('Tasa de convergencia de los iterados de r = 1 + a*(r^n-1)/(C*r^n)')


%%
%
% II) Consideramos ahora la ecuacin de punto fijo dada en anualidadpf2.

type anualidadpf2

plot(r, anualidadpf2(r), r, r)
axis equal square
title('Ecuacin de punto fijo II para el problema de la  anualidad')
legend('anualidadpf2(r)','r')
grid


%%
%
% Aplicando el mtodo de punto fijo, observamos que se requieren menos
% iteraciones que con la funcin de punto fijo anterior.

[sol, r2, incr, k] = puntofijo('anualidadpf2', 1.1, 1e-6, 30)
hold on, telaranya(r2), hold off
%axis([1.5 3.5 1.5 3.5])


%%
%
% Los iterados de anualidadpf2 convergen linealmente. La tasa de
% convergencia es menor que la de anualidadpf1, por lo que la convergencia
% es ms rpida.

e = abs(diff(r2));
tcl = e(2:end)./e(1:end-1);
tcl(end)                 % Tasa de convergencia lineal
plot(tcl,'.-'), grid
title('Tasa de convergencia de los iterados de r = (a*(r^n-1)/(C*(r-1)))^1^/^n)')


%%
%
% III) Consideramos finalmente la ecuacin de punto fijo dada en anualidadpf3.
% Observamos en la grfica que la curva corta en el punto fijo con
% pendiente mayor que 1.

type anualidadpf3

r = linspace(1, 1.8);
plot(r, anualidadpf3(r), r, r)
axis equal square
title('Ecuacin de punto fijo III para el problema de la  anualidad')
legend('anualidadpf3(r)','r')
grid


%%
%
% En consecuencia, el mtodo de punto fijo diverger. En efecto, 

[sol, r3, incr, k] = puntofijo('anualidadpf3', 1.1, 1e-6, 10)
hold on, telaranya(r3), hold off
%axis([1.5 3.5 1.5 3.5])


%%
%
% FIN del Problema 2.10