function [sol,x,incr,k] = puntofijo(g,x0,tol,maxiter)

% [sol,x,incr,k] = pfijo(g,x0,tol,maxiter)
% 
% Iteraciones de la ecuacin de punto fijo  x = g(x)
% x0: Estimacin inicial 
% tol: precisin deseada
% maxiter: tope de iteraciones
%
% sol: solucin aproximada
% x: vector de iterados
% incr: distancia entre los dos ltimos
% k: numero de iteraciones

% Ejemplo: [sol,x] = puntofijo('cos',0,0.001,100)

% Problemas resueltos de Mtodos Numricos
% Cordero, Hueso, Martnez, Torregrosa
% Ed. Thomson
%
% jlh, jun-2005

% Inicializar variables
k = 1;
x = x0;
incr = tol+1;

% Iteraciones de punto fijo
while incr>tol & k<=maxiter
    x(k+1) = feval(g,x(k));
    incr = abs(x(k+1)-x(k));
    k = k+1;					% Contador de iteraciones
end

% Salida
if incr>tol
    sol = [];
    disp('Insuficientes iteraciones')
else
    sol = x(end);
end
