function [sol,x,incr,iter]=regulafalsi(f,a,b,tol,maxiter)

% [sol,x,incr,iter]=regulafalsi(f,a,b,tol,maxiter)
% 
% Aplica Regula Falsi a la ecuacin f(x)=0 en el intervalo [a,b]
% 
% tol: precisin deseada
% maxiter: tope de iteraciones
%
% sol: solucin aproximada
% x: vector de iterados
% incr: distancia entre los dos ltimos
% iter: numero de iteraciones
%
% Ejemplo: regulafalsi('cos',1,2,0.001,10)


% Problemas resueltos de Mtodos Numricos
% Cordero, Hueso, Martnez, Torregrosa
% Ed. Thomson
%
% jlh, jun-2005

% Comprueba que hay cambio de signo en el intervalo
fa = feval(f,a);
fb = feval(f,b);
if fa*fb>0
   disp('La funcin debe tener signo distinto en los extremos del intervalo')
   return
end

% Inicializar variables
iter = 0;
x = a;
incr = abs(b-a);

% Regula falsi
while incr>tol & iter<maxiter
   c = (a*fb-b*fa)/(fb-fa);
   x = [x,c];    
   fc = feval(f,c);
   if fc==0
      a = c; b = c;
   elseif fa*fc<0
      b = c; fb = fc;
   else  %   fc*fb<0
      a = c; fa = fc;
   end
   incr = abs(x(end)-x(end-1));
   iter = iter+1;   
end

% Salida
sol = c;
if incr>tol
   disp('Diverge o insuficientes iteraciones')
end
