function [sol,x,incr,k] = secante(f,x1,x2,tol,maxiter)

% [sol,x,incr,k] = secante(f,x1,x2,tol,maxiter)
% 
% Aplica secante a la ecuacin f(x)=0
% a partir de las estimaciones iniciales x1 y x2
%
% tol: precisin deseada
% maxiter: tope de iteraciones
%
% sol: solucin aproximada
% x: vector de iterados
% incr: distancia entre los dos ltimos
% k: numero de iteraciones
%
% Ejemplo: secante('cos',1,2,0.001,10)


% Problemas resueltos de Mtodos Numricos
% Cordero, Hueso, Martnez, Torregrosa
% Ed. Thomson
%
% jlh, jun-2005

% Inicializar variables
x = [x1, x2];
y = feval(f,x);
k = 2;
incr = tol+1;
delta = x2-x1;

% Iteraciones de la secante
while incr>tol & k<maxiter
    m = (y(k)-y(k-1))/delta;
    delta = -y(k)/m;
    x(k+1) = x(k) + delta;
    y(k+1) = feval(f,x(k+1));
    incr = abs(delta);
    k = k+1;   
end

% Salida
if incr>tol
    sol = [];
    disp('Diverge o insuficientes iteraciones')
else
    sol = x(end);
end
