%% 3.2. Solubilidad de la sal de amniaco. Desplazamiento de datos.
%
% En este problema se comprueba la mejora del nmero de condicin de la
% matriz de Vandermonde que resulta al realizar la interpolacin polinmica
% con desplazamiento de datos.
%
%
% Tema 3, Problema 2
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson


%% 
%
% a) La matriz asociada al sistema para la obtencin del polinomio de
% interpolacin de grado 3 es de Vandermonde y como sabemos est mal
% condicionada.

x = (10:10:60)';
y = [33 37 42 46 51 55]';
x3 = x(2:5);
y3 = y(2:5);
A = vander(x3)
ncon = cond(A)

plot(x, y, '*')
title('Solubilidad del cloruro amnico')
xlabel('Temperatura')
grid


%%
%
% Obtenemos los coeficientes del polinomio y lo representamos grficamente.

p = A\y3

xg = linspace(x(1), x(end));
yg = polyval(p, xg);
plot(x, y, '*', xg, yg)
title('Polinomio interpolador de grado 3')
ylabel('Solubilidad')
xlabel('Temperatura')
grid


%% 
%
% b) Veamos que el nmero de condicin mejora, al utilizar otra expresin
% del polinomio, realizando un desplazamiento del origen a la media
% aritmtica de las abscisas.

xm = mean(x3);
xd = x3-xm;
Ad = vander(xd);
ncon = cond(Ad)
pd = Ad\y3


%% 
%
% Evaluamos y representamos el polinomio desplazado teniendo en cuenta el
% desplazamiento realizado. El grfico no se distingue de de la
% interpolacin sin desplazamiento.

ygd = polyval(pd, xg - xm);
plot(x, y, '*', xg, ygd)
title('Polinomio interpolador de grado 3 con desplazamiento')
ylabel('Solubilidad')
xlabel('Temperatura')
grid


%% 
%
% La estimacin de la solubilidad en las temperaturas desconocidas es 

t = polyval(pd, [25, 35, 45] - xm)

plot(x, y, '*', xg, ygd, [25 35 45], t, 'r*')
title('Solubilidad del cloruro amnico')
xlabel('Temperatura')
legend('Datos', 'Pol de grado 3', 'Valores interpolados')
grid


%%
%
% FIN del Problema 3.2