%% 3.3. Contracciones de un resorte. Polinomio de Lagrange.
%
% En este problema la interpolacin de los datos se realiza mediante el
% polinomio de Lagrange. Aunque la obtencin del mismo es inmediata, la
% evaluacin precisa un elevado nmero de operaciones.
%
%
% Tema 3, Problema 3
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson


%% 
%
% Representamos las contracciones en funcin de las distintas cargas. A
% partir de las coordenadas de los tres datos de abscisa ms prxima a x =
% 13, (10,105), (15,172) y (20,253) la obtencin del polinomio de Lagrange
% es sistemtica: 
%
%  p(x) = 172(x-10)(x-20)/(15-10)(15-20) + 105(x-10)(x-20)/(10-15)(10-20)
%  + 253(x-15)(x-10)/(20-15)(20-10)

x = 5:5:25;;
y = [49 105  172  253 352];
bar(x,y)
xlabel('Carga')
ylabel('Contraccin')


%%
%
% La evaluacin del polinomio de Lagrange requiere clculos laboriosos que
% sistematizamos en un programa.

type lagrange

%%
%
% Podemos obtener la evaluacin del polinomio de Lagrange en un vector de
% puntos equiespaciados en el intervalo de trabajo.

x = [5  10  15  20  25];
y = [ 49  105  172  253  352];
xg = linspace(x(1), x(end));
yg = lagrange(x, y, xg);

plot(x,y,'*',xg,yg)
title('Polinomio de Lagrange')
xlabel('Carga')
ylabel('Contraccin')
grid


%%
%
% En particular la contraccin del resorte para una masa de 13 Kp ser:

v = lagrange(x,y,13)

hold on
plot(13, v, 'r*')
hold off


%%
%
% FIN del Problema 3.3