%% 3.4. Polinomio de interpolacin de Newton.
%
% Mostramos en este problema el polinomio de interpolacin de Newton, tanto
% su obtencin directa como mediante el uso de la tabla de diferencias
% divididas. Para la evaluacin del mismo utilizamos su forma anidada.
%
%
% Tema 3, Problema 4
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson


%% 
%
% Para hallar el polinomio de grado 3, tomaremos los 4 puntos ms prximos
% a x = 10.5 que es donde se va a realizar la estimacin.

hora = 8 : 14;
pax = [41 35 21 9 11 17 32];
sel = [3 4 5 2];

plot(hora, pax, '*', hora(sel), pax(sel), 'r*')
title('Viajeros al metro')
xlabel('Hora')
ylabel('N de viajeros')
grid
hold on


%%
%
% El polinomio de Newton de grado 3 para los puntos (10,21), (11,9),
% (12,11) y (9,35) tiene la siguiente expresin:
%
% p(x) = c4 + c3(x-10) + c2(x-10)(x-11) + c1(x-10)(x-11)(x-12)
%
% Obtendremos los coeficientes resolviendo el sistema de ecuaciones
% lineales triangular que resulta de p(xi) = yi. Hallamos el nmero de
% condicin del sistema.

 x = hora(sel)';
 y = pax(sel)';
 A = [x.^0, x - x(1), (x - x(1)).*(x - x(2)), (x - x(1)).*(x - x(2)).*(x - x(3))]
 c = A\y
 cond(A)
 
 
%%
%
% Evaluamos la expresin de Newton del polinomio de interpolacin para
% representarlo grficamente. 

xg = linspace(8,14);
yg = c(1) + c(2)*(xg - x(1)) + c(3)*(xg - x(1)).*(xg - x(2)) + ...
    c(4)*(xg - x(1)).*(xg - x(2)).*(xg - x(3));

plot(xg, yg, 'g')
title('Polinomio de interpolacin')
hold off


%%
%
% Como alternativa a la resollucin del sistema se utiliza la tabla de
% diferencias divividas, cuya diagonal proporciona directamente los
% coeficientes del polinomio de Newton. La primera columna de la tabla de
% Newton la forman las ordenadas a interpolar.

N = y


%%
%
% El coeficiente del primer trmino del polinomio de Newton es N(1,1).
% Dicho trmino es el polinomio de grado 0 que pasa por el punto (x1, y1).

c = N(1,1);

yg = c(1)*ones(size(xg));
plot(hora, pax, '*', x, y, 'r*', xg, yg)
title('Polinomio de grado 0')
xlabel('Hora')
ylabel('N de viajeros')
grid
hold on


%%
%
% La segunda columna de la tabla de Newton son las diferencias de la
% primera columna, divididas por las diferencias de las abscisas.

N(2:4, 2) = [(y(2)-y(1))/(x(2)-x(1))
             (y(3)-y(2))/(x(3)-x(2))   
             (y(4)-y(3))/(x(4)-x(3))]


%%
%
% El coeficiente del segundo trmino del polinomio de Newton es N(2,2).
% Los dos primeros trminos forman el polinomio de grado 1 que pasa por los
% puntos (x1, y1) y (x2, y2).

c(2) = N(2,2);

yg = yg + c(2)*(xg - x(1));
plot(xg, yg, 'g')
title('Polinomios de grado <= 1')


%%
%
% La tercera columna de la tabla de Newton son las diferencias de la
% segunda columna, divididas por las diferencias de las abscisas con salto
% de 2.

N(3:4, 3) = [(N(3,2)-N(2,2))/(x(3)-x(1))   
             (N(4,2)-N(3,2))/(x(4)-x(2))]

         
%%
%
% El coeficiente del tercer trmino del polinomio de Newton es N(3,3).
% Los tres primeros trminos forman el polinomio de grado 2 que pasa por
% los puntos (x1, y1), (x2, y2) y (x3, y3).

c(3) = N(3,3);

yg = yg + c(3)*(xg - x(1)).*(xg - x(2));
plot(xg, yg, 'm')
title('Polinomios de grado <= 2')


%%
%
% En la ltima columna de esta tabla aparece la diferencia tercera.

N(4, 4) = (N(4,3)-N(3,3))/(x(4)-x(1))


%%
%
% El coeficiente del cuarto trmino del polinomio de Newton es N(4,4).
% Los cuatro primeros trminos forman el polinomio de grado 3 que pasa por
% los puntos (x1, y1), (x2, y2), (x3, y3) y (x4, y4).

c(4) = N(4,4);

yg = yg + c(4)*(xg - x(1)).*(xg - x(2)).*(xg - x(3));
plot(xg, yg, 'y')
title('Polinomios de grado <= 3')
hold off


%%
%
% La diagonal de la tabla proporciona los coeficientes del polinomio de
% Newton.

p = diag(N)


%%
%
% Si slo necesitamos el polinomio de tercer grado, podemos evaluarlo con
% la mitad de operaciones que antes, utilizando la expresin anidada.

yg = (((p(4)*(xg - x(3)) + p(3)).*(xg - x(2))) + p(2)).*(xg - x(1)) + p(1);

plot(hora, pax, '*', x, y, 'r*', xg, yg)
title('Polinomio de grado 3')
xlabel('Hora')
ylabel('N de viajeros')
grid


%%
%
% Estimamos el nmero de viajeros a las 10h 30' con el mismo calculo
% anterior, variando la entrada.

xg = 10.5;
yg = (((p(4)*(xg - x(3)) + p(3)).*(xg - x(2))) + p(2)).*(xg - x(1)) + p(1)

hold on
plot(10.5, yg, 'k*')
hold off


%%
%
% Sistematizamos los clculos realizados en la funcin polynew.m que
% obtiene los coeficientes del polinomio de Newton construyendo la tabla de
% diferencias divididas. 

type polynew


%%
%
% Consideremos todos los puntos para observar como al aadir datos, los
% coeficientes antes calculados se mantienen.

orden = [3 4 5 2 6 1 7];
x = hora(orden); 
y = pax(orden);
[c, T] = polynew(x,y)


%%
%
% Ahora sistematizamos los clculos realizados para la evaluacin del
% polinomio de Newton en en la funcin pnval.m que, evala dicho polinomio
% en el vector de valores que figure como tercer parametro de entrada.

type pnval


%%
%
% Evaluamos el polinomio de grado 6 para realizar su representacin grfica.

xg = linspace(8, 14);
yg = pnval(x, c, xg);
plot(x, y, '*', xg, yg)
title('Polinomio de grado 6')
xlabel('Hora')
ylabel('N de viajeros')
grid


%%
%
% Finalmente, calculamos el nmero de viajeros a las 10:30 horas. Notemos
% que al considerar el polinomio de mayor grado, la estimacin ha variado.

z = pnval(x, c, 10.5)
hold on
plot(10.5, z, 'r*')
hold off


%%
%
% FIN del Problema 3.4