%% 3.6.  Funcin de Runge. Splines.
%
% En este problema se aproxima la funcin de Runge mediante un spline
% cbico que interpole 9 puntos equiespaciados en el intervalo [-5,5] y
% cuya derivada segunda en los extremos coincida con la de la funcin dada.
%
%
% Tema 3, Problema 6
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson


%% 
%
% Representamos la funcin de Runge en el intervalo [-5,5] junto a los
% puntos de interpolacin.
type runge
x = linspace(-5,5,9)';
y = runge(x);
xg = linspace(-5,5);
yg = runge(xg);
plot(x,y,'*',xg,yg)
title('Funcin de Runge')

%% 
%
% La matriz M del sistema que verifican los valores de la derivada segunda
% del spline cbico en los nodos, se construye a partir del vector h de las
% diferencias de los nodos, x.
n = 8;              % Nmero de piezas
h = diff(x);        % Vector de pasos
dl = h(1:n-1)/3;
dr = h(2:n)/3;
dp = dl+dr;         % Diagonal principal
ds = h(2:n-1)/6;    % Diagonales secundarias
M = diag(dp) + diag(ds,-1) + diag(ds,1)
%% 
%
% Los valores de la derivada segunda en los extremos son
r1 = -2/(1+5^2)^2+4*5^2/(1+5^2)^3
rn1 = r1
%%
%
% Obtenemos los trminos independientes y la solucin del sistema que completamos con los
% valores calculados en los extremos:
e = diff(y)./h;
g = diff(e);
g(1) = g(1) - h(1)*r1/6;
g(n-1) = g(n-1) - h(n)*rn1/6;
r = M\g;
r = [r1;r;rn1];
%%
%
% Una vez obtenidas las derivadas segundas, aplicamos las frmulas de los coeficientes de
% los polinomios interpolantes y componemos la matriz del spline, S.
rl = r(1:n);
rr = r(2:n+1);
a = diff(r)./h/6;
b = rl/2;
c = e -(rr + 2*rl).*h/6;
d = y(1:n);
s = [a b c d]
%%
%
% Evaluamos y representamos el spline junto a la funcin que interpola.
% Pueden repetirse los clculos con mayor nmero de subintervalos y comprobar que la
% aproximacin mejora.
xg = linspace(-5,5);
yg = 1./(1+xg.^2);
ps = mkpp(x,s);
ys = ppval(ps,xg);
plot(x,y,'*',xg,yg,xg,ys,':')
legend('Nodos','Funcion','Spline natural')
%%
%
% FIN del Problema 3.6