%% 3.7.  Interpolacin global y segmentaria. 
%
% En este problema se realizan dos interpolaciones polinmicas distintas
% para la funcin y = x/(1+x^2) y otro ajuste por un spline cbico
% completo, se comparan los nmeros de condicin de los distintos sistemas
% lineales que se resuelven.
%
%
% Tema 3, Problema 7
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson


%% 
%
% En primer lugar interpolamos la funcin por un polinomio tomando 11
% puntos equiespaciados en [-5,5], resolviendo el sistema cuya matriz
% asociada es de Vandermonde, con un elevado nmero de condicin.

x = (-5:5)';
y = x./(1+x.^2);
A = vander(x);
cond(A)
p = A\y
xg = linspace(-5, 5);
yf = xg./(1 + xg.^2);
yp = polyval(p, xg);
plot(x, y, '*', xg, yf, xg, yp)
legend('Nodos', 'Funcin', 'Polinomio', 'Location', 'SouthEast')


%% 
%
% El polinomio resultante tiene sus coeficientes de grado par
% prcticamente nulos. Por lo que se propone realizar la interpolacin
% considerando slo los cinco trminos impares del polinomio y considerando
% los puntos x= 1,2,3,4,5. La matriz que se obtiene es semejante a la de
% Vandermonde y el nmero de condicin sigue siendo muy elevado.

x = (1:5)';
y = x./(1 + x.^2);
A = [x.^9, x.^7, x.^5, x.^3, x]
cond(A)
p = A\y


%% 
%
% El polinomio coincide con el obtenido a partir de 11 puntos.

xg = linspace(-5, 5)';
yf = xg./(1 + xg.^2);
A = [xg.^9, xg.^7, xg.^5, xg.^3, xg];
yp = A*p;
plot(x, y, '*', xg, yf, xg, yp)
legend('Nodos', 'Funcin', 'Polinomio', 'Location', 'SouthEast' )


%%
%
% Para obtener el spline completo se aaden dos ecuaciones al sistema
% lineal que verifican los valores de la derivada segunda del spline cbico
% en los nodos, rk, estas ecuaciones son las que resultan de expresar la
% primera derivada en funcin de los valores rk, resultando la matriz del
% sistema bastante mejor condicionada que las anteriores:

x = (-5 : 5)';
n = length(x) - 1;
y = x./(1 + x.^2);
h = diff(x);
dd = ([h; 0] + [0; h])/3;    % Diagonal principal
ds = h/6;                    % Diagonal secundaria
M = diag(dd) + diag(ds,-1) + diag(ds,1)
cond(M)


%%
%
% Utilizamos la derivada primera para obtener los trminos independientes y
% resolvemos el sistema lineal para poder formar los coeficientes del
% spline:

dy1 = (1-5^2)/(1+5^2)^2;
dyn = dy1;
e = diff(y)./h;
g = diff([dy1; e; dyn]);
r = M\g;
rl = r(1:n);
rr = r(2:n+1);
a = diff(r)./h/6;
b = rl/2;
c = e -(rr + 2*rl).*h/6;
d = y(1:n);
s = [a b c d]


%%
%
% Pasamos el spline a forma de polinomio segmentario y lo representamos grficamente
% junto a la funcin dada. En la grfica se aprecia que el polinomio
% segmentario se comporta mucho mejor que el global cerca de los extremos
% del intervalo.

ps = mkpp(x, s);
ys = ppval(ps, xg);
plot(x, y, '*', xg, yf, xg, yp, xg, ys)
legend('Nodos', 'Funcin', 'Polinomio', 'Spline completo', 'Location', 'SouthEast')


%%
%
% FIN del Problema 3.7