%% 3.9.  Demanda de energa elctrica. 
%
% En este problema se presentan distintas interpolaciones a partir de una
% tabla de datos. En primer lugar se obtiene el polinomio que pasa por
% todos los puntos, despus se propone mejorar la interpolacin dividiendo
% el intervalo en dos y hallando un polinomio para cada subintervalo.
% Finalmente se aproximan los datos mediante un spline cbico y se comparan los
% resultados obtenidos.
%
%
% Tema 3, Problema 9.
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson


%% 
%
% Almacenamos en sendos  vectores t e y las horas y los consumos,
% respectivamente y calculamos el polinomio de interpolacin con MATLAB,
% ste presenta unos coeficientes que varan mucho de escala. La grfica
% del mismo presenta oscilaciones en los extremos, particularmente en el
% izquierdo.

t = 0:2:24;
y = [31250 28000 26000 26000 34000 37500 38000 35500 34500 38500 ...
     39500 36500 33000];
p = polyfit(t, y, 12)

tg = linspace(0, 24);
yg = polyval(p, tg);
plot(t, y, '*', tg, yg)
title('Demanda de energa elctrica')
xlabel('Hora')
ylabel('Megawatios')
legend('Nodos', 'polinomio de interpolacin', 0)
grid


%%
%
% Interpolamos ahora dos polinomios dividiendo el intervalo por la mitad.
% El sistema sigue siendo mal condicionado y MATLAB muestra de nuevo el
% correspondiente warning, aunque en la representacin grfica se observa
% que se producen menos oscilaciones y la unin entre los dos polinomios es
% 'suave'.

p1 = polyfit(t(1:7), y(1:7), 6);
tg1 =  0:0.1:12; 
yp1 = polyval(p1, tg1);
p2 = polyfit(t(7:13), y(7:13), 6);
tg2 = 12:0.1:24; 
yp2 = polyval(p2, tg2);
plot(t, y, '*', tg1, yp1, tg2, yp2)
title('Demanda de energa elctrica')
xlabel('Hora')
ylabel('Megawatios')
legend('Datos', 'Polinomio 1', 'Polinomio 2', 0)
ax = axis;
grid


%%
%
% Al tratarse de datos experimentales, no tenemos informacin acerca de las
% derivadas, por lo que utilizamos el spline 'no nodo'. Los coeficientes
% del spline estn mejor escalados que los del polinomio y la
% representacin grfica sugiere una evolucin ms razonable de la demanda
% elctrica.

ps = spline(t, y);
s = ps.coefs

ys = ppval(ps, tg);
plot(t, y, '*', tg, ys)
title('Demanda de energa elctrica')
xlabel('Hora')
ylabel('Megawatios')
legend('Datos', 'Spline ''no nodo''', 0)
axis(ax);
grid


%%
%
% FIN del Problema 3.9