%% 3.10.  Trayectoria robtica. 
%
% En este problema se obtienen distintas trayectorias que puede seguir un
% robot para realizar un recorrido predefinido, utilizando diferentes
% tipos de interpolacin. 
%
%
% Tema 3, Problema 10.
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson


%% 
%
% El robot debe coger un objeto situado en la posicin 1 y dejarlo en la
% posicin 6, pasando por los puntos dados (1,1), (1,4), (4,4), (4,1),
% (7,1), (7,4), y recorriendo una trayectoria que evite los obstculos del
% recinto.
type recinto
recinto
%% 
%
% En primer lugar realizamos interpolacin polinmica de las abscisas y de
% las ordenadas respecto al tiempo, obteniendo los polinomios p(t) y q(t)
% que definen la trayectoria del robot.
t = (0:0.2:1);
x = [1 1 4 4 7 7];
y = [1 4 4 1 1 4];
p = polyfit(t,x,5)
q = polyfit(t,y,5)
%% 
%
% Para realizar las representaciones grficas modificaremos el cdigo de la
% funcin recinto.m incluyendo las trayectorias.
type recinto_trayectoria
%% 
%
% En primer lugar representamos el polinomio de interpolacin que pasa por
% todos los puntos.
tg = 0:0.01:1;
xp = polyval(p,tg);
yp = polyval(q,tg);
recinto_trayectoria(xp,yp,1,'Global')
%% 
%
% Seguimos un procedimiento anlogo, pero ahora utilizando el spline 'no
% nodo'. Si en la orden spline aadimos un tercer parmetro de entrada
% obtenemos directamente la evaluacin del polinomio segmentario en esos
% puntos.
xg = spline(t,x,tg);
yg = spline(t,y,tg);
recinto_trayectoria(xg,yg,2,'no nodo')
%% 
%
% Obtenemos ahora el spline completo con las condiciones de que la velocidad inicial
% y final del robot sean 0, aadiendo los valores de la derivada al vector de
% valores.
xg = spline(t,[0,x,0],tg);
yg = spline(t,[0,y,0],tg);
recinto_trayectoria(xg,yg,3,'Completo')
%% 
%
% Finalmente las condiciones de que la aceleracin inicial y final del
% robot sean nulas nos llevan a calcular el spline natural, para obtenerlo
% recopilamos las instrucciones del problema 3.6 en una la funcin
% natural.m.
type natural
%% 
%
% Completamos la representacin grfica y destacamos que la trayectoria
% descrita por los polinomios de interpolacin globales se sale claramente
% del recinto prescrito. El spline 'no nodo' roza ligeramente las paredes
% del mismo, mientras que los splines con condiciones sobre la derivada
% primera o segunda se comportan mucho mejor, quedando alejados en todo
% momento de los bordes y obstculos del recinto.
px = natural(t,x,0,0);
xg = ppval(px,tg); %spline(t,x,tg)
py = natural(t,y,0,0);
yg = ppval(py,tg); %yg = spline(t,y,tg)
recinto_trayectoria(xg,yg,4,'Natural')
%%
%
% FIN del Problema 3.10