%% 4.5 Analisis de la convergencia de integrales
%
% En este problema se analiza, desde un punto de vista numrico, la 
% convergencia de diferentes integrales. Obtenemos una integral impropia 
% convergente y otra divergente, as como una integral infinita convergente
% y otra que no lo es. Aproximamos las integrales convergentes mediante 
% distintos mtodos de cuadratura.
%
% Tema 4, Problema 5.
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson


%% 
% a) Integral en [1,2] de f(x)=log(x)/(x-1)^(1/5)
% 
% Es una integral impropia en x=1. Tomando E=0.01 y descomponemos el 
% intervalo de integracin en [1,1+E] y [1+E,2].
% Construimos el archivo f.m con la funcin integrando. Evaluamos la 
% integral en el primer intervalo mediante el mtodo de Gauss-Legendre con 
% 10 nodos

E=0.01;
I=GaussLegendre('f',1,1+E,10)


%% 
% Ahora hacemos Ee=0.001 y volvemos a descomponer el intervalo [1,1+E] en
% [1,1+Ee] y [1+Ee,1+E]Evaluamos la integral en [1,1+Ee] mediante 
% Gauss-Legendre y los resultados obtenidos nos permiten conjeturar que la
% integral que nos ocupa es convergente.

Ee=0.0001;
I=GaussLegendre('f',1,1+Ee,10)

%%
% Evaluamos la integral mediante Gauss-Legendre con 10 nodos

I=GaussLegendre('f',1,2,10)

%%
% b) Integral en [0,1] de f(x)=exp(x)/x^2.
% 
% Esta integral es impropia en x=0. Utilizamos el mismo razonamiento que en 
% el apartado a). Descomponemos el intervalo de integracin en [0,E] y 
% [E,1], siendo E=0.01, y evaluamos la integral en el intervalo [0,E].

E=0.01;
I=GaussLegendre('g',0,E,10)


%%
% Tomamos Ee=0.0001 y evaluamos la integral en [0,Ee]. A la vista del 
% resultado podemos conjeturar  que la integral no es  convergente.
% Conjetura fcil de comprobar mediante tcnicas analticas.

Ee=0.0001;
I=GaussLegendre('f',0,Ee,10)

%%
% c) Integral en [1, +inf] de la funcin f(x)=(1/x^4)sen(x)
%
% Para analizar esta integral construimos las funcion prob04052.m con
% el siguiente contenido:

type prob04052;

%%
% Llamamos a la funcin prob04052 para 20 nodos, 100 sumandos como maximo y 
% una tolerancia de 1e-7. Los resultados obtenidos nos permiten conjeturar
% que la integral es convergente. 

[I,v,k]=prob04052('h',20,100,1e-7)

%%
% Aproximamos el valor de la integral mediante el mtodo de Romberg

R = Romberg('h',1,2^10,64,1e-6,100)

%%
% d) Integral en [1,+inf]de la funcin f(x)=(1./x).*arctg(x)
%
% Para evaluar la cuarta integral construimos el archivo prob04053.m que va
% calculando mediante cuadratura de Romberg las aroximaciones de los
% distintos sumandos.

type prob04053;

%%
% Llamando a esta funcin, utilizando 32 subintervalos, un nmero maximo 
% de iteraciones de 50 y una tolerancia de 1e-7, se consumen
% todas las iteraciones y las componentes de v no tienden a cero. 
% Conjeturamos que la integral en cuestin no es convergente.

prob04053('z',32,50,1e-7)

%%
%
% FIN del Problema 4.5
