%% 5.1. Modelo de poblacin de Malthus-Verhulst. Mtodo de Euler.
% 
% Este problema muestra paso a paso el funcionamiento del mtodo
% de Euler para la resolucin numrica de un problema de valor inicial
% sobre modelos de poblacin. 
% 
%
% Tema 5, Problema 1
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson

%% 
%
% La ecuacin diferencial del modelo de poblacin de Maltus es p' = 0.05*p.
% La poblacin inicial es p0 = 100.
% Resolvemos el problema de valor inicial en el intervalo [0, 10] por el
% mtodo de Euler con paso h = 1. Se requiere conocer la pendiente en el
% punto (t0, p0), que viene dado por la ecuacin diferencial, dp0 = 0.05*p0.
%
p0 = 100;
h = 1;
dp0 = 0.05*p0;

plot(0, p0, '*')
hold on
plot([0 0.4], p0 + dp0*[0 0.4], 'Color', 'r', 'Linewidth', 2)
axis([-0.3 1.3 98 106])
legend('Valor inicial', 'Pendiente')
title('Modelo de Malthus: p'' = k*p. Mtodo de Euler')
xlabel('Tiempo'), ylabel('Poblacin')


%%
%
% a) El mtodo de Euler estima la solucin en un instante t+h a
% partir del valor de la solucin y su derivada en un punto anterior en
% el tiempo h unidades: (t, p(t)), p'(t).
% Geomtricamente, el clculo equivale a trazar por (t, p(t)) la tangente a
% la solucin y hallar su valor en t+h.

p1 = p0 + h*dp0

plot(1, p1, '*', [0 1], [p0 p1])
title('Modelo de Malthus: p'' = k*p. Mtodo de Euler: paso 1')
t1 = text([0 1] + 0.05, [p0 p1] - 0.4, ['(t_0, p_0)'; '(t_1, p_1)'])
legend off


%%
%
% Ahora se toma (1, p1) como valor inicial y se repite el proceso. La
% ecuacin diferencial proporciona la pendiente de la solucin por (1, p1).

dp1 = 0.05*p1

plot([1 1+0.4], p1 + dp1*[0 0.4], 'Color', 'r', 'Linewidth', 2)
axis([-0.3 2.3 98 112])
title('Modelo de Modelo de Malthus: p'' = k*p.  Mtodo de Euler: paso 2')


%%
%
% Estimamos, como antes, la solucin en t = 2, sumando a p1 el producto del
% paso por la pendiente en (1, p1).

p2 = p1 + h*dp1

plot(2, p2, '*', [1 2], [p1 p2])
t2 = text(2 + 0.05, p2 - 0.4, '(t_2, p_2)')


%%
%
% El tercer paso se hace del mismo modo.

dp2 = 0.05*p2
p3 = p2 + h*dp2
plot([2 2+0.4], p2 + dp2*[0 0.4], 'Color', 'r', 'Linewidth', 2)
plot(3, p3, '*', [2 3], [p2 p3])
title('Modelo de Modelo de Malthus: p'' = k*p.  Mtodo de Euler: paso 3')
set(gca, 'XLimMode', 'auto', 'YLimMode', 'auto')
delete(t1, t2)

%%
%
% Y as sucesivamente, hasta llegar a t = 10.

dp3 = 0.05*p3
p4 = p3 + h*dp3
plot([3 3+0.4], p3 + dp3*[0 0.4], 'Color', 'r', 'Linewidth', 2)
plot(4, p4, '*', [3 4], [p3 p4] )
title('Modelo de Modelo de Malthus: p'' = k*p.  Mtodo de Euler: paso 4')


%%
%
% Y as sucesivamente, hasta llegar a t = 10.

dp4 = 0.05*p4
p5 = p4 + h*dp4
plot([4 4+0.4], p4 + dp4*[0 0.4], 'Color', 'r', 'Linewidth', 2)
plot(5, p5, '*', [4 5], [p4 p5])
title('Modelo de Modelo de Malthus: p'' = k*p.  Mtodo de Euler: paso 5')


%%
%
% Y as sucesivamente, hasta llegar a t = 10.

dp5 = 0.05*p5
p6 = p5 + h*dp5
plot([5 5+0.4], p5 + dp5*[0 0.4], 'Color', 'r', 'Linewidth', 2)
plot(6, p6, '*', [5 6], [p5 p6])
title('Modelo de Modelo de Malthus: p'' = k*p.  Mtodo de Euler: paso 6')


%%
%
% Y as sucesivamente, hasta llegar a t = 10.

dp6 = 0.05*p6
p7 = p6 + h*dp6
plot([6 6+0.4], p6 + dp6*[0 0.4], 'Color', 'r', 'Linewidth', 2)
plot(7, p7, '*', [6 7], [p6 p7])
title('Modelo de Modelo de Malthus: p'' = k*p.  Mtodo de Euler: paso 7')


%%
%
% Y as sucesivamente, hasta llegar a t = 10.

dp7 = 0.05*p7
p8 = p7 + h*dp7
plot([7 7+0.4], p7 + dp7*[0 0.4], 'Color', 'r', 'Linewidth', 2)
plot(8, p8, '*', [7 8], [p7 p8])
title('Modelo de Modelo de Malthus: p'' = k*p.  Mtodo de Euler: paso 8')


%%
%
% Y as sucesivamente, hasta llegar a t = 10.

dp8 = 0.05*p8
p9 = p8 + h*dp8
plot([8 8+0.4], p8 + dp8*[0 0.4], 'Color', 'r', 'Linewidth', 2)
plot(9, p9, '*', [8 9], [p8 p9])
title('Modelo de Modelo de Malthus: p'' = k*p.  Mtodo de Euler: paso 9')


%%
%
% Y as sucesivamente, hasta llegar a t = 10.

dp9 = 0.05*p9
p10 = p9 + h*dp9
plot([9 9+0.4], p9 + dp9*[0 0.4], 'Color', 'r', 'Linewidth', 2)
plot(10, p10, '*', [9 10], [p9 p10])
title('Modelo de Modelo de Malthus: p'' = k*p.  Mtodo de Euler: paso 10')
hold off

%%
%
% b) Modificamos ahora la ecuacin segn el modelo de Verhulst

type verhulst


%%
%
% Aplicamos el mtodo de Euler y mostramos el resultado.

[t,p] = mieuler('verhulst',0:10,p0);

plot(t,p,'.-')
title('Modelo de Verhulst: p'' = kp - mp^2. Mtodo de Euler')
xlabel('Tiempo'), ylabel('Poblacin')
grid


%%
%
% El modelo de Verhulst puede resolverse analticamente. Codificamos la
% solucin del problema de valor inicial considerado en el fichero
% logistic.m.

type logistic


%%
%
% Hallamos los valores de la solucin exacta en los instantes en que
% hemos calculado la solucin aproximada.

solex = logistic(t,p0)';

plot(t, p, '.-', t, solex, '.-')
legend('Sol. analtica', 'Sol. numrica (Euler)')
title('Modelo de Verhulst: p'' = kp - mp^2.')
xlabel('Tiempo'), ylabel('Poblacin')
grid


%%
%
% Obtenemos el error en cada punto y el mximo en el intervalo 

E1 = max(abs(solex-p))

plot(t, abs(p-solex), '.-')
title('Modelo de Verhulst: p'' = kp - mp^2. Error local del mtodo de Euler')
xlabel('Tiempo'), ylabel('Poblacin')
grid
hold on


%%
%
% Resolvemos ahora el problema con el doble de subintervalos para obtener
% el orden del mtodo. El error de la aproximacin con paso 0.5 es
% practicamente la mitad del obtenido con paso 1, por lo que concluimos que
% el mtodo de Euler es de orden 1.

[t, p] = mieuler('verhulst', 0:0.5:10, p0);

solex = logistic(t,p0)';
E2 = max(abs(solex-p))

plot(t, abs(p-solex), 'r.-')
legend('Paso h = 1', 'Paso h = 0.5')
hold off


%%
%
% c) Los valores estacionarios de la ecuacin logstica son las soluciones
% de la ecuacin algebraica kp -mp^2 = 0. O sea, pe0 = 0 y pe1 = k/m = 50.
% Para valores iniciales positivos, la solucin tiende a 50. Obviamente, si
% la poblacin inicial es nula, lo es todo el tiempo.

P = [];
for p0 = 0:10:100
    [t,p] = mieuler('verhulst',0:0.5:200, p0);
    P = [P p];
end

plot(t, P)
title('Modelo de Verhulst: p'' = kp - mp^2. Tendencia a los valores estacionarios 50 y 0.')
xlabel('Tiempo'), ylabel('Poblacin')
grid



%%
%
% FIN del Problema 5.1
