%% 5.3. Depsito cnico. Mtodos de Euler, Euler modificado y Heun.
% 
% Compracin de los mtodos citados en la resolucin de la ecuacin
% diferencial que da la variacin de altura de un lquido que fluye por un
% orificio de un depsito cnico.
% 
%
% Tema 5, Problema 3
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson

%% 
%
% Editamos un fichero con la ecuacin diferencial que regula la variacin de la altura x del
% lquido en el depsito cnico.

type deposito

image(imread('deposito.jpg'))
axis off


%%
%
% Resolvemos el problema con distintos mtodos en el intervalo [0, 864000]
% y paso 18000.

t = 0:18000:864000;
[t,y1] = mieuler('deposito', t, 3);
[t,y2] = heun('deposito', t, 3);
[t,y3] = eulermod('deposito', t, 3);

plot(t, [y1 y2 y3])
title('Depsito cnico')
xlabel('Tiempo'), ylabel('Nivel del lquido')
legend('Euler','Heun','Euler mod.')
grid


%%
%
% La diferencia es inapreciable en la grfica. Con 4 decimales el resultado
% de Euler se diferencia de los otros, que son, a esta precisin, iguales.

disp('Tiempo   Euler   Heun   Euler mod.')
disp([y1 y2 y3])


%%
%
% Conociendo la solucin exacta, podemos calcular los errores de cada
% mtodo.
% Codificamos la solucin en el fichero nivelex.m.

type nivelex


%%
%
% Evalumos la solucin analtica y la comparamos con las soluciones
% numricas.
% Observamos que los errores de los dos ltimos mtodos son insignificantes
% frente al error del mtodo de Euler.

x0 = 3; 
se = nivelex(t,x0)';
e1 = max(abs(y1-se))
e2 = max(abs(y2-se))
e3 = max(abs(y3-se))

plot(t, [y1-se y2-se y3-se])
title('Error de la solucin aproximada')
xlabel('Tiempo'), ylabel('Diferencia')
legend('Euler','Heun','Euler mod.')
grid


%%
%
% Para cuantificar la precisin de cada mtodo, determinamos el orden de
% stos comparando los erores actuales con los obtenidos con doble nmero
% de pasos.
% En primer lugar, comprobamos que el mtodo de Euler es de orden 1 pues el
% error se reduce a la mitad cuando el paso lo hace.

t2 = 0 : 18000/2 : 864000;
[t2, y12] = mieuler('deposito', t2, 3);
se2 = nivelex(t2, x0)';
e1
e12 = max(abs(y12-se2))
e1/e12

plot(t, abs(y1 - se), '.-', t2, abs(y12 - se2), '.-')
title('Mtodo de Euler. Reduccin del error con el paso')
xlabel('Tiempo'), ylabel('Error')
legend('h = 18000','h =  9000')
grid


%%
%
% En cambio, el mtodo de Heun es de orden 2 pues el error se reduce a la
% cuarta parte cuando el paso se reduce a la mitad.

[t2, y22] = heun('deposito', t2, 3);
e2
e22 = max(abs(y22-se2))
e2/e22

plot(t, abs(y2 - se), '.-', t2, abs(y22 - se2), '.-')
title('Mtodo de Heun. Reduccin del error con el paso')
xlabel('Tiempo'), ylabel('Error')
legend('h = 18000','h =  9000')
grid


%%
%
% El mtodo de Euler modificado de orden 2, como el de Heun, aunque en este
% problema el error de Heun es algo menor. 

[t3, y32] = eulermod('deposito', t2, 3);
e3
e32 = max(abs(y32-se2))
e3/e32

plot(t, abs(y3 - se), '.-', t2, abs(y32 - se2), '.-')
title('Mtodo de Euler modificado. Reduccin del error con el paso')
xlabel('Tiempo'), ylabel('Error')
legend('h = 18000','h =  9000')
grid


%%
%
% FIN del Problema 5.3