%% 5.8. Movimiento del pndulo. Mtodo de Runge-Kutta.
% 
% En este problema se resuelve por el mtodo de Runge-Kutta el sistema de
% ecuaciones diferenciales que modeliza la oscilacin de un pndulo con o
% sin resistencia del medio y sujeto o no a fuerzas externas.
% 
%
% Tema 5, Problema 8
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson


%% 
%
% Transformanos la ecuacin de segundo orden que describe el movimiento del
% pndulo en un sistema de dos ecuaciones de orden 1.

type pendulo

image(imread('pendulo.jpg'))
axis equal off

%%
%
% Supongamos que el ngulo inicial es de 1 radin y la velocidad angular
% nula. El pndulo describe un movimiento peridico de periodo aproximado
% al terico T = 2*pi*sqrt(L/g), donde L es la longitud del pndulo.

T = 2*pi*sqrt(0.6/9.81)


%%
%
% Resolvemos el sistema diferencial por el mtodo de Runge-Kutta en un
% intervalo que abarque una oscilacin completa.

[t, y] = rungekutta ('pendulo', 0:.1:1.8, [1, 0]);
disp('Tiempo   ngulo  Velocidad angular')
disp([t(:) y])

plot(t, y)
title('Pndulo en medio no resistente: k = 0. Mtodo de Runge-Kutta')
xlabel('Tiempo')
legend('ngulo', 'Velocidad angular')
grid


%%
%
% Si resolvemos el problema con un intervalo mayor [0, 10], observamos la
% periodicidad de la amplitud y de la velocidad.

[t, y] = rungekutta('pendulo', 0:.01:10, [1, 0]);

plot(t, y)
title('Pndulo en medio no resistente: k = 0. Mtodo de Runge-Kutta')
xlabel('Tiempo')
legend('ngulo', 'Velocidad angular')
grid


%%
%
% El pndulo amortiguado se modeliza dando un valor no nulo a la
% resistencia del aire, por ejemplo k = 0.5.
% Resolviendo el PVI, vemos cmo decrece la amplitud de las oscilaciones y
% la velocidad, tendiendo al reposo.

[t, y] = rungekutta('pendulo05', 0:.01:10, [1, 0]);

plot(t, y)
title('Pndulo subamortiguado: k = 0.5')
xlabel('Tiempo')
legend('ngulo', 'Velocidad angular')
grid


%%
%
% Al aumentar el valor de k, las oscilaciones se amortiguan ms
% rpidamente. 

[t, y] = rungekutta('pendulo10', 0:.01:10, [1, 0]);

plot(t, y)
title('Pndulo subamortiguado: k = 1')
xlabel('Tiempo')
legend('ngulo', 'Velocidad angular')
grid


%%
%
% Cambiamos la funcin pendulo.m para considerar valores distintos de L y m
% y que el pndulo recibe un impulso exterior g(t) = cos(t).

type penduloimp


%%
%
% Resolvemos el problema en un intervalo de tiempo relativamente largo con
% resistencia del medio pequea.
% Observamos que el pndulo, a la larga, se comporta como el forzamiento.

[t, y] = rungekutta('penduloimp', 0:.1:150, [0, 1.7]);

plot(t,y(:,1))
title('Pndulo forzado: k = 0.1')
xlabel('Tiempo')
ylabel('ngulo')

%%
%
% El sistema tiende a estabilizarse despus de dar alguna vuelta completa.
% El nmero de vueltas antes de estabilizarse depende de las condiciones
% iniciales, pero esta dependencia no es lineal, sino ms bien catica.

hold on, text(120, y(1200,1)-2, 'w_0 = 1.7')
[t, y] = rungekutta('penduloimp', 0:.2:150, [0, 1]); 
plot(t, y(:,1), 'g'), text(120, y(600,1)-2, 'w_0 = 1')
[t, y] = rungekutta('penduloimp', 0:.1:150, [0, 1.3]);
plot(t, y(:,1), 'r'), text(120, y(1200,1)-2, 'w_0 = 1.3')
[t, y] = rungekutta('penduloimp', 0:.1:150, [0, 1.5]);
plot(t, y(:,1), 'y'), text(120, y(1200,1)-2, 'w_0 = 1.5')
set(gca, 'YGrid', 'on')
hold off


%%
%
% FIN del Problema 5.8
