%% 5.10. Control biolgico de plagas. Mtodo de Runge-Kutta.
% 
% En este problema se resuelve por el mtodo de Runge-Kutta el sistema de
% ecuaciones diferenciales que modeliza el control biolgico de una plaga,
% en comparacin con el uso de insecticidas. 
% La mosca blanca es una plaga que afecta a los ctricos y puede
% controlarse con su parsito Cales Noacki.
%
%
% Tema 5, Problema 10
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson

%% 
%
% Editamos un fichero con el sistema de ecuaciones diferenciales del
% modelo. Declaramos alfa como variable global para darle valores
% diferentes sin modificar el fichero del sistema.

type mosca

image(imread('mosca.jpg'))
title('Mosca blanca de la fruta')
axis equal off


%%
%
% Resolvemos el sistema para distintos niveles de fumigacin, alfa.
% Observamos que la poblacin final decrece con el aumento de insecticida,
% pero se mantiene en valores bastante elevados.

global alfa
y0 = [0 300];
for k = 1:11
    alfa = 0.05*(k-1);
    [t,y] = rungekutta('mosca', 0:0.1:7, y0);
    a(k) = alfa;
    m(k) = y(end,2);
end

plot(a,m,'*')
title('Control qumico de plagas')
xlabel('Nivel de fumigacin')
ylabel('Poblacin final de mosca')
grid

%%
%
% Alternativamente, la lucha biolgica consiste en introducir el parsito
% en las plantaciones afectadas. Variamos las condiciones iniciales para
% estudiar el efecto del tratamiento combinado: fumigando con insecticida
% adems de introducir el parsito.
% Ahora, la poblacin final es menor cuanto menos insecticida se use. En
% todos los casos la poblacin de mosca resulta menor que cuando no hay
% Cales.

global alfa
y0 = [30 300];
for k = 1:11
alfa = 0.05*(k-1);
[t,y] = rungekutta('mosca',0:0.1:7,y0);
a(k) = alfa;
m(k) = y(end,2);
end

hold on
plot(a,m,'r*')
legend('Slo insecticida', 'Insecticida y Cales')
hold off


%%
%
% Segn los resultados anteriores, podemos concluir que el tratamiento
% ptimo es introducir parsito sin insecticida. Representamos el campo de
% direcciones del sistema, cambiando la escala del eje de ordenadas para
% obtener una mejor representacin.

clear alfa, global alfa, alfa = 0;
n = 6:2:34; m = 50:50:950;
[N,M] = meshgrid(n,m);
[dN, dM] = moscapdf(N, M); 
quiver(N, M/20, dN, dM/20)
title('Direcciones del modelo Plaga-Parsito')
xlabel('Parsito (Cales)')
ylabel('Plaga (Mosca)/20')


%%
%
% Representamos junto al campo de direcciones distintas trayectorias y
% observamos que ambas poblaciones tienden a un punto de equilibrio.

hold on
[t, y] = rungekutta('mosca', 0:0.05:21, [30 900]);
plot(y(:,1), y(:,2)/20, 'g')
[t, y] = rungekutta('mosca', 0:0.05:21, [30 100]);
plot(y(:,1), y(:,2)/20, 'k')
[t, y] = rungekutta('mosca', 0:0.05:21, [10 100]);
plot(y(:,1), y(:,2)/20, 'c')
[t, y] = rungekutta('mosca', 0:0.05:21, [10 800]);
plot(y(:,1), y(:,2)/20, 'r'), hold off


%%
%
% FIN del Problema 5.10