function [x, l, c, u] = crout(a, b, c, z)

% [x, l, c, u] = crout(a, b, c, z)
%
% Solucin del sistema 
%           Ax = z
% siendo A una matriz tridiagonal con diagonal principal a, superdiagonal b
% y subdiagonal c.
% Crout es la eliminacin gaussiana, escalando la fila para que el pivote
% sea uno.
% Como producto de la eliminacion, obtenemos una factorizacion de la matriz
% A = LU con 
% L bidiagonal inferior, con diagonal l y subdiagonal c; y
% U bidiagonal superior unidad, con superdiagonal u


% Eliminacin
n = length(a);
l(1) = a(1);
y(1) = z(1)/l(1);
for i = 2:n
	u(i-1) = b(i-1)/l(i-1);
	l(i) = a(i) - c(i-1)*u(i-1);
	y(i) = (z(i) - c(i-1)*y(i-1))/l(i);
end

% Sustitucin regresiva
x(n) = y(n);
for i = n-1: -1: 1
	x(i) = y(i) - u(i)*x(i+1);
end
