%% 6.1. Eliminacin de Gauss con pivotacin parcial. Contrabarra y comandos lu y rref.
% 
% Este problema muestra la resolucin paso a paso de un sistema lineal por
% el mtodo de eliminacin de Gauss con pivotacin parcial, operando por
% filas en notacin matricial.
% Se obtiene la misma solucin directamente con la contrabarra de MATLAB.
% Las operaciones elementales realizadas en el proceso proporcionan la
% factorizacin LU de la matriz del sistema.
% El comando rref aplica la eliminacin para hallar la matriz inversa de la
% matriz de coeficientes, as como la solucin del sistema.
%
%
% Tema 6, Problema 1
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson


%% 
%
% Definimos la matriz del sistema A y los trminos independientes b para
% formar la matriz ampliada M.

A = [1 2 -1; 2 -3 1; 1 0 4]
b = [-1 -6 2]'
M = [A b]
L = eye(3);

%% 
%
% El primer paso es eliminar en la primera columna. Situamos en la posicin
% del pivote el elemento de mayor valor absoluto de la columna.

disp(' '), disp(rats(M))
disp(' '), disp('Intercambiamos las filas primera y segunda:')
M = M([2 1 3],:);
disp(' '), disp(rats(M))


%% 
%
% Eliminamos ahora los elementos por debajo del pivote de la primera
% columna. Primero la segunda fila.

disp(' '), disp(rats(M))
disp(' '), disp('Restamos a la segunda fila un mltiplo adecuado de la primera:')
L(2,1) = M(2,1)/M(1,1);
M(2,:) = M(2,:) - L(2,1)*M(1,:);
disp(' '), disp(rats(M))


%% 
%
% Luego la tercera.

disp(' '), disp(rats(M))
disp(' '), disp('y restamos a la tercera fila un mltiplo adecuado de la primera:')
L(3,1) = M(3,1)/M(1,1);
M(3,:) = M(3,:) - L(3,1)*M(1,:);
disp(' '), disp(rats(M))


%%
%
% Para eliminar en la segunda columna no hace falta intercambiar filas,
% puesto que M(2, 2) es el mayor elemento de la columna.

disp(' '), disp(rats(M))
disp(' '), disp('Restamos a la tercera fila un mltiplo adecuado de la segunda:')
L(3,2) = M(3,2)/M(2,2);
M(3,:) = M(3,:) - L(3,2)*M(2,:);
disp(' '), disp(rats(M))


%%
%
% La matriz escalonada obtenida, nos indica que el rango es 3 y el sistema
% es compatible determinado. 
% Procedemos a obtener la solucin por sustitucin regresiva.

U = M(:, 1:3);
disp(' '), disp(rats(M))
disp(' '), disp('Despejamos la tercera incgnita:')
M(3,:) = M(3,:)/M(3,3);
disp(' '), disp(rats(M))
disp(' '), disp(['En notacin clsica, z = ',num2str(M(3,4))])

%%
%
% Sustituimos en las ecuaciones anteriores la incgnita otenida eliminando
% los coeficientes de la tercera columna.

disp(' '), disp(rats(M))
disp(' '), disp('Sustituimos en la primera ecuacin:')
M(1,:) = M(1,:) - M(3,:)*M(1,3);
disp(' '), disp(rats(M))


%%
%
% Sustituimos en las ecuaciones anteriores la incgnita otenida eliminando
% los coeficientes de la tercera columna.

disp(' '), disp(rats(M))
disp(' '), disp('y luego en la segunda ecuacin:')
M(2,:) = M(2,:) - M(3,:)*M(2,3);
disp(' '), disp(rats(M))


%%
%
% Despejamos ahora la segunda incgnita.

disp(' '), disp(rats(M))
disp(' '), disp('Dividimos la segunda ecuacin por el pivote:')
M(2,:) = M(2,:)/M(2,2);
disp(' '), disp(rats(M))
disp(' '), disp(['En notacin clsica, y = ',num2str(M(2,4))])


%%
%
% Sustituimos ahora la segunda incgnita en la primera ecuacin. 

disp(' '), disp(rats(M))
disp(' '), disp('Restamos a la primera ecuacin un mltiplo adecuado de la segunda:')
M(1,:) = M(1,:) - M(2,:)*M(1,2);
disp(' '), disp(rats(M))


%%
%
% Y, finalmente, despejamos la primera incgnita.

disp(' '), disp(rats(M))
disp(' '), disp('Dividimos la primera ecuacin por el pivote:')
M(1,:) = M(1,:)/M(1,1);
disp(' '), disp(rats(M))
disp(' '), disp(['En notacin clsica, x = ',num2str(M(1,4))])


%%
%
% Comprobamos que la solucin verifica el sistema inicial.

A*M(:,4)
b


%%
%
% Utilizando los comandos \ o rref de MATLAB, obtenemos el mismo resultado. 
 
disp(' ')
disp('A\b = '), disp(A\b)
disp(' ')
disp('rref([A,b]) = '), disp(rref([A,b]))

%%
%
% La matriz L registra las operaciones efectuadas. Multiplicando por U se
% recupera la matriz inicial (salvo permutacin).

A, L, U, L*U

%%
%
% La matriz de la permutacin es
 
P = eye(3);
P = P([2 1 3],:)


%%
%
% Se verifica P*A = L*U

P*A
L*U

%%
%
% Las matrices P, L y U se pueden obtener directamente con MATLAB

[Lm, Um, Pm] = lu(A)
L, U, P

%%
%
% La inversa de la matriz A se obtiene resolviendo tres sistemas cuyos
% trminos independientes son las columnas de la matriz identidad del mismo
% orden. 
% Los tres sistemas pueden resolverse simultneamente por eliminacin como
% hemos hecho antes, o bien, mediante los comandos \ o rref.

I = eye(3);
disp(' '), disp('A\I = '), disp(A\I)
disp(' '), disp('rref([A,I]) = '), disp(rref([A,I]))


%%
%
% FIN del Problema 6.1