%% 6.3. Discretizacin del problema del calor en una placa rectangular.
% 
% En este problema se formula el problema del calor en una placa en versin
% discreta y se resuelve el sistema lineal resultante por eliminacin y por
% mtodos iterativos.
%
%
% Tema 6, Problema 3
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson


%% 
%
% 
% El diagrama adjunto representa la discretizacin del problema del calor
% en una placa. Se trata de determinar la temperatura en los nodos
% interiores de la malla Tj , j = 1, 2, . . . , 6, conocidas las
% temperaturas en el borde y suponiendo que hay equilibrio trmico, es
% decir que las temperaturas no varan.

image(imread('malla.jpg'))
axis equal off
title('Discretizacin del problema del calor en una placa rectangular')

%%
%
% En el modelo discreto se supone que, en el equilibrio, la temperatura en
% cada nodo es la media de las temperaturas en los nodos vecinos. Aplicando
% esta condicin a cada nodo, obtenemos un sistema lineal ya preparado para
% aplicar un mtodo iterativo.
%
%       T1 = 1/4*(50 + 100 + T2 + T3)
%       T2 = 1/4*(50 + T1 + 0 + T4)
%       T3 = 1/4*(T1 + 50 + T4 + T5)
%       T4 = 1/4*(T2 + T3 + 50 + T6)
%       T5 = 1/4*(T3 + 100 + T6 + 50)
%       T6 = 1/4*(T4 + T5 + 0 + 50)


%%
%
% Escribimos el sistema en forma normal.
%
%       4T1   -T2   -T3                   = 150
%       -T1 + 4T2         -T4             =  50
%       -T1       + 4T3   -T4   -T5       =  50
%             -T2   -T3 + 4T4         -T6 =  50
%                   -T3       + 4T5   -T6 = 150
%                         -T4   -T5 + 4T6 =  50


%%
%
% Construimos la matriz del sistema por bloques, poniendo de manifiesto su
% estructura:

m4 = [ 4 -1
      -1  4];
  
m1 = [-1  0
       0 -1];
   
m0 = [0 0
      0 0];
  
A = [m4 m1 m0
     m1 m4 m1
     m0 m1 m4]
 
spy(A)
title('Estructura de la matriz del problema del calor')


%%
%
% Editamos la columna de trminos independientes y resolvemos el sistema.

b = [150 50 50 50 150 50]';
sol = A\b


%%
%
% Para representar grficamente la solucin, disponemos los resultados en
% una matriz que representa la forma de la malla y orlamos esta matriz con
% las temperaturas del borde.

P = reshape(sol, 2, 3)

M = [75 100 50 100 75];
M(2:3,:) = [[50; 50] P [50; 50]];
M(4,:) = [25 0 50 0 25]

imagesc(M)
colorbar
axis equal off
title('Solucin por un mtodo directo')

%%
%
% Mtodo de Jacobi.
% Partiendo de una estimacin inicial, por ejemplo Tj = 50 en todos los
% nodos, la sustituimos en las ecuaciones
%
%       T1(k) = 1/4*(50 + 100 + T2(k-1) + T3(k-1))
%       T2(k) = 1/4*(50 + T1(k-1) + 0 + T4(k-1))
%       T3(k) = 1/4*(T1(k-1) + 50 + T4(k-1) + T5(k-1))
%       T4(k) = 1/4*(T2(k-1) + T3(k-1) + 50 + T6(k-1))
%       T5(k) = 1/4*(T3(k-1) + 100 + T6(k-1) + 50)
%       T6(k) = 1/4*(T4(k-1) + T5(k-1) + 0 + 50)

[T1, T2, T3, T4, T5, T6] = deal(50);

M(2:3, 2:4) = [T1 T3 T5
               T2 T4 T6]
imagesc(M)
colorbar
axis equal off
title('Mtodo de Jacobi. Estimacin inicial')


%%
%
% Efectuando las operaciones indicadas obtenemos la primera iteracin de
% Jacobi:

k = 2;
T1(k) = 1/4*(50 + 100 + T2(k-1) + T3(k-1))
T2(k) = 1/4*(50 + T1(k-1) + 0 + T4(k-1))
T3(k) = 1/4*(T1(k-1) + 50 + T4(k-1) + T5(k-1))
T4(k) = 1/4*(T2(k-1) + T3(k-1) + 50 + T6(k-1))
T5(k) = 1/4*(T3(k-1) + 100 + T6(k-1) + 50)
T6(k) = 1/4*(T4(k-1) + T5(k-1) + 0 + 50)

M(2:3, 2:4) = [T1(k) T3(k) T5(k)
               T2(k) T4(k) T6(k)]
imagesc(M)
colorbar
axis equal off
title('Mtodo de Jacobi. Primera iteracin')


%%
%
% La segunda iteracin consiste en repetir los clculos, sustituyendo a la
% derecha los resultados de la primera.

k = 3;
T1(k) = 1/4*(50 + 100 + T2(k-1) + T3(k-1))
T2(k) = 1/4*(50 + T1(k-1) + 0 + T4(k-1))
T3(k) = 1/4*(T1(k-1) + 50 + T4(k-1) + T5(k-1))
T4(k) = 1/4*(T2(k-1) + T3(k-1) + 50 + T6(k-1))
T5(k) = 1/4*(T3(k-1) + 100 + T6(k-1) + 50)
T6(k) = 1/4*(T4(k-1) + T5(k-1) + 0 + 50)

M(2:3, 2:4) = [T1(k) T3(k) T5(k)
               T2(k) T4(k) T6(k)]
imagesc(M)
colorbar
axis equal off
title('Mtodo de Jacobi. Segunda iteracin')


%%
%
% Las iteraciones se repiten hasta que los resultados varan menos que una
% determinada tolerancia.

k = 4;
T1(k) = 1/4*(50 + 100 + T2(k-1) + T3(k-1))
T2(k) = 1/4*(50 + T1(k-1) + 0 + T4(k-1))
T3(k) = 1/4*(T1(k-1) + 50 + T4(k-1) + T5(k-1))
T4(k) = 1/4*(T2(k-1) + T3(k-1) + 50 + T6(k-1))
T5(k) = 1/4*(T3(k-1) + 100 + T6(k-1) + 50)
T6(k) = 1/4*(T4(k-1) + T5(k-1) + 0 + 50)

M(2:3, 2:4) = [T1(k) T3(k) T5(k)
               T2(k) T4(k) T6(k)]
imagesc(M)
colorbar
axis equal off
title('Mtodo de Jacobi. Tercera iteracin')


%%
%
% Las iteraciones se repiten hasta que los resultados varan menos que una
% determinada tolerancia.
% 

k = 5;
T1(k) = 1/4*(50 + 100 + T2(k-1) + T3(k-1))
T2(k) = 1/4*(50 + T1(k-1) + 0 + T4(k-1))
T3(k) = 1/4*(T1(k-1) + 50 + T4(k-1) + T5(k-1))
T4(k) = 1/4*(T2(k-1) + T3(k-1) + 50 + T6(k-1))
T5(k) = 1/4*(T3(k-1) + 100 + T6(k-1) + 50)
T6(k) = 1/4*(T4(k-1) + T5(k-1) + 0 + 50)

M(2:3, 2:4) = [T1(k) T3(k) T5(k)
               T2(k) T4(k) T6(k)]
imagesc(M)
colorbar
axis equal off
title('Mtodo de Jacobi. Cuarta iteracin')


%%
%
% Las iteraciones se repiten hasta que los resultados varan menos que una
% determinada tolerancia.
% 

k = 6;
T1(k) = 1/4*(50 + 100 + T2(k-1) + T3(k-1))
T2(k) = 1/4*(50 + T1(k-1) + 0 + T4(k-1))
T3(k) = 1/4*(T1(k-1) + 50 + T4(k-1) + T5(k-1))
T4(k) = 1/4*(T2(k-1) + T3(k-1) + 50 + T6(k-1))
T5(k) = 1/4*(T3(k-1) + 100 + T6(k-1) + 50)
T6(k) = 1/4*(T4(k-1) + T5(k-1) + 0 + 50)

M(2:3, 2:4) = [T1(k) T3(k) T5(k)
               T2(k) T4(k) T6(k)]
imagesc(M)
colorbar
axis equal off
title('Mtodo de Jacobi. Quinta iteracin')


%%
%
% Las iteraciones se repiten hasta que los resultados varan menos que una
% determinada tolerancia.
% 

k = 7;
T1(k) = 1/4*(50 + 100 + T2(k-1) + T3(k-1))
T2(k) = 1/4*(50 + T1(k-1) + 0 + T4(k-1))
T3(k) = 1/4*(T1(k-1) + 50 + T4(k-1) + T5(k-1))
T4(k) = 1/4*(T2(k-1) + T3(k-1) + 50 + T6(k-1))
T5(k) = 1/4*(T3(k-1) + 100 + T6(k-1) + 50)
T6(k) = 1/4*(T4(k-1) + T5(k-1) + 0 + 50)

M(2:3, 2:4) = [T1(k) T3(k) T5(k)
               T2(k) T4(k) T6(k)]
imagesc(M)
colorbar
axis equal off
title('Mtodo de Jacobi. Sexta iteracin')


%%
%
% Las iteraciones se repiten hasta que los resultados varan menos que una
% determinada tolerancia.
% 

k = 8;
T1(k) = 1/4*(50 + 100 + T2(k-1) + T3(k-1))
T2(k) = 1/4*(50 + T1(k-1) + 0 + T4(k-1))
T3(k) = 1/4*(T1(k-1) + 50 + T4(k-1) + T5(k-1))
T4(k) = 1/4*(T2(k-1) + T3(k-1) + 50 + T6(k-1))
T5(k) = 1/4*(T3(k-1) + 100 + T6(k-1) + 50)
T6(k) = 1/4*(T4(k-1) + T5(k-1) + 0 + 50)

M(2:3, 2:4) = [T1(k) T3(k) T5(k)
               T2(k) T4(k) T6(k)]
imagesc(M)
colorbar
axis equal off
title('Mtodo de Jacobi. Sptima iteracin')


%%
%
% Las iteraciones se repiten hasta que los resultados varan menos que una
% determinada tolerancia.
% 

k = 9;
T1(k) = 1/4*(50 + 100 + T2(k-1) + T3(k-1))
T2(k) = 1/4*(50 + T1(k-1) + 0 + T4(k-1))
T3(k) = 1/4*(T1(k-1) + 50 + T4(k-1) + T5(k-1))
T4(k) = 1/4*(T2(k-1) + T3(k-1) + 50 + T6(k-1))
T5(k) = 1/4*(T3(k-1) + 100 + T6(k-1) + 50)
T6(k) = 1/4*(T4(k-1) + T5(k-1) + 0 + 50)

M(2:3, 2:4) = [T1(k) T3(k) T5(k)
               T2(k) T4(k) T6(k)]
imagesc(M)
colorbar
axis equal off
title('Mtodo de Jacobi. Octava iteracin')


%%
%
% Las iteraciones se repiten hasta que los resultados varan menos que una
% determinada tolerancia.
% 

k = 10;
T1(k) = 1/4*(50 + 100 + T2(k-1) + T3(k-1))
T2(k) = 1/4*(50 + T1(k-1) + 0 + T4(k-1))
T3(k) = 1/4*(T1(k-1) + 50 + T4(k-1) + T5(k-1))
T4(k) = 1/4*(T2(k-1) + T3(k-1) + 50 + T6(k-1))
T5(k) = 1/4*(T3(k-1) + 100 + T6(k-1) + 50)
T6(k) = 1/4*(T4(k-1) + T5(k-1) + 0 + 50)

M(2:3, 2:4) = [T1(k) T3(k) T5(k)
               T2(k) T4(k) T6(k)]
imagesc(M)
colorbar
axis equal off
title('Mtodo de Jacobi. Novena iteracin')


%%
%
% Se observa que los valores calculados se estabilizan. Numricamente, tras
% 15 iteraciones, la diferencia mxima entre un iterado y el anterior es
% inferior a 0.01. 
% La diferencia mxima de la iteracin actual con la solucin es:

max(abs(sol - [T1(k); T2(k); T3(k); T4(k); T5(k); T6(k)]))

title('Mtodo de Jacobi')


%%
%
% El mtodo de Gauss-Seidel hace las mismas operaciones que el de Jacobi,
% pero utiliza los valores nuevos en cuanto estn disponibles. Las
% ecuaciones de la iteracin para el ejemplo de la placa son:
%
%       T1(k) = 1/4*(50 + 100 + T2(k-1) + T3(k-1))
%       T2(k) = 1/4*(50 + T1(k) + 0 + T4(k-1))
%       T3(k) = 1/4*(T1(k) + 50 + T4(k-1) + T5(k-1))
%       T4(k) = 1/4*(T2(k) + T3(k) + 50 + T6(k-1))
%       T5(k) = 1/4*(T3(k) + 100 + T6(k-1) + 50)
%       T6(k) = 1/4*(T4(k) + T5(k) + 0 + 50)

title('Mtodo de Gauss-Seidel')


%%
%
% Numricamente, Gauss-Seidel es muy fcil de aplicar: basta utilizar
% variables escalares en lugar de vectoriales, para que la actualizacin
% sea inmediata.
% Partimos de la misma estimacin inicial:

[T1, T2, T3, T4, T5, T6] = deal(50);

M(2:3, 2:4) = [T1 T3 T5
               T2 T4 T6]
imagesc(M)
colorbar
axis equal off
title('Mtodo de Gauss-Seidel. Estimacin inicial')


%%
%
% Como las ecuaciones se evalan sucesivamente y el valor del segundo
% miembro se asigna a una variable con el mismo nombre, en las ecuaciones
% siguientes se utiliza el valor actualizado.

T1 = 1/4*(50 + 100 + T2 + T3)
T2 = 1/4*(50 + T1 + 0 + T4)
T3 = 1/4*(T1 + 50 + T4 + T5)
T4 = 1/4*(T2 + T3 + 50 + T6)
T5 = 1/4*(T3 + 100 + T6 + 50)
T6 = 1/4*(T4 + T5 + 0 + 50)

M(2:3, 2:4) = [T1 T3 T5
               T2 T4 T6]
imagesc(M)
colorbar
axis equal off
title('Mtodo de Gauss-Seidel. Primera iteracin')


%%
%
% La segunda iteracin consiste en repetir los clculos, sustituyendo a la
% derecha los resultados de la primera.

T1 = 1/4*(50 + 100 + T2 + T3)
T2 = 1/4*(50 + T1 + 0 + T4)
T3 = 1/4*(T1 + 50 + T4 + T5)
T4 = 1/4*(T2 + T3 + 50 + T6)
T5 = 1/4*(T3 + 100 + T6 + 50)
T6 = 1/4*(T4 + T5 + 0 + 50)

M(2:3, 2:4) = [T1 T3 T5
               T2 T4 T6]
imagesc(M)
colorbar
axis equal off
title('Mtodo de Gauss-Seidel. Segunda iteracin')


%%
%
% Las iteraciones se repiten hasta que los resultados varan menos que una
% determinada tolerancia.

T1 = 1/4*(50 + 100 + T2 + T3)
T2 = 1/4*(50 + T1 + 0 + T4)
T3 = 1/4*(T1 + 50 + T4 + T5)
T4 = 1/4*(T2 + T3 + 50 + T6)
T5 = 1/4*(T3 + 100 + T6 + 50)
T6 = 1/4*(T4 + T5 + 0 + 50)

M(2:3, 2:4) = [T1 T3 T5
               T2 T4 T6]
imagesc(M)
colorbar
axis equal off
title('Mtodo de Gauss-Seidel. Tercera iteracin')


%%
%
% Las iteraciones se repiten hasta que los resultados varan menos que una
% determinada tolerancia.

T1 = 1/4*(50 + 100 + T2 + T3)
T2 = 1/4*(50 + T1 + 0 + T4)
T3 = 1/4*(T1 + 50 + T4 + T5)
T4 = 1/4*(T2 + T3 + 50 + T6)
T5 = 1/4*(T3 + 100 + T6 + 50)
T6 = 1/4*(T4 + T5 + 0 + 50)

M(2:3, 2:4) = [T1 T3 T5
               T2 T4 T6]
imagesc(M)
colorbar
axis equal off
title('Mtodo de Gauss-Seidel. Cuarta iteracin')


%%
%
% Las iteraciones se repiten hasta que los resultados varan menos que una
% determinada tolerancia.

T1 = 1/4*(50 + 100 + T2 + T3)
T2 = 1/4*(50 + T1 + 0 + T4)
T3 = 1/4*(T1 + 50 + T4 + T5)
T4 = 1/4*(T2 + T3 + 50 + T6)
T5 = 1/4*(T3 + 100 + T6 + 50)
T6 = 1/4*(T4 + T5 + 0 + 50)

M(2:3, 2:4) = [T1 T3 T5
               T2 T4 T6]
imagesc(M)
colorbar
axis equal off
title('Mtodo de Gauss-Seidel. Quinta iteracin')


%%
%
% Las iteraciones se repiten hasta que los resultados varan menos que una
% determinada tolerancia.

T1 = 1/4*(50 + 100 + T2 + T3)
T2 = 1/4*(50 + T1 + 0 + T4)
T3 = 1/4*(T1 + 50 + T4 + T5)
T4 = 1/4*(T2 + T3 + 50 + T6)
T5 = 1/4*(T3 + 100 + T6 + 50)
T6 = 1/4*(T4 + T5 + 0 + 50)

M(2:3, 2:4) = [T1 T3 T5
               T2 T4 T6]
imagesc(M)
colorbar
axis equal off
title('Mtodo de Gauss-Seidel. Sexta iteracin')


%%
%
% Las iteraciones se repiten hasta que los resultados varan menos que una
% determinada tolerancia.

T1 = 1/4*(50 + 100 + T2 + T3)
T2 = 1/4*(50 + T1 + 0 + T4)
T3 = 1/4*(T1 + 50 + T4 + T5)
T4 = 1/4*(T2 + T3 + 50 + T6)
T5 = 1/4*(T3 + 100 + T6 + 50)
T6 = 1/4*(T4 + T5 + 0 + 50)

M(2:3, 2:4) = [T1 T3 T5
               T2 T4 T6]
imagesc(M)
colorbar
axis equal off
title('Mtodo de Gauss-Seidel. Sptima iteracin')


%%
%
% Las iteraciones se repiten hasta que los resultados varan menos que una
% determinada tolerancia.

T1 = 1/4*(50 + 100 + T2 + T3)
T2 = 1/4*(50 + T1 + 0 + T4)
T3 = 1/4*(T1 + 50 + T4 + T5)
T4 = 1/4*(T2 + T3 + 50 + T6)
T5 = 1/4*(T3 + 100 + T6 + 50)
T6 = 1/4*(T4 + T5 + 0 + 50)

M(2:3, 2:4) = [T1 T3 T5
               T2 T4 T6]
imagesc(M)
colorbar
axis equal off
title('Mtodo de Gauss-Seidel. Octava iteracin')


%%
%
% Se observa que los valores calculados se estabilizan ms rpidamente que
% con el mtodo de Jacobi. En 8 iteraciones se obtiene un resultado que
% difiere del anterior en menos de 0.01.
% La diferencia mxima con la solucin es:

max(abs(sol - [T1; T2; T3; T4; T5; T6]))

title('Mtodo de Gauss-Seidel')


%%
%
% El mtodo de sobrerrelajacin combina la nueva estimacin de Gauss-Seidel
% con la anterior, mediante un factor w. Para el presente sistema, la
% iteracin de sobrerrelajacin es:
%
%       T1(k) = (1 - w)*T1(k-1) + w/4*(50 + 100 + T2(k-1) + T3(k-1))
%       T2(k) = (1 - w)*T2(k-1) + w/4*(50 + T1(k) + 0 + T4(k-1))
%       T3(k) = (1 - w)*T3(k-1) + w/4*(T1(k) + 50 + T4(k-1) + T5(k-1))
%       T4(k) = (1 - w)*T4(k-1) + w/4*(T2(k) + T3(k) + 50 + T6(k-1))
%       T5(k) = (1 - w)*T5(k-1) + w/4*(T3(k) + 100 + T6(k-1) + 50)
%       T6(k) = (1 - w)*T6(k-1) + w/4*(T4(k) + T5(k) + 0 + 50)


title('Mtodo de Sobrerrelajacin')


%%
%
% El factor de relajacin w ha de estar entre 0 y 2 para que haya
% convergencia. Eligiendo un valor adecuado se consigue alcanzarla con
% menos iteraciones que Gauss-Seidel. Tomamos w = 1.15 y la misma
% estimacin inicial que en los mtodos anteriores.

w = 1.15;

[T1, T2, T3, T4, T5, T6] = deal(50);

M(2:3, 2:4) = [T1 T3 T5
               T2 T4 T6]
imagesc(M)
colorbar
axis equal off
title('Mtodo de Sobrerrelajacin, w = 1.15. Estimacin inicial')


%%
%
% Efectuamos la primera iteracin de Sobrerrelajacin. En los clculos no
% hace falta utilizar subndices para registrar los valores de cada paso,
% pues siempre se emplea el ltimo valor obtenido.

T1 = (1 - w)*T1 + w/4*(50 + 100 + T2 + T3)
T2 = (1 - w)*T2 + w/4*(50 + T1 + 0 + T4)
T3 = (1 - w)*T3 + w/4*(T1 + 50 + T4 + T5)
T4 = (1 - w)*T4 + w/4*(T2 + T3 + 50 + T6)
T5 = (1 - w)*T5 + w/4*(T3 + 100 + T6 + 50)
T6 = (1 - w)*T6 + w/4*(T4 + T5 + 0 + 50)

M(2:3, 2:4) = [T1 T3 T5
               T2 T4 T6]
imagesc(M)
colorbar
axis equal off
title('Mtodo de Sobrerrelajacin, w = 1.15. Primera iteracin')


%%
%
% El mtodo converge ms rpidamente que los anteriores.

T1 = (1 - w)*T1 + w/4*(50 + 100 + T2 + T3)
T2 = (1 - w)*T2 + w/4*(50 + T1 + 0 + T4)
T3 = (1 - w)*T3 + w/4*(T1 + 50 + T4 + T5)
T4 = (1 - w)*T4 + w/4*(T2 + T3 + 50 + T6)
T5 = (1 - w)*T5 + w/4*(T3 + 100 + T6 + 50)
T6 = (1 - w)*T6 + w/4*(T4 + T5 + 0 + 50)

M(2:3, 2:4) = [T1 T3 T5
               T2 T4 T6]
imagesc(M)
colorbar
axis equal off
title('Mtodo de Sobrerrelajacin, w = 1.15. Primera iteracin')


%%
%
% El mtodo converge ms rpidamente que los anteriores.

T1 = (1 - w)*T1 + w/4*(50 + 100 + T2 + T3)
T2 = (1 - w)*T2 + w/4*(50 + T1 + 0 + T4)
T3 = (1 - w)*T3 + w/4*(T1 + 50 + T4 + T5)
T4 = (1 - w)*T4 + w/4*(T2 + T3 + 50 + T6)
T5 = (1 - w)*T5 + w/4*(T3 + 100 + T6 + 50)
T6 = (1 - w)*T6 + w/4*(T4 + T5 + 0 + 50)

M(2:3, 2:4) = [T1 T3 T5
               T2 T4 T6]
imagesc(M)
colorbar
axis equal off
title('Mtodo de Sobrerrelajacin, w = 1.15. Segunda iteracin')


%%
%
% El mtodo converge ms rpidamente que los anteriores.

T1 = (1 - w)*T1 + w/4*(50 + 100 + T2 + T3)
T2 = (1 - w)*T2 + w/4*(50 + T1 + 0 + T4)
T3 = (1 - w)*T3 + w/4*(T1 + 50 + T4 + T5)
T4 = (1 - w)*T4 + w/4*(T2 + T3 + 50 + T6)
T5 = (1 - w)*T5 + w/4*(T3 + 100 + T6 + 50)
T6 = (1 - w)*T6 + w/4*(T4 + T5 + 0 + 50)

M(2:3, 2:4) = [T1 T3 T5
               T2 T4 T6]
imagesc(M)
colorbar
axis equal off
title('Mtodo de Sobrerrelajacin, w = 1.15. Tercera iteracin')


%%
%
% El mtodo converge ms rpidamente que los anteriores.

T1 = (1 - w)*T1 + w/4*(50 + 100 + T2 + T3)
T2 = (1 - w)*T2 + w/4*(50 + T1 + 0 + T4)
T3 = (1 - w)*T3 + w/4*(T1 + 50 + T4 + T5)
T4 = (1 - w)*T4 + w/4*(T2 + T3 + 50 + T6)
T5 = (1 - w)*T5 + w/4*(T3 + 100 + T6 + 50)
T6 = (1 - w)*T6 + w/4*(T4 + T5 + 0 + 50)

M(2:3, 2:4) = [T1 T3 T5
               T2 T4 T6]
imagesc(M)
colorbar
axis equal off
title('Mtodo de Sobrerrelajacin, w = 1.15. Cuarta iteracin')


%%
%
% El mtodo converge ms rpidamente que los anteriores.

T1 = (1 - w)*T1 + w/4*(50 + 100 + T2 + T3)
T2 = (1 - w)*T2 + w/4*(50 + T1 + 0 + T4)
T3 = (1 - w)*T3 + w/4*(T1 + 50 + T4 + T5)
T4 = (1 - w)*T4 + w/4*(T2 + T3 + 50 + T6)
T5 = (1 - w)*T5 + w/4*(T3 + 100 + T6 + 50)
T6 = (1 - w)*T6 + w/4*(T4 + T5 + 0 + 50)

M(2:3, 2:4) = [T1 T3 T5
               T2 T4 T6]
imagesc(M)
colorbar
axis equal off
title('Mtodo de Sobrerrelajacin, w = 1.15. Quinta iteracin')


%%
%
% El mtodo converge ms rpidamente que los anteriores. 

T1 = (1 - w)*T1 + w/4*(50 + 100 + T2 + T3)
T2 = (1 - w)*T2 + w/4*(50 + T1 + 0 + T4)
T3 = (1 - w)*T3 + w/4*(T1 + 50 + T4 + T5)
T4 = (1 - w)*T4 + w/4*(T2 + T3 + 50 + T6)
T5 = (1 - w)*T5 + w/4*(T3 + 100 + T6 + 50)
T6 = (1 - w)*T6 + w/4*(T4 + T5 + 0 + 50)

M(2:3, 2:4) = [T1 T3 T5
               T2 T4 T6]
imagesc(M)
colorbar
axis equal off
title('Mtodo de Sobrerrelajacin, w = 1.15. Sexta iteracin')

%%
%
% En la sexta iteracin la variacin mxima respecto a la iteracin
% anterior es menor de una centsima.
% La diferencia mxima con la solucin es:

max(abs(sol - [T1; T2; T3; T4; T5; T6]))

title('Mtodo de Sobrerrelajacin, w = 1.15')

%%
%
% FIN del Problema 6.3