%% 6.4. Red de servidores. Estado estacionario.
% 
% En este problema se analiza el trfico de mensajes en una red y se
% resuelve la ecuacin del estado estacionario por mtodos directos e
% iterativos.
%
%
% Tema 6, Problema 4
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson


%% 
%
% 
% La matriz pedida se denomina matriz de transicin de estados. El elemento
% de la fila i y columna j de la matriz M es la probabilidad de pasar del
% estado j al estado i, en nuestro caso, de que un mensaje que se encuentra
% en el servidor j, al segundo siguiente est en el servidor i. La matriz M
% para un bus de 5 servidores es:

M = [0.5 0.3 0   0   0
     0.5 0.2 0.3 0   0
     0   0.5 0.2 0.3 0
     0   0   0.5 0.2 0.3
     0   0   0   0.5 0.7];
 
imagesc(M)
axis equal off
title('Matriz de transicin de estados')


%%
%
% Partiendo de una distribucin inicial cualquiera y multiplicando 50 veces por M, se obtiene un resultado
% aproximado de (5.6, 9.4, 15.6, 26, 43.4).
% Estos porcentajes no varan sensiblemente tomando otra distribucin
% inicial.

x = rand(5,1);
x = x/sum(x)*100;
bar(x,1)
title('Distribucin inicial')
colormap(winter)

%%
%
% Partiendo de una distribucin inicial cualquiera y multiplicando 50 veces por M, se obtiene un resultado
% aproximado de (5.6, 9.4, 15.6, 26, 43.4).
% Estos porcentajes no varan sensiblemente tomando otra distribucin
% inicial.

for k = 1:50
    x = M*x;
    bar(x, 1)
    set(gca, 'YLim', [0 45], 'YGrid', 'on')
    title(['Distribucin en el instante ', num2str(k)])
    pause(1/1.2^k)
end
disp(['Distribucin final: ', num2str(x')])

%%
%
% La matriz del sistema (M - I)x = 0 tiene las filas dependientes porque
% suman 0, por tanto, es singular.

A = eye(5) - M

rank(A)


%%
%
% Como todas las ecuaciones intervienen en la relacin de dependencia, se
% puede sustituir cualquiera de ellas, por ejemplo la ltima, por la
% condicin adicional de que las incgnitas sumen 100, quedando un sistema
% compatible determinado.

A(5,:) = [1 1 1 1 1];
rank(A)
b = zeros(5,1);
b(5) = 100
sol = A\b

bar([x sol])
title('Distribucin estacionaria de mensajes en servidores')
legend('Iterando 50 veces', 'Resolviendo ecuacin de equilibrio', ...
    'Location', 'Best')


%%
%
% Para el caso de 20 servidores la matriz del sistema es:

n = 20;
dp = ones(1,n);
ds = ones(1,n-1);
M = diag(0.2*dp) + diag(0.3*ds, 1) + diag(0.5*ds, -1);
M(1,1) = 1 - M(2,1);
M(n,n) = 1 - M(n-1,n);

spy(M)
title('Estructura de la matriz para n = 20')


%%
%
% El sistema a resolver ser:

A = eye(n) - M
A(n,:) = ones(1,n);
b = zeros(n,1);
b(n) = 100


%%
%
% Aplicando el mtodo de Jacobi con tolerancia 10^-4 a partir de la
% estimacin inicial nula, se obtiene la solucin en 163 iteraciones. 

x0 = zeros(20,1);
[x, incr, iter] = jacobi(A, b, x0, 1e-4, 200)

bar(x, 1)
title('Distribucin en 20 servidores. Solucin por Jacobi')
xlabel([num2str(iter), ' iteraciones'])
set(gca, 'YGrid', 'on', 'XLim', [0 21])


%%
%
% El mtodo de Gauss-Seidel es ms rpido, convergiendo en 86 iteraciones.

[x, incr, iter] = gseidel(A, b, x0, 1e-4, 200)

bar(x, 1)
title('Distribucin en 20 servidores. Solucin por Gauss-Seidel')
xlabel([num2str(iter), ' iteraciones'])
set(gca, 'YGrid', 'on', 'XLim', [0 21])


%%
%
% Con un factor de relajacin w = 1.3 el mtodo de sobrerrelajacin
% converge en 48 iteraciones.

[x, incr, iter] = sobrerrel(A, b, x0, 1.3, 1e-4, 200)

bar(x, 1)
title('Distribucin en 20 servidores. Solucin por Sobrerrelajacin con w = 1.3')
xlabel([num2str(iter), ' iteraciones'])
set(gca, 'YGrid', 'on', 'XLim', [0 21])


%%
%
% FIN del Problema 6.4