%% 6.6. Circuito elctrico. Factorizacin LU.
% 
% En este problema se plantea un sistema lineal que relaciona las
% magnitudes de un circuito elctrico y se resuelve para determinar las
% incgnitas en funcin de los datos.
% Tambin se muestra la utilidad de la factorizacin LU para resolver un
% sistema repetidas veces, camiando slo los trminos independientes.
%
%
% Tema 6, Problema 6
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson


%% 
%
% La ley de Ohm para un circuito establece una relacin entre la fuerza
% electromotriz del generador y la intensidad que pasa por el circuito.
% Para redes ms complicadas, debemos aplicar tambin las leyes de
% Kirchhoff:
% 1. Ley de los nudos: La suma algebraica de las intensidades que concurren
% en un nudo es cero.
% 2. Ley de las mallas: La suma de las fuerzas electromotrices es igual a
% las cadas de potencial para toda malla de la red.

circuito = imread('circuito.jpg');
imagesc(circuito)
axis equal off
title('Circuito')


%%
%
% Aplicando las leyes de Kirchhoff a la red se obtiene el siguiente sistema
% de ecuaciones lineales:
% 
%               I1 - I6 + I8 = 0
%               I3 - I2 - I1 = 0
%               I2 + I4 - I5 = 0
%         -I3 + I5 + I6 + I7 = 0
%      I1*R1 + I3*R3 + I6*R6 = E1 + E2 - E3
%     -I2*R2 - I5*R5 - I3*R3 = -E2
%      I4*R4 - I7*R7 + I5*R5 = E4 - E5
%     -I6*R6 + I7*R7 - I8*R8 = E3 + E5


%%
%
% Escribimos la expresin matricial del sistema para los valores de las
% resistencias y fuerzas electromotrices dadas.

A = [1  0  0  0  0 -1  0  1
    -1 -1  1  0  0  0  0  0
     0  1  0  1 -1  0  0  0
     0  0 -1  0  1  1  1  0
     1  0  1  0  0  2  0  0
     0 -1 -1  0 -2  0  0  0
     0  0  0  1  2  0  2  0
     0  0  0  0  0 -2  2 -2];

 b = [0
      0
      0
      0
      3
     -3
     -2
      7];
  
spy(A)
title('Estructura de la matriz A del sistema')


%%
%
% Hallamos la factorizacin LU de A con:

[L,U,P] = lu(A)

spy(L)
title('Estructura del factor triangular inferior L')


%%
%
% Los factores obtenidos cumplen P*A = L*U;

P*A

L*U

spy(U)
title('Estructura del factor triangular superior U')


%%
%
% El sistema inicial Ax = b se puede escribir, previa permutacin de filas,
% como  LUx = PAx = Pb
% Para obtener la solucin debemos resolver los sistemas triangulares
% Ly = Pb, Ux = y
% La funcin factlu.m resuelve estos sistemas, el primero por sustitucin
% directa y el segundo por sustitucin regresiva.

type factlu


%%
%
% Obtenemos la solucin resolviendo el sistema factorizado con la funcin
% factlu. El vector I contiene las intensidades del circuito. El signo
% negativo de alguna intensidad significa que la corriente circula en
% sentido contario al asignado inicialmente.

[y, I] = factlu(A, b, L, U, P);

disp('Intensidades')
disp(I)

imagesc(circuito)
axis equal off
title('Circuito')


%%
%
% Para resolver el problema variando slo los trminos independientes, nos
% ahorramos las operaciones de eliminacin y pasamos directamente a
% resolver los sistemas triangulares.

c = ones(8,1);
[y, I] = factlu(A, c, L, U, P);

disp('Intensidades')
disp(I)


%%
%
% Notemos que no era necesario crear las funciones que resuelven los
% sistemas triangulares puesto que el comando \ de MATLAB tambin tiene en
% cuenta este hecho.

I = A\c;

disp('Intensidades')
disp(I')


%%
%
% FIN del Problema 6.6