%% 6.7. Mtodos iterativos y directos. Matrices dispersas.
% 
% En este problema se analiza la convergencia del mtodo de Jacobi para el
% sistema dado en funcin de los valores relativos de los parmetros y el
% coste temporal de los mtodos directos actuando sobre sistemas de gran
% tamao, segn se considere la matriz llena o dispersa.
%
%
% Tema 6, Problema 7
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson


%% 
%
% La matriz A del sistema se construye por diagonales, en funcin de a, b y n.

n = 20; a = 2; b = 1;
v = ones(1,n-1);
A = a*eye(n) + b*diag(v,1) + b*diag(v,-1);

imagesc(A)
axis equal off
title('Representacin de la matriz del sistema para n = 100, a = 2, b = 1.')


%% 
%
% Iterando el sistema por Jacobi para distintos valores de n, a y b, se
% comprueba que las iteraciones convergen si |a| > 2|b|. 
%
% Damos valores aleatorios a a y b y representamos el punto (a, b) de color
% rojo si las iteraciones divergen y de azul, si convergen.
%
% Cerca de las rectas a = 2b y a = -2b, pueden aparecer puntos verdes
% correspondientes a valores de a y b  para los que habra que realizar ms
% iteraciones para decidir si hay convergencia o no.

convjac(100, 20);


%%
%
% Tomamos n = 100, a = 2 y variamos b de 0 a 1. Aplicamos Jacobi a partir
% de la estimacin inicial nula, iterando hasta que la diferencia en norma
% supremo sea menor que 10-3. Se observa que el nmero de iteraciones
% necesarias crece al aumentar b. Primero el crecimiento es lento, para
% luego aumentar rpidamente.


x0 = zeros(n,1);
c = zeros(n,1);
c(1) = 1;
for k = 1:9
    b(k) = 0.1*(k-1);
    A = a*eye(n) + b(k)*diag(v,1) + b(k)*diag(v,-1);
    [x, incr, iter(k)] = jacobi(A, c, x0, 0.001, 1000);
end
for k = 10:19
    b(k) = 0.8 + 0.02*(k-9);
    A = a*eye(n) + b(k)*diag(v,1) + b(k)*diag(v,-1);
    [x, incr, iter(k)] = jacobi(A, c, x0, 0.001, 1000);
end
for k = 20:24
    b(k) = 0.98 + 0.004*(k-19);
    A = a*eye(n) + b(k)*diag(v,1) + b(k)*diag(v,-1);
    [x, incr, iter(k)] = jacobi(A, c, x0, 0.001, 1000);
end

stem(b, iter, 'filled')
title('Convergencia del mtodo de Jacobi en funcin de b')
xlabel('b')
ylabel('Iteraciones')
set(gca, 'YGrid', 'on')


%%
%
% Tratamos de comparar los tiempos de ejecucin, al resolver por distintos
% mtodos sistemas de tamao cada vez mayor. Estos datos pueden variar
% segn la versin del programa o las caractersticas del ordenador, aunque
% la dependencia del tamao del sistema debe ser parecida. Tambin pueden
% aparecer valores anormalmente altos, debido a peculiaridades del programa.
% Resolvemos en primer lugar medante la contrabarra de MATLAB.

a = 2; b = 1;
for k = 1:20
    n(k) = 50*k; 
    v = ones(1, n(k)-1);
    A = a*eye(n(k)) + b*diag(v, 1) + b*diag(v, -1);
    z = zeros(n(k), 1); z(1) = 1;
    tic
    x=A\z;
    tnormal(k)=toc;
end
tnormal

plot(n, tnormal, '.-')
title('Tiempo de resolucin mediante la contrabarra')
xlabel('Tamao del sistema')
ylabel('Tiempo')
grid

%%
%
% Utilizamos ahora el algoritmo de Crout, que hace uso de la
% tridiagonalidad de la matriz para ahorrar operaciones en la resolucin
% del sistema.

for k = 1:20
    n(k) = 50*k; 
    v = b*ones(1, n(k)-1);
    A = a*eye(n(k)) + diag(v, 1) + diag(v, -1);
    z = zeros(n(k), 1); z(1) = 1;
    a1 = diag(A); b1 = diag(A,1); c1 = diag(A,-1);
    tic
    x = crout(a1, b1, c1, z);
    tcrout(k) = toc;
end
tcrout

plot(n, tcrout, 'r.-')
title('Tiempo de resolucin mediante el algoritmo de Crout')
xlabel('Tamao del sistema')
ylabel('Tiempo')
grid    

%%
%
% La contrabarra de MATLAB que efecta la eliminacin gaussiana, tarda
% menos que el algoritmo de Crout para sistemas pequeos, pero el coste del
% primer mtodo se incrementa cuadrticamente, mientras que el de Crout
% crece linealmente, por lo que es ms adecuado cuanto mayor es el sistema.

plot(n, tnormal, '.-', n, tcrout, 'r.-')
title('Tiempo de resolucin de sistemas grandes')
xlabel('Tamao del sistema')
ylabel('Tiempo')
legend('Contrabarra', 'Algoritmo de Crout')
grid


%%
%
% Usando matrices dispersas, MATLAB ahorra en operaciones y almacenamiento.
% logrando unos tiempos muy inferiores a los obtenidos con matrices llenas.

for k = 1:20
    n(k) = 50*k; 
    v = ones(1, n(k)-1);
    A = a*eye(n(k)) + b*diag(v, 1) + b*diag(v, -1);
    z = zeros(n(k), 1); z(1) = 1;
    A = sparse(A);
    z = sparse(z);
    tic
    x = A\z;
    tdispersa(k) = toc;
end
tdispersa

plot(n, tdispersa, 'g.-')
title('Tiempo de resolucin usando matrices dispersas')
xlabel('Tamao del sistema')
ylabel('Tiempo')
grid


%%
%
% Utilizando matrices
% dispersas, MATLAB consigue tiempos de ejecucin dos rdenes de magnitud
% inferiores a los del algoritmo de Crout. Adems, las matrices dispersas
% suponen un gran ahorro de memoria, al almacenar slo los elementos no
% nulos de la matriz.

plot(n, tcrout, 'r.-', n, tdispersa, 'g.-')
title('Tiempo de resolucin de sistemas grandes')
xlabel('Tamao del sistema')
ylabel('Tiempo')
legend('Algoritmo de Crout', 'Matriz dispersa')
grid


%%
%
% FIN del Problema 6.7