%% 6.8. Ecuacin de Laplace en un rectngulo.
% 
% En este problema se discretiza la ecuacin de Laplace en un dominio
% rectangular y se resuelve el sistema lineal resultante por mtodos
% directos e iterativos.
%
%
% Tema 6, Problema 8
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson


%% 
%
% Consideramos la ecuacin de Laplace en el rectngulo [0, a][0, b],
% siendo a = (n+1)h y b = (m+1)h con n y m naturales. Superponemos al
% dominio [0, a] [0, b] una malla formada por cuadrados de lado h.
% Obtendremos la solucin aproximada en los nodos de la malla.

image(imread('malla.jpg'))
axis equal off
title('Discretizacin del dominio rectangular')


%%
%
% Para construir el sistema de ecuaciones, ordenamos las incgnitas u(i,j)
% de arriba abajo y de izquierda a derecha, tal como indica la figura.Con
% esta ordenacin, la variable ui,j queda en la posicin k = (m - j + 1) +
% m(i - 1). 



%%
%
% Construiremos la matriz del sistema mediante la funcin calor2D.

type calor2D


%%
%
% Para el caso m = 4, n = 3, la matriz est formada por 3  3 bloques de
% tamao 4  4. Los bloques diagonales corresponden a las relaciones entre
% nodos de la misma columna y los bloques contiguos a la diagonal a las
% relaciones entre los nodos de una columna y los de la anterior o
% posterior.

n = 3; m = 4; 
A = calor2D(n,m)

spy(A)
title('Estructura de la matriz A para una malla 43')
set(gca, 'XTick', 1:12, 'YTick', 1:12)
hold on
for k = 0 : m : m*n
    plot([0.5 12.5], [k + 0.5, k + 0.5], 'r')
    plot([k + 0.5, k + 0.5], [0.5 12.5], 'r')
end
hold off

%%
%
% Al discretizar la ecuacin de Laplace en el rectngulo [0, 1]  [0, 1]
% con h = 0,1, se obtiene una malla de 11  11 nodos, de los cuales 9  9
% son interiores. El sistema correspondiente se obtiene con

n = 9; m = 9;
A = calor2D(n,m);

spy(A)
title('Estructura de la matriz A para una malla 99')
set(gca, 'XTick', 9:9:81, 'YTick', 9:9:81)
hold on
for k = 0 : m : m*n
    plot([0.5, m*n + 0.5], [k + 0.5, k + 0.5], 'r')
    plot([k + 0.5, k + 0.5], [0.5, m*n + 0.5], 'r')
end
hold off


%%
%

% Las condiciones de frontera
%       u(x, 0) = u(x, 1) = 0 
%       u(0, y) = u(1, y) = y(1 - y)
% dan lugar a trminos independientes

% Trminos independientes
p = m*n;
b = zeros(p,1);
% Condiciones de frontera
h = 1/(n+1);
y = h:h:1-h;
uy = y.*(1-y);
uy = flipud(uy');
b(1:m) = b(1:m) - uy; % Lado izquierdo
b(p-m+1:p) = b(p-m+1:p) - uy; % Lado derecho


%%
%
% Resolvemos el sistema y disponemos las soluciones en una matriz de las
% dimensiones de la malla.

% Solucin
sol = A\b;
Tsol = reshape(sol,m,n)


%%
%
% Para obtener la representacin grfica, aadimos las condiciones de
% contorno que dan los valores de u en los nodos del borde.

M = zeros(m+2,n+2);
M(2:m+1,1) = uy;
M(2:m+1,n+2) = uy;
M(2:m+1,2:n+1) = Tsol;

imagesc(M)
colorbar
axis equal off
title('Representacin de la solucin en la malla')


%%
%
% Resolvemos ahora el sistema por el mtodo de Jacobi. Tomamos como
% estimacin inicial u(0)i,j = u(0, yj) = yj(1 - yj).

u0 = uy*ones(1,n);
u0 = u0(:);

T = reshape(u0, m, n)
M(2:m+1, 2:n+1) = T;
imagesc(M)
colorbar
axis equal off
title('Mtodo de Jacobi. Estimacin inicial')


%%
%
% Iteramos hasta que la diferencia entre dos estimaciones consecutivas de u
% sea menor que 10^-3 en cada nodo.

tol = 1e-3;
iter = 0;
incr = tol +1;
u = u0;
while incr > tol & iter < 100
    [u, incr, k] = jacobi(A, b, u, tol, 1);
    iter = iter + 1;
    T = reshape(u, m, n)
    M(2:m+1,2:n+1) = T;
    imagesc(M)
    colorbar
    axis equal off
    title(['Mtodo de Jacobi. Paso ', num2str(iter)])
    pause(1/1.2^iter) 
end


%%
%
% La convergencia del mtodo de Jacobi es muy lenta, por lo que la
% diferencia entre la iteracin final y la solucin obtenida mediante la
% contrabarra es mayor que la tolerancia requerida.

disp(' ')
disp(['Nmero de iteraciones: ' num2str(iter)])
disp(' ')
disp(['Diferencia con la solucin exacta: ', num2str(norm(u-sol, inf))])


%%
%
% Iteramos en segundo lugar por el mtodo de Gauss-Seidel partiendo de la
% misma estimacin inicial.

u0 = uy*ones(1,n);
u0 = u0(:);

T = reshape(u0, m, n)
M(2:m+1, 2:n+1) = T;
imagesc(M)
colorbar
axis equal off
title('Mtodo de Gauss-Seidel. Estimacin inicial')


%%
%
% Iteramos hasta que la diferencia entre dos estimaciones consecutivas de u
% sea menor que 10^-3 en cada nodo.

tol = 1e-3;
iter = 0;
incr = tol +1;
u = u0;
while incr > tol & iter < 100
    [u, incr, k] = gseidel(A, b, u, tol, 1);
    iter = iter + 1;
    T = reshape(u, m, n)
    M(2:m+1,2:n+1) = T;
    imagesc(M)
    colorbar
    axis equal off
    title(['Mtodo de Gauss-Seidel. Paso ', num2str(iter)])
    pause(1/1.2^iter) 
end


%%
%
% La convergencia del mtodo de Gauss-Seidel es algo ms rpida que la del
% mtodo de Jacobi, pero, an asi, la diferencia entre la iteracin final y
% la solucin obtenida mediante la contrabarra supera la tolerancia
% requerida.

disp(' ')
disp(['Nmero de iteraciones: ' num2str(iter)])
disp(' ')
disp(['Diferencia con la solucin exacta: ', num2str(norm(u-sol, inf))])


%%
%
% Finalmente utilizamos el mtodo de sobrerrelajacin.

u0 = uy*ones(1,n);
u0 = u0(:);

T = reshape(u0, m, n)
M(2:m+1, 2:n+1) = T;
imagesc(M)
colorbar
axis equal off
title('Mtodo de sobrerralajacin. Estimacin inicial')


%%
%
% Iteramos con un valor de w apropiado hasta que la diferencia entre dos
% estimaciones consecutivas de u sea menor que 10^-3 en cada nodo.

w = 1.6;   % Factor de relajacin
tol = 1e-3;
iter = 0;
incr = tol +1;
u = u0;
while incr > tol & iter < 100
    [u, incr, k] = sobrerrel(A, b, u, w, tol, 1);
    iter = iter + 1;
    T = reshape(u, m, n)
    M(2:m+1,2:n+1) = T;
    imagesc(M)
    colorbar
    axis equal off
    title(['Mtodo de sobrerralajacin. Paso ', num2str(iter)])
    pause(1/1.2^iter)
end


%%
%
% El mtodo de sobrerrelajacin converge ms rpidamente que los anteriores
% y logra que diferencia entre la iteracin final y la solucin obtenida
% mediante la contrabarra sea del mismo orden que la tolerancia requerida.

disp(' ')
disp(['Nmero de iteraciones: ' num2str(iter)])
disp(' ')
disp(['Diferencia con la solucin exacta: ', num2str(norm(u-sol, inf))])


%%
%
% FIN del Problema 6.8