%% 6.10. Ecuacin de Laplace en coordenadas polares.
% 
% En este problema se discretiza la ecuacin de Laplace en un recinto
% circular utilizando coordenadas polares y se resuelve el sistema lineal
% resultante por mtodos directos e iterativos.
%
%
% Tema 6, Problema 10
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson


%% 
%
% Consideramos la ecuacin de Laplace en polares en el disco de centro el
% origen y radio 1 con la condicin de frontera
% u(1,z) = sen z,   0 <= z <= 2pi.
% Discretizamos el problema considerando una malla formada por los puntos
% de coordenadas polares (ri, zj) = (ih, jk), i = 0, 1, ... , n+1, j = 1,
% ... ,m, siendo h = 1/(n+1) y k = 2pi/m.

image(imread('mallacirc.jpg'))
axis equal off
title('Coordenadas polares de los nodos del disco')


%%
% Dividimos el radio en n+1 = 3 partes y la circunferencia en m = 5 sectores. 
% Consideramos slo los nodos de radio 0 < r < R = 1 y ngulo 0 < z <= 2pi. 
% Generamos la matriz del sistema mediante el programa polapmat. La matriz
% A est formada por 22 bloques de tamao 55.

n = 2; m = 5; p = n*m;
A = polapmat(m,n)

spy(A)
title('Estructura de la matriz A para una malla 25')
set(gca, 'XTick', 1:10, 'YTick', 1:10)
hold on
for k = 0 : m : p
    plot([0.5, m*n + 0.5], [k + 0.5, k + 0.5], 'r')
    plot([k + 0.5, k + 0.5], [0.5, m*n + 0.5], 'r')
end
hold off


%%
%
% Los trminos independientes aparecen en las ecuaciones correspondientes a
% los nodos de la primera y la ltima columnas.

R = 1;
h = R/(n+1);
r = h:h:1-h;
k = 2*pi/m;
z = k:k:2*pi;
b = zeros(p,1);
u1 = sin(z);
u1 = flipud(u1');
u0 = mean(u1);
b(p-m+1:p) = -k^2*(1+h/2/r(n))*u1; % Frontera
b(1:m) = -k^2*(1-h/2/r(1))*u0


%%
%
% Resolvemos el sistema y disponemos los resultados segn la malla.

sol = A\b;
Tsol = reshape(sol,m,n)


%%
%
% Representamos grficamente la solucin, situando los valores obtenidos en
% el disco y aadiendo los valores en la frontera.

rg = 1 : -h : 0;
zg = 2*pi : -k : 0;
[R, Z] = meshgrid(rg, zg);
X = R.*cos(Z);
Y = R.*sin(Z);
T = [u1 fliplr(Tsol) u0*ones(m,1)];
T = [T; T(1,:)];
pcolor(X,Y,T)
colorbar
axis equal off
title('Solucin de la ecuacin de Laplace en un disco')
xlabel('Valor en la frontera: sen z')


%%
%
% Calculamos la diferencia con la solucin exacta, que es u = r sen z.

[R,Z] = meshgrid(r,fliplr(z));
solex = R.*sin(Z);
max(max(abs(Tsol-solex)))


%%
%
% Consideramos ahora la ecuacin anterior con la condicin de frontera 
% u(1,z) = sen 2z. Obtenemos la solucin discretizando con ms nodos,
% dividiendo el radio en 5 partes y la circunferencia en 16.

n = 4; m = 16; p =n*m;
A = polapmat(m,n);

spy(A, 5)
title('Estructura de la matriz A para una malla 416')
set(gca, 'XTick', 8:8:64, 'YTick', 8:8:64)
hold on
for k = 0 : m : p
    plot([0.5, m*n + 0.5], [k + 0.5, k + 0.5], 'r')
    plot([k + 0.5, k + 0.5], [0.5, m*n + 0.5], 'r')
end
hold off


%%
%
% Los trminos independientes aparecen en las ecuaciones correspondientes a
% los nodos de la primera y la ltima columnas.

R = 1;
h = R/(n+1);
r = h:h:1-h;
k = 2*pi/m;
z = k:k:2*pi;
b = zeros(p,1);
u1 = sin(2*z);
u1 = flipud(u1');
u0 = mean(u1);
b(p-m+1:p) = -k^2*(1+h/2/r(n))*u1; % Frontera
b(1:m) = -k^2*(1-h/2/r(1))*u0;


%%
%
% Resolvemos el sistema y disponemos los resultados segn la malla.

sol = A\b;
Tsol = reshape(sol,m,n)


%%
%
% Representamos grficamente la solucin, situando los valores obtenidos en
% el disco y aadiendo los valores en la frontera.

rg = 1 : -h : 0;
zg = 2*pi : -k : 0;
[R, Z] = meshgrid(rg, zg);
X = R.*cos(Z);
Y = R.*sin(Z);
T = [u1 fliplr(Tsol) u0*ones(m,1)];
T = [T; T(1,:)];
pcolor(X,Y,T)
colorbar
axis equal off
title('Solucin de la ecuacin de Laplace en un disco')
xlabel('Valor en la frontera: sen 2z')


%%
%
% La representacin en 3D nos da idea de cmo vara la solucin segn el
% ngulo y el radio.

surf(X,Y,T)
view(35,30)
title('Solucin de la ecuacin de Laplace en un disco')
xlabel('x')
ylabel('y')


%%
%
% Calculamos la diferencia con la solucin exacta, que es u = r^2 sen 2z.

[R,Z] = meshgrid(r,fliplr(z));
solex = R.^2.*sin(2*Z);
max(max(abs(Tsol-solex)))


%%
%
% Resolvemos ahora el sistema por mtodos iterativos, tomando como
% estimacin inicial 0 en todos los nodos incgnita e iteramos hasta que la
% diferencia entre dos estimaciones en cada nodo sea menor que 10^-3.

u = zeros(p,1);
tol = 1e-3;

T(1:m, 2:n+1) = zeros(m, n)
pcolor(X, Y, T)
colorbar
axis equal off
title('Mtodos iterativos. Estimacin inicial')

    
%%
%
% Iteramos en primer lugar por el mtodo de Jacobi.

iter = 0;
incr = tol +1;
while incr > tol & iter < 100
    [u, incr, k] = jacobi(A, b, u, tol, 1);
    iter = iter + 1;
    Titer = reshape(u, m, n);
    T(1:m, 2:n+1) = fliplr(Titer);
    T(m+1,:) = T(1,:);
    pcolor(X, Y, T)
    colorbar
    axis equal off
    title(['Mtodo de Jacobi. Paso ', num2str(iter)])
    pause(1/1.2^iter) 
end


%%
%
% El mtodo de Jacobi converge en 17 iteraciones, con tolerancia 10^-3. La
% diferencia entre la iteracin final y la solucin exacta de este problema
% es del orden de centsimas.

disp(' ')
disp(['Nmero de iteraciones: ' num2str(iter)])
disp(' ')
disp(['Diferencia con la solucin discreta: ', num2str(max(max(abs(Titer-Tsol))))])
disp(' ')
disp(['Diferencia con la solucin exacta: ', num2str(max(max(abs(Titer-solex))))])


%%
%
% En segundo lugar, iteramos por el mtodo de Gauss-Seidel.

iter = 0;
incr = tol +1;
u = zeros(p,1);
while incr > tol & iter < 100
    [u, incr, k] = gseidel(A, b, u, tol, 1);
    iter = iter + 1;
    Titer = reshape(u, m, n);
    T(1:m, 2:n+1) = fliplr(Titer);
    T(m+1,:) = T(1,:);
    pcolor(X, Y, T)
    colorbar
    axis equal off
    title(['Mtodo de Gauss-Seidel. Paso ', num2str(iter)])
    pause(1/1.2^iter) 
end


%%
%
% El mtodo de Gauss-Seidel es algo ms rpido que el de Jacobi,
% necesitando slo 14 iteraciones. La diferencia entre la iteracin final y
% la solucin exacta es del mismo orden que en el caso anterior, aunque
% algo mayor.

disp(' ')
disp(['Nmero de iteraciones: ' num2str(iter)])
disp(' ')
disp(['Diferencia con la solucin discreta: ', num2str(max(max(abs(Titer-Tsol))))])
disp(' ')
disp(['Diferencia con la solucin exacta: ', num2str(max(max(abs(Titer-solex))))])


%%
%
% Resolvemos ahora el sistema por sobrerrelajacin, con factor w = 1.4.

w = 1.4;
iter = 0;
incr = tol +1;
u = zeros(p,1);
while incr > tol & iter < 100
    [u, incr, k] = sobrerrel(A, b, u, w, tol, 1);
    iter = iter + 1;
    Titer = reshape(u, m, n);
    T(1:m, 2:n+1) = fliplr(Titer);
    T(m+1,:) = T(1,:);
    pcolor(X, Y, T)
    colorbar
    axis equal off
    title(['Mtodo de sobrerrelajacin. Paso ', num2str(iter)])
    pause(1/1.2^iter) 
end


%%
%
% En este ejemplo, el mtodo de sobrerrelajacin no supone una mejora
% respecto a los anteriores. Los mtodos iterativos utilizados aproximan
% las solucin del sistema lineal procedente de la discretizacin con
% precisin de milsimas. El sistema discretizado aproxima la solucin
% exacta con error de centsimas, por lo que los mtodos iterativos
% obtienen la misma precisin respecto a la solucin exacta que el mtodo
% directo, pero realizan menos operaciones.

disp(' ')
disp(['Nmero de iteraciones: ' num2str(iter)])
disp(' ')
disp(['Diferencia con la solucin discreta: ', num2str(max(max(abs(Titer-Tsol))))])
disp(' ')
disp(['Diferencia con la solucin exacta: ', num2str(max(max(abs(Titer-solex))))])


%%
%
% FIN del Problema 6.10