%% 7.1. Tiempo de ejecucin de un programa. Polinomio mnimo-cuadrtico.
% 
% En este problema se obtiene por diversos procedimientos el polinomio que
% ajusta por mnimos cuadrados determinados datos.
%
%
% Tema 7, Problema 1
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson


%% 
%
% Los tiempos t de ejecucin de un programa de n datos son:

n = [5 10 15 20 25 30 35 40]';
t = [4 10 17 35 42 67 71 85]';

plot(n, t, '*')
title('Tiempos de ejecucin de un programa')
xlabel('Tamao del problema')
ylabel('Tiempo')
grid


%%
%
% Al imponer que un polinomio p de grado 2 pase por todos los puntos dados,
% obtenemos un sistema de ecuaciones lineales, A*p = t, cuya matriz es:

A = [n.^2 n n.^0]


%%
%
% El sistema es sobredeterminado, tiene muchas ms ecuaciones que
% incgnitas y con gran probabilidad ser incompatible. En este caso, el
% rango de A es 3 y el de la matriz ampliada, 4.

rank(A)

rank([A t])


%%
%
% El polinomio que minimiza el error cuadrtico es la solucin del sistema
% de ecuaciones normales A'*A*p = A'*t.

N = A'*A

c = A'*t


%%
%
% Resolviendo el sistema N*p = c obtenemos el polinomio mnimo-cuadrtico.

p = N\c

x = linspace(5,40);
y = polyval(p,x);
plot(n, t, '*', x, y)
title('Polinomio mnimo-cuadrtico de grado 2')
xlabel('Tamao del problema')
ylabel('Tiempo')
grid


%%
%
% El nmero de condicin del sistema es bastante elevado para tratarse de
% un sistema 33, lo que indica que es mal condicionado.

cond(N)


%%
%
% Intentamos mejorar la estabilidad del sistema mediante un desplazamiento
% de los datos.

nd = n - mean(n);
Ad = [nd.^2 nd ones(size(nd))];
Nd = Ad'*Ad;
cd = Ad'*t;
ncond = cond(Nd)


%%
%
% Resovemos el sistema, que est mejor condicionado que el inicial y
% obtenemos los coeficientes del polinomio mnimo cuadratico expresado como
% una combinacin lineal de potencias desplazadas a la media de las
% abscisas. El polinomio tiene distintos coeficientes, pero es el mismo que
% antes.

pd = Nd\cd

y = polyval(pd, x - mean(n));


%%
%
% Al usar el comando \ para un sistema sobredeterminado, MATLAB da la
% solucin por mnimos cuadrados.

p = A\t


%%
%
% Ms comodo todava es utilizar la orden polyfit, pues no hace falta
% construir la matriz del sistema.

p = polyfit(n,t,2)


%%
%
% Del mismo modo, obtenemos el polinomio referido a los datos desplazados
% con la contrabarra.

pd =Ad\t


%%
%
% O con polyfit:

pd = polyfit(nd, t, 2)


%%
%
% Calculamos el error cuadrtico para los polinomios mnimo-cuadrtico de
% grado 0 a 7. Podemos observar que el error cuadrtico se va reduciendo al
% aumentar el grado del polinomio, hasta llegar al polinomio que pasa por
% todos los puntos, cuyo error es nulo.

for i = 0:7
    p = polyfit(nd, t, i);
    E(i+1) = sum((polyval(p, nd) - t).^2)
end

bar([0:7], E)
title('Aproximacin por mnimos cuadrados')
xlabel('Grado del polinomio')
ylabel('Error cuadrtico')


%%
%
% FIN del Problema 7.1