%% 7.2. Estimacin del paro
% 
% En este problema se ajustan los datos del paro mediante la recta de
% regresin y polinomios mnimo-cuadrticos de distinto grado. Se muestra
% la utilidad del ndice de determinacin para determina el grado del
% ajuste y se revisan distintas opciones de la orden polyfit.
%
%
% Tema 7, Problema 2
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson


%% 
%
% Introducimos los datos en vectores columna. Los valores del nmero de
% parados son en millones.

x = (1991:2001)';
y = [2.362373; 2.337472; 2.471228; 2.774579; 2.575873; 2.427015; ...
2.262721; 2.067830; 1.783940; 1.659820; 1.598920];

plot(x, y, '*')
title('Datos del paro en febrero de 1991 a 2001')
xlabel('Ao')
ylabel('Millones de parados')
grid


%%
%
% Generamos las ecuaciones normales de la recta de regresin y resolvemos
% el sistema.

m = length(x);
N = [sum(x.^2), sum(x); sum(x), m];
c = [sum(x.*y); sum(y)];
p = N\c

yp = polyval(p,x);
plot(x, y, '*', x, yp)
title('Recta de regresin')
xlabel('Ao')
ylabel('Millones de parados')
legend('Datos', 'Ajuste lineal')
grid


%%
%
% Para determinar el ajuste mnimo-cuadrtico de grado 3, construimos
% primero la matriz del sistema A, obtenemos las ecuaciones normales,
% multiplicando A y c por la transpuesta de A y resolvemos el sistema
% resultante.

A = [x.^3, x.^2, x, x.^0];
N = A'*A;
c = A'*y;
p = N\c

xg = linspace(1991, 2001);
yg = polyval(p, xg);
plot(x, y, '*', xg, yg)
title('Ajuste mnimo cuadrtico')
xlabel('Ao')
ylabel('Millones de parados')
legend('Datos', 'Polinomio de grado 3')
grid


%%
%
% El nmero de condicin del sistema es muy elevado. El sistema es casi
% singular. Si desplazamos el origen a la media de los datos, el
% condicionamiento mejora y el resultado es ms preciso.

xn = x - mean(x);
A = [xn.^3, xn.^2, xn, xn.^0];
N = A'*A;
c = A'*y;
pd = N\c

ygd = polyval(pd, xg - mean(x));
plot(x, y, '*', xg, yg, xg, ygd)
title('Polinomio mnimo cuadrtico de grado 3')
xlabel('Ao')
ylabel('Millones de parados')
legend('Datos', 'Abscisas originales', 'Abscisas desplazadas')
grid


%%
%
% Se aprecia que al ajustar con las abscisas iniciales se pierde el trmino
% de grado 3, debido a la diferencia relativa de los valores de los
% coeficientes del polinomio.


%%
%
% Para calcular el error cuadrtico utilizamos el polinomio desplazado,
% puesto que es ms estable.

yp = polyval(pd, x-mean(x));
E2 = norm(y-yp)^2


%%
%
% El ndice de determinacin se obtiene, al comparar las desviaciones de
% las ordenadas calculadas y de las ordenadas de los datos respecto a su
% media. Un valor del ndice cercano a 1 nos indica que se ha obtenido un
% buen ajuste.

ym = mean(y);
I = (norm(yp-ym)/norm(y-ym))^2


%%
%
% La evaluacin del polinomio en 2002 (previo desplazamiento) estima el
% nmero de parados de ese ao.

valor_estimado = polyval(pd, 2002 - mean(x))

xp = linspace(1991, 2002);
yp = polyval(pd, xp - mean(x));
plot(x, y, '*', xg, yg, xp, yp, 2002, valor_estimado, '*')
title('Extrapolacin')
xlabel('Ao')
ylabel('Millones de parados')
legend('Datos', 'Abscisas originales', 'Abscisas dsplazadas', 'Estimacin en 2002')
grid

%%
%
% Repetimos los clculos con polyfit. Primero con las abscisas originales:
% MATLAB avisa de la necesidad de centrar los datos, aunque el resultado es
% ms preciso que el obtenido a partir de las ecuaciones normales.
% Evaluamos el polinomio para comparar con los otros ajustes.

p = polyfit(x,y,3)
yg = polyval(p,xg);

yg = polyval(p, xg);
plot(x, y, '*', xg, yg)
title('Ajuste mnimo-cuadrtico con polyfit')
xlabel('Ao')
ylabel('Millones de parados')
legend('Datos', 'Polinomio de grado 3')
grid


%%
%
% Los coeficientes del polinomio ajustado tienen escalas muy diferentes: de
% 10^-4 a 10^4. Centrando las abscisas a la media, evitamos el aviso de
% inestabilidad y obtenemos un polinomio cuyos coeficentes estn mejor
% escalados:

p = polyfit(xn, y, 3)

ygn = polyval(p, xg -mean(x));
plot(x, y, '*', xg, ygn)
title('Ajuste con polyfit y desplazamiento de origen')
xlabel('Ao')
ylabel('Millones de parados')
legend('Datos', 'Polinomio de grado 3')
grid


%%
%
% La sintaxis ms compleja de polyfit normaliza las abscisas,
% proporcionando el polinomio con coeficientes ms equilibrados.

[p, s, mu] = polyfit(x, y, 3)

xgu = (xg-mu(1))/mu(2);         % mu = [media, desviacin tpica]
ygu = polyval(p, xgu);
plot(x, y, '*', xg, ygu)
title('Ajuste con polyfit normalizando las abscisas')
xlabel('Ao')
ylabel('Millones de parados')
legend('Datos', 'Polinomio de grado 3')
grid


%%
%
% Los valores de los tres ajustes obtenidos con polyfit son suficientemente
% aproximados para no ser grficamente distinguibles. La diferencia entre
% los dos ltimos es del orden de 10-16 y la de stos con el primero, del
% orden de 10-8.

plot(x, y, '*', xg, yg, xg, ygn, '+', xg, ygu, '.')
max(abs(ygn-yg))
max(abs(ygn-ygu))
title('Ajustes con polyfit de grado 3')
xlabel('Ao')
ylabel('Millones de parados')
legend('Datos', 'Abscisas originales', 'Centradas a la media', 'Normalizadas')
grid

%%
%
% FIN del Problema 7.2