%% 7.3. Difusin de una epidemia
% 
% En este problema se ajusta un polinomio, cuyo grado se determina a base
% del ndice de determinacin, al nmero de afectados por una enfermedad
% contagiosa y se comprueba que es menos eficiente que el ajuste por un
% modelo de tipo exponencial.
%
%
% Tema 7, Problema 3
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson


%% 
%
% Representamos los datos de incidencia de la efermedad.

t = (0 : 7)';
c = [1 7.34 51.82 287.66 748.99 956.61 993.89 999.17]';

plot(t, c, '*')
title('Incidencia de la enfermedad')
xlabel('Da')
ylabel('Nmero de contagiados')
grid


%%
%
% Ajustamos un polinomio de tercer grado, desplazando las abscisas.

td = t - mean(t);
p = polyfit(td, c, 3)

tg = linspace(0, 7);
cg = polyval(p, tg - mean(t));
plot(t, c, '*', tg, cg)
title('Polinomio mnimo cuadrtico de grado 3')
xlabel('Da')
ylabel('Nmero de contagiados')
grid


%%
%
% El ndice de determinacin del ajuste de grado 3 no llega a 0.99

cp = polyval(p, td);
cm = mean(c);
indice = (norm(cp-cm)/norm(c-cm))^2


%%
%
% Ajustamos ahora por un polinomio de grado 4.

p = polyfit(td, c, 4)

cg = polyval(p, tg - mean(t));
plot(t, c, '*', tg, cg)
title('Polinomio mnimo cuadrtico de grado 4')
xlabel('Da')
ylabel('Nmero de contagiados')
grid


%%
%
% Comprobamos que para grado 4 el ndice de determinacin an no toma el
% valor pedido.

cp = polyval(p, td);
indice = (norm(cp-cm)/norm(c-cm))^2


%% 
%
% Construimos pues el polinomio de grado 5:

p = polyfit(td, c, 5)

cg = polyval(p, tg - mean(t));
plot(t, c, '*', tg, cg)
title('Polinomio mnimo cuadrtico de grado 5')
xlabel('Da')
ylabel('Nmero de contagiados')
grid


%%
%
% Y comprobamos que el polinomio pedido es el de grado 5, pues es el
% primero con ndice de determinacin mayor que 0.99.

cp = polyval(p, td);
indice = (norm(cp-cm)/norm(c-cm))^2


%%
%
% El error cuadrtico es:

E2 = norm(cp-c)^2


%%
%
% El ajuste no parece adecuado para obtener el nmero de contagiados en ms
% de una semana de tiempo, ya que a diferencia de la tendencia de los datos
% de la tabla a estabilizarse, el polinomio obtenido crece indefinidamente.

tg = linspace(0, 8);
cg = polyval(p, tg - mean(t));

plot(t, c, '*', tg, cg, 8, cg(end), '*')
title('Prediccin mediante el polinomio de grado 5')
xlabel('Da')
ylabel('Nmero de contagiados')
grid


%%
%
% Realizamos el ajuste de tipo logstico segn el modelo propuesto y
% representamos la funcin obtenida.

cn = 1./c;
A = [ones(size(t)) exp(-2*t)]
N = A'*A;
d = A'*cn;
p = N\d

yg = 1./(p(1) + p(2)*exp(-2*tg));
plot(t, c, '*', tg, yg, 8, yg(end), '*')
title('Prediccin logstica')
xlabel('Da')
ylabel('Nmero de contagiados')
grid


%%
%
% Ahora el error cuadrtico es bastante menor y el ajuste tiende a
% estabilizarse.

y = 1./(p(1) + p(2)*exp(-2*t));
E2 = norm(y - c)^2
c10 = 1./(p(1) + p(2)*exp(-2*8))


%%
%
% FIN del Problema 7.3