%% 7.5. Aproximacin trigonomtrica
% 
% En este problema se aproxima una funcin peridica mediante polinomios
% trigonomtricos y se analiza la convergencia de los mismos.
%
%
% Tema 7, Problema 5
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson


%% 
%
% La corriente producida por un inversor en un periodo viene dada por la
% funcin signo.

t = linspace(-pi, pi, 200)';
y = sign(t);

plot(t, y)
title('Coriente salida del inversor')
axis([-4 4 -1.5 1.5])


%%
%
% Calculamos el mltiplo de la funcin seno cuyo error cuadrtico respecto
% a la funcin dada, medido en 200 puntos equiespaciados en [-pi, pi], es
% mnimo, resolviendo las ecuaciones normales.

A = sin(t);
N = A'*A;
yt = A'*y;
b = N\yt    % Coeficiente en sen(t)
yp = A*b;

plot(t, y, t, yp)
title('Aproximacin de orden 1')


%%
%
% El error cuadrtico medio de la aproximacin es

E2 = 1/200*norm(yp-y)^2


%%
%
% Aproximamos ahora la funcin signo mediante una combinacin lineal de
% sen(t) y  sen(3t), por el mismo procedimiento. Notemos que el coeficiente
% en sen(t) no ha cambiado.

A = [sin(t) sin(3*t)];
N = A'*A;
yt = A'*y;
b = N\yt   
yp = A*b;

plot(t, y, t, yp)
title('Aproximacin de orden 3')


%%
%
% Construyo un vector con los sucesivos errores cuadrticos medios, que,
% obviamente, van decreciendo.

E2(2) = 1/200*norm(yp-y)^2


%%
%
% Anlogamente se obtiene la aproximacin de orden 5.

A = [sin(t) sin(3*t) sin(5*t)];
N = A'*A;
yt = A'*y;
b = N\yt   
yp = A*b;

plot(t, y, t, yp)
title('Aproximacin de orden 5')


%%
%
% Aado el error cuadrtico medio de la aproximacin de orden 5.

E2(3) = 1/200*norm(yp-y)^2


%%
%
% Finalmente, obtenemos la aproximacin de orden 7.

A = [sin(t) sin(3*t) sin(5*t) sin(7*t)];
N = A'*A;
yt = A'*y;
b = N\yt   
yp = A*b;

plot(t, y, t, yp)
title('Aproximacin de orden 7')


%%
%
% As como su error cuadrtico medio.

E2(4) = 1/200*norm(yp-y)^2


%%
%
% El error cuadrtico disminuye porque las aproximaciones son cada vez
% mejores.

bar(1:2:7, E2)
title('Aproximacin trigonomtrica')
xlabel('Orden')
ylabel('Error cuadrtico medio')


%%
%
% La energa total de la funcin signo es E = 2pi y la de los armnicos se
% calcula con:

F = pi*b.^2

bar(1:2:7, F)
title('Aproximacin trigonomtrica')
xlabel('Armnico')
ylabel('Energa del armnico')


%%
%
% Observamos que la mayor parte de la energa se concentra en el primer
% armnico. Hallamos la energa de las sumas parciales de los cuatro
% primeros armnicos y la diferencia de stas con E.

dist = 2*pi - cumsum(F)

bar(1:2:7, dist)
title('Aproximacin trigonomtrica')
xlabel('Armnico')
ylabel('Energa restante')


%%
%
% Estas diferencias miden la aproximacin a la funcin de las sumas
% parciales del desarrollo de Fourier, coincidiendo apoximadamente con
% 2*pi*E2.
% En este ejemplo, la convergencia es lenta debido a la discontinuidad de
% la funcin signo.

dist' - 2*pi*E2


%%
%
% FIN del Problema 7.5