%% 7.8. Control de calidad
% 
% En este problema ajustamos por mnimmos cuadrados dos distribuciones
% distintas a los errores en la medida de las piezas producidas por una
% mquina.
%
%
% Tema 7, Problema 8
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson


%% 
%
% Los errores de la medida de ciertas piezas se distribuyen segn:

x = (-1:0.2:1)';
y = [4 4 11 10 15 18 16 8 9 2 3]';

bar(x, y)
title('Control de calidad')
xlabel('Error')
ylabel('Frecuencia')


%%
%
% Suponiendo que los errores tienen una distribucin de tipo
%           y = 1/(a*x*x + b)
% hemos de determinar los parmetros a y b.
% Invirtiendo ambos miembros se obtiene un modelo lineal
%           z = a*x*x + b

z = 1./y;


%%
%
% Imponiendo que los datos satisfagan el modelo, obtenemos un sistema de
% matriz

A = [x.^2 x.^0]


%%
%
% Resolvemos el sistema directamente mediante la contrabarra de MATLAB. Al
% tratarse de un sistema incompatible, se obtiene la solucin
% mnimo-cuadrtica.

p = A\z

xg = linspace(-1, 1);
zr = p(1)*xg.^2 + p(2);
plot(x, y, '*', xg, 1./zr)
title('y = 1/(ax^2 + b)')
xlabel('Error')
ylabel('Frecuencia')
grid


%%
%
% Hallamos el error cuadrtico en trminos de los datos originales:

zp = A*p;
E2 = norm(y - 1./zp)^2


%%
%
% Ajustamos ahora el modelo y = k exp(-ax^2)). Linealizamos el modelo
% tomando logaritmos.

z = log(y);


%%
%
% Usando los datos, construimos un sistema de ecuaciones, lo resolvemos
% como antes y representamos grficamente el resultado junto al ajuste
% anterior.

A = [x.^0 -x.^2]; 
p = A\z

zg = p(1) - p(2)*xg.^2;
plot(x, y, '*', xg, 1./zr, xg, exp(zg))
legend('Datos', '1/(a + bx^2)', 'K exp(-ax^2)')
title('Control de calidad')
xlabel('Error')
ylabel('Frecuencia')
grid


%%
%
% Hallamos el error cuadrtico en trminos de los datos originales,
% quitando logaritmos:

zp = A*p;
E2 = norm(y - exp(zp))^2


%%
%
% El menor error cuadrtico obtenido en el primer caso indica que el ajuste
% con la funcin racional es mejor que con la exponencial.


%% 
%
% FIN del Problema 7.8