%% 7.10.  Densidad del aire.
%
% En este problema se realizan ajustes polinmicos a unos datos hasta
% conseguir un valor dado para el indice de determinacin. No obstante la
% aproximacin hallada no es satisfactoria procediendose a efectuar una
% aproximacin logaritmca que produce resultados ms aceptables.
%
%
% Tema 7, Problema 10.
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson


%% 
%
% Almacenamos los datos en dos vectores, h para las alturas y r para las
% densidades. Utilizamos polyfit para hallar la aproximacin mnimo
% cuadrtica de primer grado, y realizamos la representacin grfica.
h = [7 10 15 21 27 34 39 43 47 51 55 59 61];
r = [556 369 191 75 26.2 9.9 4.4 2.3 1.4 0.8 0.5 0.33 0.25];
j = 1;
p = polyfit(h,r,j);
hg = linspace(5,65);
rg = polyval(p,hg);
plot(h,r,'*',hg,rg)
%%
%
% Hallamos la media aritmtica de las densidades y evaluamos el polinomio
% obtenido para hallar el error cuadrtico, y el ndice de
% determinacin.
rm = mean(r);
rp = polyval(p,h);
E(1) = norm(r-rp)^2
I(1) = (norm(rp-rm)/norm(r-rm))^2
%%
%
% Repetimos los clculos para hallar las aproximaciones de grados 2,3 y 4.
% Como es de esperar el error cuadrtico va disminuyendo y el ndice de
% determinacin aproximndose a 1. El primer ajuste con ndice de
% determinacin mayor que 0.999 es el de grado 4. Realizamos la
% representacin grfica en [30,65].

hg = linspace(30,65);
while I(end) < 0.999
    j = j+1;
    p = polyfit(h,r,j);
    rp = polyval(p,h);
    rg = polyval(p,hg);  % para la representacin grfica
    subplot(2,2,j-1)
    plot(h(6:end),r(6:end),'*',hg,rg)
    hold on 
    E(j) = norm(r-rp)^2;
    I(j) = (norm(rp-rm)/norm(r-rm))^2;
end 
hold off
E,I, clear E I
%%
%
% En las grficas se observa que, para valores grandes de h, el polinomio
% oscila, tomando incluso valores negativos, lo que carece totalmente de
% sentido fsico. Procedemos a realizar la aproximacin logartmica.
% El ajuste de grado 2 tiene el ndice requerido. El polinomio obtenido es
% q(h) = 0.000369125h^2 - 0.171046h + 7.61761
lr = log(r);
lrm = mean(lr);
j = 0; 
I(1) = 0;
while I(end) < 0.999
    j = j+1;
    q = polyfit(h,lr,j);
    lrq = polyval(q,h);  
    E(j) = norm(lr-lrq)^2;
    I(j) = (norm(lrq-lrm)/norm(lr-lrm))^2;   
end 
q
%%
%
% Obtenemos ahora el error cuadrtico respecto a los datos iniciales,
% aunque ste es mayor que el del ajuste polinmico, la aproximacin
% logartmica es ms coherente pues la funcin resultante es decreciente
% respecto a h y no se obtienen valores negativos.
% Realizamos en la ltima grfica la aproximacin exponencial para r, 
% a partir de la aproximacin logartmica.
rq = exp(lrq) ; 
E2= norm(r-rq)^2
lrq = polyval(q,hg);
rqg = exp(lrq); 
subplot(2,2,4)
plot(h(6:end),r(6:end),'*',hg,rqg)
hold off
%%
%
% FIN del Problema 7.10