%% 8.1. Cortes entre una elipse y una parbola
% 
% Dado un sistema no lineal de ecuaciones, se aproxima grficamente su
% solucin y se aplica el mtodo de punto fijo para estimar dicha solucin.
% 
% Tema 8, Problema 1
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson


%% 
%
% a) Estimacin grfica de la solucin
%
% Representamos grficamente las curvas y1=x^2-2x+0.5,
% y2=sqrt(1-1/4*x^2), e  y3=-sqrt(1-1/4*x^2);
% Podemos observar que el sistema tiene dos soluciones  alrededor de los
% puntos (-0.3,1) y (1.8,0.3). 

ezplot('-y+x.^2-2*x+0.5')
hold on
ezplot('y^2+x.^2/4-1')
Xlim([-3 3])
Ylim([-3 3])
title('Curvas del sistema')
hold off


%%
%
% b) Aplicacin del mtodo de punto fijo
%
%  Utilizaremos el sistema de punto fijo equivalente:
%       x = 1/2*(x^2-y+0.5) 
%       y = x^2+4*y^2-4+y
% que almacenamos en el fichero funcion1.m
%
% function y = funcion1(x)
% y(1) = 0.5*(x(1).^2-x(2)+0.5);
% y(2) = x(1).^2+4*x(2).^2-4+x(2);
%
% Llamamos a la funcin puntofijo.m y observamos que el mtodo es
% divergente.

 [sol,iter,A] = puntofijo('funcion1',[-0.3,1],1e-7,50)

 
%%
%
% Utilizando el sistema de punto fijo:
% x = 1/2(x^2-y+0.5) 
% y = 1/8(-x^2-4y^2+4+8y)
%
% Almacenamos en funcion2.m el nuevo sistema y llamamos de nuevo a la
% funcin puntofijo.m

 [sol,iter,A] = puntofijo('funcion2',[-0.3,1],1e-7,50)

 
%%
% En este caso la sucesin converge a la solucin que est cerca del punto
% inicial (-0.3,1).
%
% Utilizando otra estimacin inicial observamos que el mtodo converge a la
% misma solucin.

 [sol,iter,A] = puntofijo('funcion2',[1.8,0.3],1e-7,50)

 
%%
% Para obtener la grfica de las coordenadas de los distintos iterados 
% basta representar los vectores columna de A,

plot(A)


%%
% mientras que para dibujar la tasa de convergencia lineal del mtodo:

[sol,iter,A] = puntofijo('funcion2',[-0.3,1],1e-7,50);
for i=1:iter-1
       conv(i)=norm(A(i+1,:)-A(i,:));
end
tcl = conv(2:end)./conv(1:end-1);
plot(tcl)
    

%%
% Para obtener una aproximacin de la segunda solucin del sistema
% obtenemos el sistema equivalente:
%
%       x = 1/2*(-x^2+4*x+y-0.5) 
%       y = 1/11*(-x^2-4y^2+4+11y)
%
% que almacenamos en funcion3.m.

 [sol,iter,A] = puntofijo('funcion3',[1.8,0.3],1e-7,50)


%%
%
% c) Transformamos el sistema no lineal en una sla ecuacin:
%
% 4*x^4-16*x^3+21*x^2-8*x-3 = 0
%
% Representamos grficamente el polinomio de grado 4 
 
x = -2:.1:2;
y = 4*x.^4-16*x.^3+21*x.^2-8*x-3;
plot(x,y)
grid on


%%
% Observamos que tiene dos races reales, una alrededor de x=-0.2 y cerca
% de x=1.8.
% 
% A continuacin almacenamos la ecuacin de punto fijo 
% x = 1/8*(4*x^4-16*x^3+21*x^2-3)
%
% en el fichero de funcion 'g.m' y ejecutamos el mtodo unidimensional
% 'pfijo.m':

 [sol,x,incr,iter] = pfijo('g',1.8,1e-7,50)


%%
% Con distintas aproximaciones, vemos que no converge.

 [sol,x,incr,iter] = pfijo('g',-0.3,1e-7,50)

 
%%
% Sin embargo, llamando de nuevo a pfijo con la expresin equivalente
%       x = -1/8(4x^4-16x^3+21x^2-16x-3)
% obtenemos la solucin de la ecuacin ms prxima a 1.8.

[sol,x,incr,iter] = pfijo('g2',1.8,1e-7,50)


%%
%
% FIN del Problema 8.1
