%% 8.1. Sistemas no lineales
%
% Dado un sistema no lineal de ecuaciones, se aproxima grficamente su
% solucin y se aplica el mtodo de punto fijo para estimar dicha solucin.
%
% Tema 8, Problema 1
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson


%%
%
% a) Estimacin grfica de la solucin
%
% Representamos grficamente las curvas y1 = x^2 - 2x + 0.5,
% y2 = sqrt(1 - x^2/4), e  y3 = -sqrt(1 - x^2/4);
% Podemos observar que el sistema tiene dos soluciones  alrededor de los
% puntos (-0.3,1) y (1.8,0.3).

% ezplot('-y + x.^2 - 2*x + 0.5')
xg = linspace(-1,3);
yg = xg.^2 - 2*xg + 0.5;
plot(xg, yg, 'Color', [0 .5 0])
hold on
ezplot('y^2 + x.^2/4 - 1')
axis([-3 3 -3 3])
% Xlim([-3 3])
% Ylim([-3 3])
title('Curvas del sistema')
grid


%%
%
% b) Aplicacin del mtodo de punto fijo
%
%  Utilizaremos el sistema de punto fijo equivalente:
%       x = 1/2*(x^2-y+0.5)
%       y = x^2+4*y^2-4+y
% que almacenamos en el fichero funcion1.m
%
% function y = funcion1(x)
% y(1) = 0.5*(x(1).^2-x(2)+0.5);
% y(2) = x(1).^2+4*x(2).^2-4+x(2);
%
% Llamamos a la funcin puntofijo.m y observamos que el mtodo es
% divergente.

[sol, iter, A] = puntofijo('funcion1', [-0.3, 1], 1e-7, 50)

plot(A(1,1), A(1,2), 'r*'), pause(1)
plot(A(2,1), A(2,2), 'r*'), pause(1)
plot(A(3,1), A(3,2), 'r*')


%%
%
% Utilizando el sistema de punto fijo:
% x = 1/2(x^2-y+0.5)
% y = 1/8(-x^2-4y^2+4+8y)
%
% Almacenamos en funcion2.m el nuevo sistema y llamamos de nuevo a la
% funcin puntofijo.m.
% En este caso la sucesin converge a la solucin que est cerca del punto
% inicial (-0.3,1).

[sol, iter, A] = puntofijo('funcion2', [-0.3, 1], 1e-7, 50)

axis([-0.5 0 0.8 1.2])
pause(1)
for k = 1 : length(A)
    plot(A(k,1), A(k,2), 'g*')
    pause(2/2^k)
end


%%
%
% Utilizando otra estimacin inicial observamos que el mtodo converge a la
% misma solucin.

[sol,iter,A] = puntofijo('funcion2',[1.8,0.3],1e-7,50)

axis([-3 3 -3 3])
for k = 1:length(A)
    plot(A(k,1), A(k,2), 'm*')
    pause(1/2^k)
end
hold off


%%
% Para obtener la grfica de las coordenadas de los distintos iterados
% basta representar los vectores columna de A,

plot(A, '.-')
title('Mtodo de punto fijo')
xlabel('Iteracin')
ylabel('Valor')
legend('Abscisa','Ordenada')
grid


%%
% mientras que para dibujar la tasa de convergencia lineal del mtodo:

[sol,iter,A] = puntofijo('funcion2',[-0.3,1],1e-7,50);
for i=1:iter-1
    conv(i)=norm(A(i+1,:)-A(i,:));
end
tcl = conv(2:end)./conv(1:end-1);
plot(tcl)
title('Tasa de convergencia')
xlabel('Iteracin')
grid


%%
% Para obtener una aproximacin de la segunda solucin del sistema
% obtenemos el sistema equivalente:
%
%       x = 1/2*(-x^2+4*x+y-0.5)
%       y = 1/11*(-x^2-4y^2+4+11y)
%
% que almacenamos en funcion3.m.

type funcion2


%%
%
% Partiendo de un punto cercano a la solucin, las iteraciones convergen.

[sol,iter,A] = puntofijo('funcion3',[1.8,0.3],1e-7,50)

ezplot('-y+x.^2-2*x+0.5')
hold on
ezplot('y^2+x.^2/4-1')
axis([1.5 2 0 0.5])
title('Curvas del sistema')
grid
for k = 1:length(A)
    plot(A(k,1), A(k,2), 'm*')
    pause(1/2^k)
end
hold off


%%
%
% c) Transformamos el sistema no lineal en una sla ecuacin y la
% representamos grficamente.

x = -1 : .03 : 2;
y = 4*x.^4 - 16*x.^3 + 21*x.^2 - 8*x - 3;
plot(x,y)
title('y = 4x^4 - 16x^3 + 21x^2 - 8x - 3')
grid


%%
%
% Observamos que tiene dos races reales, una alrededor de x = -0.2 y otra
% cerca de x = 1.8.
%
% A continuacin almacenamos la ecuacin de punto fijo
% x = 1/8*(4*x^4 - 16*x^3 + 21*x^2 - 3)
%
% en el fichero de funcion 'g.m'

type g


%%
%
% Representamos la ecuacin de punto fijo cerca de x = 1.8

xg = linspace(-1,4);
yg = g(xg);
plot(xg, yg, xg, xg)
axis([-1 4 -1 4])
axis equal
title('Punto fijo x = 1/8(4x^4 - 16x^3 + 21x^2 - 3)')
legend('Funcin de punto fijo', 'Funcin identidad')


%%
%
% Ejecutamos el mtodo de punto fijo unidimensional a partir de dicho punto.

[sol, x, incr, iter] = pfijo('g', 1.8, 1e-7, 50)

hold on

for k = 1 : 6
    plot(x(k), x(k+1), 'r*')
    pause(1/2^k)
end


%%
% Con distintas aproximaciones, vemos que no converge.

[sol, x, incr, iter] = pfijo('g', -0.3, 1e-7, 50)

for k = 1 : 20
    plot(x(k), x(k+1), 'm*')
    pause(1/2^k)
end
hold off

%%
% Sin embargo, llamando de nuevo a pfijo con la expresin equivalente
%       x = -1/8(4x^4 - 16x^3 + 21x^2 - 16x - 3)
% obtenemos la solucin de la ecuacin ms prxima a 1.8.

[sol, x, incr, iter] = pfijo('g2', 1.8, 1e-7, 50)

xg = linspace(1.75, 2);
yg = g2(xg);
plot(xg, yg, xg, xg)
axis equal
title('Punto fijo x = 1/8(4x^4 - 16x^3 + 21x^2 - 16x - 3)')
legend('Funcin de punto fijo', 'Funcin identidad')
hold on
for k = 1 : length(x) - 1
    plot(x(k), x(k+1), 'r*')
    pause(1/2^k)
end
hold off


%%
%
% FIN del Problema 8.1
