%% 8.2. Interseccin de curvas no continuas
% 
% Dado un sistema no lineal de ecuaciones, se aproxima 
% grficamente su solucin y se aplican los mtodos de 
% punto fijo y Newton para estimar dicha solucin.
% 
% Tema 8, Problema 2
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson

%% 
%
% a) Estimacin grfica de la solucin
%
% Representamos grficamente las curvas del sistema:
% Concluimos que las curvas se cortan en dos puntos que estn alrededor
% de (0.2,0.8) y (0.7,0.3).

ezplot('asin(exp(x^2)/(8*x))')
hold on
ezplot('1-x',[-1.8,1.8])
grid on
title('Curvas del sistema')
hold off


%%
%
% b) Aplicacin del mtodo de punto fijo
%  
%  Utilizaremos el sistema de punto fijo:
%       x = exp(x.^2)./(8 *sin(y))  
%       y = 1-x
% almacenado en el fichero funcion1.m y llamamos a la funcin puntofijo.m 
% Observamos que MATLAB nos proporciona una aproximacin a la primera de las races
% sol=(0.175599, 0.824401).

 [sol,iter,A] = puntofijo('funcion1',[0.2,0.8],1e-7,50)

 
%% 
% Podemos observar el comportamiento de la constante de convergencia lineal

for i=1:iter-1
       conv1(i)=norm(A(i+1,:)-A(i,:));
end
tcl = conv1(2:end)./conv1(1:end-1);
plot(tcl)


%%
% y la grfica de las dos coordenadas de las distintos iterados.

plot(A)


%%
% Si repetimos el proceso tomando otras aproximaciones iniciales observamos
% que el mtodo de punto fijo converge a la misma solucin.

 [sol,iter,A] = puntofijo('funcion1',[0.7,0.3],1e-7,50)
 

%%
% A continuacin, las grficas de los iterados

plot(A)


%%
% y de la tasa de convergencia lineal

for i=1:iter-1
       conv2(i)=norm(A(i+1,:)-A(i,:));
end
tcl = conv2(2:end)./conv2(1:end-1);
plot(tcl)


%%
% Para conseguir una aproximacin de la segunda raz debemos hacer otra
% transformacin del sistema 
%         x = 1-y  
%         y = asin(exp(x^2)/(8*x))
% almacenado en funcion2.m 
% Llamamos de nuevo a la funcin puntofijo.m, obteniendo una aproximacin
% de la segunda solucin del sistema. 

 [sol,iter,A] = puntofijo('funcion2',[0.7,0.3],1e-7,50)

 
%%
%
% c) La matriz jacobiana asociada al sistema es:
% J=[-2*x*exp(x^2)+8*sin(y), 8*x*cos(y);
%     1, 1];
% Debemos tener precaucin en aquellos puntos que hacen que el determinante
% de la matriz jacobiana est prximo a cero.

x = 0; y = pi; J = [-2*x*exp(x^2)+8*sin(y), 8*x*cos(y); 1, 1]; d(1)=det(J); d(1)
x = 0; y = 2*pi; J = [-2*x*exp(x^2)+8*sin(y), 8*x*cos(y); 1, 1]; d(2)=det(J); d(2)
x = 0; y = 3*pi; J = [-2*x*exp(x^2)+8*sin(y), 8*x*cos(y); 1, 1]; d(3)=det(J); d(3)
x = pi; y = pi/2; J = [-2*x*exp(x^2)+8*sin(y), 8*x*cos(y); 1, 1]; d(4)=det(J); d(4)


%%
% d) Construimos el fichero funcion3.m en el que definimos la funcin
% F(x,y), que describe el sistema, y la matriz jacobiana de F. Llamamos a
% la funcin Newton.m con la estimacin inicial (0.2,0.8) y obtenemos la
% aproximacin a la primera raz. 

[sol,incr,iter,A] = Newton('funcion3',[0.2,0.8],1e-7,50)


%%
% podemos observar en este caso la grfica de la convergencia cuadrtica del mtodo

for i=1:iter
       conv3(i)=norm(A(i+1,:)-A(i,:));
end
tcc = conv3(2:end)./conv3(1:end-1).^2;
plot(tcc)


%%
% y la representacin de las dos coordenadas de los distintos iterados.

plot(A)


%%
% Repetimos con la aproximacin inicial (0.7, 0.3):

[sol,incr,iter,A] = Newton('funcion3',[0.7, 0.3],1e-7,50)



%%
%
% FIN del Problema 8.2
