%% 8.2. Sistemas no lineales
%
% Dado un sistema no lineal de ecuaciones, se aproxima
% grficamente su solucin y se aplican los mtodos de
% punto fijo y Newton para estimar dicha solucin.
%
% Tema 8, Problema 2
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson


%%
%
% a) Estimacin grfica de la solucin
%
% Representamos grficamente las curvas del sistema:
% Concluimos que las curvas se cortan en dos puntos que estn alrededor
% de (0.2, 0.8) y (0.7, 0.3).

ezplot('asin(exp(x^2)/(8*x))')
hold on
plot([-1.8, 1.8], [2.8, -0.8], 'Color', [0 .5 0])
grid
title('Curvas del sistema')


%%
%
% b) Aplicacin del mtodo de punto fijo
%
%  Utilizaremos el sistema de punto fijo:
%       x = exp(x.^2)./(8*sin(y))
%       y = 1-x
% almacenado en el fichero funcion1.m y llamamos a la funcin puntofijo.m
% Observamos que MATLAB nos proporciona una aproximacin a la primera de las races
% sol=(0.175599, 0.824401).

[sol, iter, A1] = puntofijo('funcion1', [0.2, 0.8], 1e-7, 50)

title('Punto fijo 1. Iterados')
axis([0.15 0.2 0.8 0.85])

for k = 1 : length(A1)
    plot(A1(k,1), A1(k,2), 'r*')
    pause(2/2^k)
end
hold off


%%
%
% Podemos observar el comportamiento de la constante de convergencia lineal

for i=1:iter-1
    conv1(i)=norm(A1(i+1,:)-A1(i,:));
end
tcl = conv1(2:end)./conv1(1:end-1);

plot(tcl, '.-')
title('Punto fijo 1. Tasa de convergencia lineal')
xlabel('Iteracin')
grid


%%
%
% y la grfica de las dos coordenadas de los distintos iterados.

plot(A1, '.-')
title('Punto fijo 1. Evolucin de las coordenadas')
xlabel('Iteracin')
ylabel('Valor')
legend('Abscisa x', 'Ordenada y')
grid


%%
%
% Si repetimos el proceso tomando otras aproximaciones iniciales, observamos
% que el mtodo de punto fijo converge a la misma solucin, an partiendo
% de un punto cercano a la otra solucin.

[sol, iter, A2] = puntofijo('funcion1', [0.7, 0.3], 1e-7, 50)

ezplot('asin(exp(x^2)/(8*x))')
hold on
plot([0, 1], [1, 0], 'Color', [0 .5 0])
grid
axis([0 1 0 1]) 
plot(A1(:,1), A1(:,2), 'r*')
title('Punto fijo 2. Iterados')

for k = 1 : length(A2)
    plot(A2(k,1), A2(k,2), 'g*')
    pause(1/2^k)
end
hold off


%%
%
% A continuacin, las grficas de los iterados

plot(A2, '.-')
title('Punto fijo 2. Evolucin de las coordenadas')
xlabel('Iteracin')
ylabel('Valor')
legend('Abscisa x', 'Ordenada y')
grid


%%
%
% y de la tasa de convergencia lineal

for i=1:iter-1
    conv2(i)=norm(A2(i+1, :)-A2(i, :));
end
tcl = conv2(2:end)./conv2(1:end-1);

plot(tcl, '.-')
title('Punto fijo 2. Tasa de convergencia lineal')
xlabel('Iteracin')
grid


%%
%
% Para conseguir una aproximacin de la segunda raz debemos hacer otra
% transformacin del sistema
%         x = 1-y
%         y = asin(exp(x^2)/(8*x))
% almacenado en funcion2.m
% Llamamos de nuevo a la funcin puntofijo.m, obteniendo una aproximacin
% de la segunda solucin del sistema.

[sol, iter, A3] = puntofijo('funcion2', [0.6, 0.4], 1e-7, 50)

ezplot('asin(exp(x^2)/(8*x))')
hold on
plot([0, 1], [1, 0], 'Color', [0 .5 0])
grid
axis([0.6 0.8 0.2 0.4]) 
title('Punto fijo 3. Iterados')

for k = 1 : length(A3)
    plot(A3(k,1), A3(k,2), 'r*')
    pause(4/2^k)
end
hold off




%%
%
% Representamos como antes las coordenadas de los iterados

plot(A3, '.-')
title('Punto fijo 3. Evolucin de las coordenadas')
xlabel('Iteracin')
ylabel('Valor')
legend('Abscisa x', 'Ordenada y')
grid


%%
%
% y la tasa de convergencia lineal

for i=1:iter-1
    conv3(i)=norm(A3(i+1, :)-A3(i, :));
end
tcl = conv3(2:end)./conv3(1:end-1);

plot(tcl, '.-')
title('Punto fijo 3. Tasa de convergencia lineal')
xlabel('Iteracin')
grid



%%
%
% d) Construimos el fichero funcion3.m en el que definimos la funcin
% F(x,y) que describe el sistema y la matriz jacobiana de F. 

type funcion3

%%
%
% Llamamos a la funcin Newton.m con la estimacin inicial (0.2, 0.8) y
% obtenemos la aproximacin a la primera raz.

[sol, incr, iter, A1] = Newton('funcion3', [0.2, 0.8], 1e-7, 50)

ezplot('asin(exp(x^2)/(8*x))')
hold on
plot([0, 1], [1, 0], 'Color', [0 .5 0])
grid
axis([0.17 0.2 0.8 0.83]) 
title('Iterados del mtodo de Newton')

for k = 1 : length(A1)
    plot(A1(k,1), A1(k,2), 'r*')
    pause(4/2^k)
end
hold off


%%
%
% Podemos observar en este caso la convergencia cuadrtica del mtodo.

for i=1:iter
    conv3(i)=norm(A1(i+1, :)-A1(i, :));
end
tcc = conv3(2:end)./conv3(1:end-1).^2;

plot(tcc, '.-')
title('Newton 1. Tasa de convergencia cuadrtica')
xlabel('Iteracin')
grid


%%
%
% y la representacin de las dos coordenadas de los distintos iterados.

plot(A1, '.-')
title('Newton 1. Evolucin de las coordenadas')
xlabel('Iteracin')
ylabel('Valor')
legend('Abscisa x', 'Ordenada y')
grid


%%
%
% Repetimos con la aproximacin inicial (0.7, 0.3):

[sol, incr, iter, A2] = Newton('funcion3', [0.7, 0.3], 1e-7, 50)

ezplot('asin(exp(x^2)/(8*x))')
hold on
plot([0, 1], [1, 0], 'Color', [0 .5 0])
grid
axis([0.69 0.71 0.29 0.31]) 
plot(A2(:,1), A2(:,2), 'r*')
title('Newton 2. Iterados')

for k = 1 : length(A2)
    plot(A2(k,1), A2(k,2), 'g*')
    pause(4/2^k)
end
hold off


%%
%
% FIN del Problema 8.2
