%% 8.3. Sistemas no lineales
%
% Dado un sistema no lineal de ecuaciones, para estimar su solucin se aplican
% los mtodos de punto fijo (desplazamientos sucesivos y simultneos), Newton y la
% variante del mtodo de Newton que actualiza peridicamente la matriz jacobiana.
%
% Tema 8, Problema 3
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson

%%
%
% a) Transformamos el sistema despejando la variable x de la primera
% ecuacin, la y de la segunda y la z de la tercera.
%
% Definimos la funcin G(x, y, z) que describe el sistema de punto fijo
% en el fichero funcion1.m. Tiene dos versiones que se utilizan en el
% mtodo de punto fijo con desplazamientos simultneos o sucesivos, 
% respectivamente.
%
% Cada ecuacin define una superficie en R3 y la solucin del sistema es la
% interseccin de las tres superficies.

type funcion1

dibu


%%
%
% Aadimos la variable 'tipo' como parmetro de entrada de la
% funcin puntofijo, resultando el fichero de funcin puntofijot.m.
% Ejecutaremos puntofijot primero para desplazamientos simultneos.

[sol, iter, A] = puntofijot('funcion1', [0, 0, 0], 1e-7, 50, 1)

title('Punto fijo desplazamientos simultneos')
for k = 1 : length(A)
    pause(2/2^k)
    plot3(A(k, 1), A(k, 2), A(k, 3), 'r*')
end
hold off


%%
% Posteriormente ejecutamos puntofijot con tipo=2 para desplazamientos
% sucesivos.

[sol, iter, A] = puntofijot('funcion1', [0, 0, 0], 1e-7, 50, 2)

dibu
title('Punto fijo desplazamientos sucesivos')
for k = 1 : length(A)
    pause(2/2^k)
    plot3(A(k,1), A(k,2), A(k,3), 'r*')
end
hold off


%%
%
% b) Construimos el archivo funcion2.m en el que definimos
% la funcin F que describe el sistema inicial y su matriz jacobiana.

type funcion2

dibu
title('Mtodo de Newton')


%%
%
% Si llamamos a la funcin Newton, obtenemos un mensaje de error provocado
% por la singularidad de la matriz jacobiana en la iteracin inicial.

[sol, incr, iter, A] = Newton('funcion2', [0, 0, 0], 1e-7, 50)


%%
%
% Cambiando la aproximacin inicial, obtenemos una solucin aproximada.

[sol, incr, iter, A] = Newton('funcion2', [0.5, 0.5, 0.5], 1e-7, 50), 

view(-230, 30)
title('Mtodo de Newton')
for k = 1 : length(A)
    pause(2/2^k)
    plot3(A(k,1), A(k,2), A(k,3), 'y*')
end
hold off


%%
% A continuacin podemos observar la representacin grfica
% de la tasa de convergencia cuadrtica

for i=1:iter
    conv1(i) = norm(A(i+1, :) - A(i, :));
end
tcc = conv1(2:end)./conv1(1:end-1).^2;

plot(tcc, '.-')
title('Tasa de convergencia cuadrtica')
xlabel('Iteracin')
grid


%%
% y la grfica de las coordenadas de los distintos iterados.

plot(A, '.-')
title('Mtodo de Newton. Evolucin de las coordenadas')
xlabel('Iteracin')
ylabel('Valor')
legend('Abscisa x', 'Ordenada y', 'Cota z')
grid


%%
% Si tomamos como aproximacin inicial (-0.5, -0.5, -0.5) obtenemos
% la aproximacin a una nueva raz.

[sol, incr, iter, A] = Newton('funcion2', [-0.5, -0.5, -0.5], 1e-7, 50)


dibu
title('Mtodo de Newton')
for k = 1 : length(A)
    pause(2/2^k)
    plot3(A(k,1), A(k,2), A(k,3), 'r*')
end
hold off


%%
%
% c) Construimos un fichero, Newtonmod.m, que contiene la variante del
% mtodo de Newton consistente en actualizar la jacobiana tras un nmero
% fijo de iteraciones, que denotaremos por 'per' y  formar parte de los
% parmetros de entrada. Llamamos a esta funcin con actualizacin de la
% jacobiana cada dos iteraciones:

[sol, iter, A] = Newtonmod('funcion2', [0.5, 0.5, 0.5], 1e-7, 50, 2)

dibu
view(-230, 30)
title('Mtodo de Newton modificado')
for k = 1 : length(A)
    pause(2/2^k)
    plot3(A(k,1), A(k,2), A(k,3), 'y*')
end
hold off


%%
% Repetimos el proceso con per = 5:

[sol, iter, A] = Newtonmod('funcion2', [0.5, 0.5, 0.5], 1e-7, 50, 5)

dibu
view(-230, 30)
title('Mtodo de Newton modificado')
for k = 1 : length(A)
    pause(2/2^k)
    plot3(A(k,1), A(k,2), A(k,3), 'y*')
end
hold off


%%
% Para encontrar la otra solucin cambiamos de estimacin inicial:

[sol, iter, A] = Newtonmod('funcion2', [-0.5, -0.5, -0.5], 1e-7, 50, 2)

dibu
title('Mtodo de Newton modificado')
for k = 1 : length(A)
    pause(2/2^k)
    plot3(A(k,1), A(k,2), A(k,3), 'r*')
end
hold off


%%
%
% FIN del Problema 8.3