%% 8.5. Clculo del beneficio mximo de una empresa
% 
% Dado un problema de optimizacin empresarial, se utilizan los mtodos de
% punto fijo y Newton para estimar los niveles de produccin que maximizan 
% los beneficios.
% 
% Tema 8, Problema 5
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson

%% 
%
% a)  La funcin beneficio se puede expresar de la forma
%
%       B(q1,q2) = -1/6*q1^3-2*q1^2-q2^2+6*q1*q2+80*q1+110*q2-90
%
% Para determinar el mximo de esta funcin debemos resolver el sistema
%
%                     -0.5*q1^2-4*q1+80+6*q2 = 0 
%                              6*q1+110-2*q2 = 0
%
% Dado que slo estamos interesados en soluciones positivas, representamos 
% las curvas del sistema en el intervalo (0,100).
% Concluimos, a la vista de la grfica, que el sistema tiene una nica
% raz real positiva que est alrededor de (45, 200). 

   q1=0:.1:100;
   q2a=(0.5*q1.^2+4*q1-80)/6;
   q2b=(6*q1+110)/2;
   plot(q1,[q2a',q2b']), title('Grfica curvas sistema')

%%
%
% b) Primer sistema de punto fijo equivalente (almacenado en funcion1.m):
%                       q1 = 0.25*(-0.5*q1^2+6*q2+80) 
%                       q2 = 3*q1+55
% Observamos que en este caso el mtodo de punto fijo es divergente.
 
  [x,iter] = puntofijo('funcion1',[45,200],1e-7,200)


%%
%
% Aplicando el mtodo de punto fijo para distintas aproximaciones iniciales
% sobre el segundo sistema de punto fijo equivalente (almacenado en
% funcion2.m),
%                       q1 = sqrt(-8*q1+12*q2+160)
%                       q2 = 3*q1+55
% observamos cmo el mtodo converge a la solucin estimada grficamente en
% el apartado a).

  [x,iter] = puntofijo('funcion2',[45,200],1e-7,200)


%%

  [x,iter] = puntofijo('funcion2',[1,1],1e-7,200)


%%

  [x,iter] = puntofijo('funcion2',[10,10],1e-7,200)


%%

  [x,iter] = puntofijo('funcion2',[500,1000],1e-7,200)


%%
%
% Mostramos en la figura la tasa de convergencia lineal y cuadrtica para
% la primera aproximacin inicial. Se observa como la tasa de convergencia
% lineal se estabiliza por debajo de 1, mientras que la tasa de
% convergencia cuadrtica no est acotada. 

  [xg,iterg,Ag] = puntofijo('funcion2',[45,200],1e-7,200);
  for i=1:iterg-1
      conv(i)=norm(Ag(i+1,:)-Ag(i,:));
  end
  tcl=conv(2:end)./conv(1:end-1);
  tcc=conv(2:end)./conv(1:end-1).^2;
  subplot(2,1,1),plot(tcl),title('tasa convergencia lineal')
  subplot(2,1,2),plot(tcc),title('tasa convergencia cuadrtica')


%%
%
% c) Construimos el archivo funcion3.m en el que definimos la funcin que
% describe el sistema y su matriz jacobiana. 

type('funcion3')

%%
%
% Llamamos a la funcin Newton.m con las distintas aproximaciones iniciales
% utilizadas en el apartado b), obtenindose en algn caso soluciones
% diferentes a la obtenida grficamente. 

  [x1, incr1, iter1] = Newton('funcion3',[45,200],1e-7,100)


%%

  [x2, incr2, iter2] = Newton('funcion3',[1,1],1e-7,100)


%%

  [x3, incr3, iter3] = Newton('funcion3',[10,10],1e-7,100)


%%

  [x4, incr4, iter4,A4] = Newton('funcion3',[500,1000],1e-7,100)



%%
% Calculemos los valores que va tomando la tasa de convergencia cuadrtica
% cuando utilizamos la aproximacin inicial (500, 10000). Observamos que la
% tasa de convergencia cuadrtica se estabiliza, como ocurre en todos los
% casos restantes.
  
  [xg2, incrg2, iterg2,Ag2] = Newton('funcion3',[5000,10000],1e-7,100)
  for j=1:iterg2-1
      convg2(j) = norm(Ag2(j+1,:)-Ag2(j,:));
  end
  tcl=convg2(2:end)./convg2(1:end-1);
  tcc=convg2(2:end)./convg2(1:end-1).^2;
  subplot(2,1,1),plot(tcl),title('tasa convergencia lineal')
  subplot(2,1,2),plot(tcc),title('tasa convergencia cuadrtica')


%%
%
% d)  La matriz hessiana asociada a la funcin B(q1,q2) es
%
% H = [-q1-4, 6; 6, -2]
%
% Comprobamos que esta matriz evaluada en el punto (45.874755, 192.624265)
% es una matriz definida negativa. As, el punto en cuestin es un mximo
% de la funcin beneficio.

  q1 = 45.874755;
  q2 = 192.624265;
  H = [-q1-4, 6; 6, -2]
  sign(H(1,1)), 
  sign(det(H))

  
%%
% FIN del Problema 8.5
