%% 8.6. Aproximacin de datos por funciones no lineales
% 
% Se pretende ajustar, mediante un modelo no lineal, un conjunto de datos 
% experimentales. Para ello aplicamos la tcnica de mnimos cuadrados, utilizando el 
% mtodo de Newton para obtener el valor de los parmetros del modelo.
% 
% Tema 8, Problema 6
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson

%% 
%
% a)   El error cuadrtico es:
%
% E=(d1-exp(-alfa*t1)-beta*sin(t1))^2+(d2-exp(-alfa*t2)-beta*sin(t2))^2+...
%     +(d_7-exp(-alfa*t7}-beta*sin(t7))^2
%
% donde tk, k=1,2,...,7 son los instantes en los que se ha medido d(t), y 
% dk denota el valor de dicha magnitud en el instante tk, k=1,2,...,7.
% Por tanto, el sistema a resolver es:
%
% (d1-exp(-x*t1)-y*sin(t1))*t1*exp(-x*t1)+...+(d7-exp(-x*t7)-y*sin(t7))*t7*exp(-x*t7)=0
% (d1-exp(-x*t1)-y*sin(t1))*sin(t1)+...+(d7-exp(-x*t7)-y*sin(t7))*sin(t7)=0
%
% Representamos grficamente los datos tabulados como paso previo a su
% ajuste por el modelo no lineal proporcionado.

  t = 0:0.5:3;
  d = [1.095 -0.1569 -1.0157 -1.474 -1.3616 -0.8342 -0.0135];
  plot(t,d,'*')

  
%%
%
% b) Almacenamos en el fichero funcion.m los datos de la tabla
% experimental, el sistema no lineal anterior y su matriz jacobiana. 
%
% function [F,dF] = funcion(x)
%
% t = 0:0.5:3;
% d = [1.095 -0.1569 -1.0157 -1.474 -1.3616 -0.8342 -0.0135];
% u = (d-exp(-x(1).*t)-x(2).*sin(t)).*(t.*exp(-x(1).*t));
% v = (d-exp(-x(1).*t)-x(2).*sin(t)).*sin(t);
%
% F(1) = sum(u);
% F(2) = sum(v);
%
% w1 = (t.*exp(-x(1).*t)).^2;
% w2 = (d-exp(-x(1).*t)-x(2).*sin(t)).*(-t.^2).*exp(-x(1).*t);
% w = w1+w2;
%
% dF(1,1) = sum(w);
% dF(1,2) = sum([-sin(t).*t.*exp(-x(1).*t)]);
% dF(2,1) = sum([sin(t).*t.*exp(-x(1).*t)]);
% dF(2,2) = sum([-sin(t).*sin(t)]);

%%
%
% Llamamos a la funcin Newton.m para distintas aproximaciones iniciales,

  [sol,incr,iter] = Newton('funcion',[0,0],1e-7,100)


%%

  [sol,incr,iter] = Newton('funcion',[-1,1],1e-7,100)


%%

  [sol,incr,iter] = Newton('funcion',[1,-1],1e-7,100)


%%

  [sol,incr,iter] = Newton('funcion',[-1,-1],1e-7,100)


%%
% Por tanto, el modelo v(t) que mejor ajusta los datos de la tabla por
% mnimos cuadrados es
%
% v(t) = exp(-0.42953*t)-1.984219*sin(t)
%
% Representamos en una misma grfica los puntos de la tabla y la funcin
% v(t) en el intervalo [0, 3]. A la vista de la grfica concluimos que el
% modelo no lineal se ajusta bastante bien a los datos.

  x = 0:0.01:3;   y = exp(-0.429530*x)-1.984219*sin(x);
  plot(t,d,'*',x,y)


%%
%
% c) El error cuadrtico cometido con la aproximacin alfa = 0.429530 y
% beta = -1.984219 es:

  u = (d-exp(-0.42953.*t)+1.984219*sin(t)).^2;
  E = sum(u)


%%
%
% d)  Podemos responder a este apartado estudiando las races de la ecuacin
%
%       exp(-0.42953*t)-1.984219*sin(t) = 0
%
% Grficamente, observamos que la funcin v(t) tiene dos ceros, uno
% alrededor de t = 0.4 y otro alrededor de t = 3.

  ezplot('exp(-0.42953*t)-1.984219*sin(t)')
  grid on
  

%%
% 
% Llamamos dos veces a la funcin newton1var.m, construida en el captulo
% 2, con sendas aproximaciones iniciales, y obtenemos los instantes
% temporales en los que la funcin v(t) se anula, t = 0.431941 y 
% t = 3.002358. Para ello, definimos en el fichero funcion2.m la expresin 
% cuyo cero estamos buscando y su derivada.
% 
% function [y,dy] = funcion2(x)
% y = exp(-0.42953*t)-1.984219*sin(t);
% dy = -0.42953*exp(-0.42953*t)-1.984219*cos(t);

  [sol,x,incr,iter] = newton1var('funcion2',0.4,1e-7,50)


%%

  [sol,x,incr,iter] = newton1var('funcion2',3,1e-7,50)


%%
% FIN del Problema 8.6
