%% 8.7. Distancia mnima entre dos curvas
% 
% Dadas dos curvas calculamos su distancia mnima, planteando el sistema no
% lineal adecuado y aproximando su solucin mediante los mtodos de Newton
% y Broyden.
% 
% Tema 8, Problema 7
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson

%% 
%
% a)  Inicialmente, representamos la parbola y = x^2 y la recta v = u-5.

          x = -3:0.1:6;
          y1 = x.^2;
          y2 = x-5;
          plot(x,[y1',y2']), title('Curvas C_1 y C_2')


%%
% Para obtener la la distancia mnima entre un punto de la parbola (x,y) y
% un punto de la recta (u,v) debemos resolver el sistema:
%
%         22*x+4*x^3-2*u-4*x*u = 0 
%         -2*x+4*u-2*x^2-10 = 0


%% 
%
% b)   Almacenamos el sistema y su matriz jacobiana asociada en funcion.m.
%
% function [F,dF] = funcion(x)
% F(1) = 22*x(1)+4*x(1)^3-2*x(2)-4*x(1)*x(2);
% F(2) = -2*x(1)+4*x(2)-2*x(1)^2-10;
% if nargout==2
%    dF(1,1) = 22+12*x(1)^2-4*x(2);
%    dF(1,2) = -2-4*x(1);
%    dF(2,1) = dF(1,2);
%    dF(2,2) = 4;
% end


%%
%
% Debemos utilizar aproximaciones iniciales que no cumplan u = 5.75+x+4*x^2 para 
% evitar que se anule el determinante de la jacobiana.

  [sol1,incr1,iter1] = Newton('funcion',[0,0],1e-7,50)


%%

  [sol2,incr2,iter2] = Newton('funcion',[-1,4],1e-7,50)


%%

  [sol3,incr3,iter3] = Newton('funcion',[1,-1],1e-7,50)


%%

  [sol4,incr4,iter4] = Newton('funcion',[0.5,3],1e-7,50)


%%
%
% En cualquier caso, el nmero de iteraciones es relativamente pequeo y 
% la tasa de convergencia cuadrtica se estabiliza.

  [solg, incrg, iterg,Ag] = Newton('funcion',[1,-1],1e-7,50);
  for i=1:iterg-1
      convg(i) = norm(Ag(i+1,:)-Ag(i,:));
  end
  tcc=convg(2:end)./convg(1:end-1).^2;
  tcc(end)
  plot(tcc)

%% 
%
% c)   Repetimos el apartado b) pero ahora utilizando el mtodo de Broyden:

  [sol5,iter5] = Broyden('funcion',[0,0],1e-7,100)


%%

  [sol6,iter6] = Broyden('funcion',[-1,4],1e-7,100)


%%

  [sol7,iter7] = Broyden('funcion',[1,-1],1e-7,100)


%%

  [sol8,iter8] = Broyden('funcion',[0.5,3],1e-7,100)


%%
% Para cualquier aproximacin inicial, la solucin aproximada es siempre la misma, 
% con un nmero de iteraciones, en general, superior al del mtodo de Newton.
% Podemos hablar de convergencia superlineal en todos los casos.

  [solg2,iterg2,Ag2] = Broyden('funcion',[-1,4],1e-7,100);
  for j=1:iterg2-1
      convg2(j) = norm(Ag2(j+1,:)-Ag2(j,:));
  end
  tcl=convg2(2:end)./convg2(1:end-1);
  tcl(end)
  plot(tcl)
  
%% 
%
% d)   Para comprobar que el punto (0.5, 2.875) es, efectivamente, un mnimo
% debemos evaluar en este punto la matriz hessiana asociada, 
% que no es mas que la matriz jacobiana calculada anteriormente. 

  [F,dF] = funcion([0.5,2.875]);
  sign(dF(1,1))
  sign(det(dF))


%%
% Podemos concluir que (0.5, 2.875) es un mnimo de la funcin D(x,u). 
% Los puntos donde se alcanza el mnimo son, en la parbola:

x = 0.5, y = x^2 


%%
% y en la recta:

u = 2.875, v = u-5


%%
% siendo la distancia mnima entre las dos curvas:

d = sqrt((x-u)^2+(x^2-(u-5))^2)


%%
% FIN del Problema 8.7
