%% 8.8. Ecuacin integral
% 
% Dada una ecuacin integral, se aplica la frmula de cuadratura de trapecios
% para transformarla en un sistema de ecuaciones no lineal, cuyas soluciones se
% aproximan mediante las tcnicas de punto fijo y Newton.
% 
% Tema 8, Problema 8
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson

%% 
%
% a)  Consideramos h=1/5 y los nodos ti=0+i*h, i=0,1,...,5. El sistema 6x6,
% cuyas incgnitas u0, u1,..., u5 representan valores aproximados de la
% solucin de la ecuacin integral en los nodos elegidos, es:
%
% -5*exp(2)*u0+u0^2+2*u1^2+ ... + 2*u4^2+u5^2 = -5  
% -50/11*exp(2)*u1+u0^2+2*exp(1/25)*u1^2+...+2*exp(4/25)*u4^2+exp(1/5)*u5^2 = -50/11
% -50/12*exp(2)*u2+u0^2+2*exp(2/25)*u1^2+...+2*exp(8/25)*u4^2+exp(2/5)*u5^2 = -50/12
% -50/13*exp(2)*u3+u0^2+2*exp(3/25)*u1^2+...+2*exp(12/25)*u4^2+exp(3/5)*u5^2 = -50/13
% -50/14*exp(2)*u4+u0^2+2*exp(4/25)*u1^2+...+2*exp(16/25)*u4^2+exp(4/5)*u5^2 = -50/14
% -50/15*exp(2)*u5+u0^2+2*exp(5/25)*u1^2+...+2*exp(20/25)*u4^2+exp(1)*u5^2 = -50/15


%%
%
% b) Transformamos el sistema anterior en uno equivalente y lo almacenamos
% en el archivo funcion1.m.
%
% function G = funcion1(u)
% e = exp([1:4]/25);  alpha = 50*exp(2);
% u2 = u(2:5).^2;
% G(1) = (10/alpha)*(2*sum(u2)+u(1)^2+u(6)^2+50/10);
% G(2) = (11/alpha)*(2*sum(e.^1.*u2)+u(1)^2+exp(1/5)*u(6)^2+50/11);
% G(3) = (12/alpha)*(2*sum(e.^2.*u2)+u(1)^2+exp(2/5)*u(6)^2+50/12);
% G(4) = (13/alpha)*(2*sum(e.^3.*u2)+u(1)^2+exp(3/5)*u(6)^2+50/13);
% G(5) = (14/alpha)*(2*sum(e.^4.*u2)+u(1)^2+exp(4/5)*u(6)^2+50/14);
% G(6) = (15/alpha)*(2*sum(e.^5.*u2)+u(1)^2+exp(5/5)*u(6)^2+50/15);


%%
%
% Utilizamos el mtodo de punto fijo con una aproximacin inicial dentro
% del dominio de convergencia D, por ejemplo (0.25, 0.25, 0.25, 0.25, 0.25,
% 0.25), obteniendo una aproximacin de la solucin:

[x1,iter1,A1] = puntofijo('funcion1',0.25*ones(1,6),1e-7,100)

%%
% Si tomamos aproximaciones iniciales fuera del dominio D, en ocasiones el mtodo
% no converge pero otras veces s lo hace. Por ejemplo,

[x2,iter2,A2] = puntofijo('funcion1',[1 2 3 4 5 6],1e-7,100)


%%
% Observemos el comportamiento de la tasa de convergencia lineal y cuadrtica. 
% Mientras la primera se estabiliza por debajo de 1, la segunda no est acotada.

  [xg,iterg,Ag] = puntofijo('funcion1',ones(1,6),1e-7,100)
  for i=1:iterg-1
      convg(i)=norm(Ag(i+1,:)-Ag(i,:));
  end
  tcl=convg(2:end)./convg(1:end-1);
  tcc=convg(2:end)./convg(1:end-1).^2;
  subplot(2,1,1),plot(tcl),title('tasa convergencia lineal')
  subplot(2,1,2),plot(tcc),title('tasa convergencia cuadrtica')


%%
%
% c) Construimos el archivo funcion2.m en el que definimos la funcin
% vectorial F cuyo cero buscamos y su matriz jacobiana asociada.
%
% function [F,dF] = funcion2(u)
%
% e = exp([1:4]/25);  alpha = 50*exp(2);
% u2 = u(2:5).^2;
% F(1) = 2*sum(u2)+u(1)^2+u(6)^2-(alpha/10)*u(1)+50/10;
% F(2) = 2*sum(e.^1.*u2)+u(1)^2+exp(1/5)*u(6)^2-(alpha/11)*u(2)+50/11;
% F(3) = 2*sum(e.^2.*u2)+u(1)^2+exp(2/5)*u(6)^2-(alpha/12)*u(3)+50/12;
% F(4) = 2*sum(e.^3.*u2)+u(1)^2+exp(3/5)*u(6)^2-(alpha/13)*u(4)+50/13;
% F(5) = 2*sum(e.^4.*u2)+u(1)^2+exp(4/5)*u(6)^2-(alpha/14)*u(5)+50/14;
% F(6) = 2*sum(e.^5.*u2)+u(1)^2+exp(5/5)*u(6)^2-(alpha/15)*u(6)+50/15;
% u = 2*u;
% u(2:5) = 2*u(2:5);
% dF(1,:) = u;                        dF(1,1) = dF(1,1) - alpha/10;
% dF(2,:) = exp([0:5]/25).*u;         dF(2,2) = dF(2,2) - alpha/11;
% dF(3,:) = exp([0:2:10]/25).*u;      dF(3,3) = dF(3,3) - alpha/12;
% dF(4,:) = exp([0:3:15]/25).*u;      dF(4,4) = dF(4,4) - alpha/13;
% dF(5,:) = exp([0:4:20]/25).*u;      dF(5,5) = dF(5,5) - alpha/14;
% dF(6,:) = exp([0:5:25]/25).*u;      dF(6,6) = dF(6,6) - alpha/15;


%%
%
% Llamamos a la funcin Newton.m con la aproximacin inicial en D y 
% obtenemos, con un nmero de iteraciones inferior, la misma solucin que
% proporcionaba el mtodo de punto fijo. 

[sol1,incr1,iter1] = Newton('funcion2',0.25*ones(1,6),1e-7,100)


%%
%
% Sin embargo, utilizando aproximaciones iniciales fuera del dominio D,
% para las que el mtodo de punto fijo es divergente, obtenemos una nueva
% solucin del sistema.

[sol2,incr2,iter2] = Newton('funcion2',[1 2 3 4 5 6],1e-7,100)


%%
%
% A continuacin, observamos las distintas tasas de convergencia para este
% ltimo caso. 

  [solg2,incrg2,iterg2,Ag2] = Newton('funcion2',[1 2 3 4 5 6],1e-7,100)
  for i=1:iterg2-1
      convg2(i)=norm(Ag2(i+1,:)-Ag2(i,:));
  end
  tcl=convg2(2:end)./convg2(1:end-1);
  tcc=convg2(2:end)./convg2(1:end-1).^2;
  subplot(2,1,1),plot(tcl),title('tasa convergencia lineal')
  subplot(2,1,2),plot(tcc),title('tasa convergencia cuadrtica')
  
  
%%
%
% d) Es sencillo comprobar que la funcin e^x satisface la ecuacin
% integral. 
% Conocida la solucin, observamos que el mtodo de Newton nos ha
% proporcionado dos soluciones del sistema, sol1 que coincide con la
% obtenida por punto fijo, y sol2 que es la que se acerca ms a la solucin
% exacta.

x = linspace(0,1);
y = exp(x);
t = 0:.2:1;
subplot(2,1,1),plot(x,y,t,sol1','+'),title('Solucin 1 (punto fijo)')
subplot(2,1,2),plot(x,y,t,sol2','*'),title('Solucin 2 (Newton)')


%%
% FIN del Problema 8.8
