%% 8.9. Puntos crticos de una funcin de dos variables
% 
% Dada una funcin de dos variables, aproximaremos sus puntos crticos
% tanto grfica como numricamente, mediante el mtodo de Newton.
% 
% Tema 8, Problema 9
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson

%% 
%
% a)  Teniendo en cuenta que la funcin es peridica, de periodo 2*pi,
% vamos a representarla en [0,2*pi]x[0,2*pi].

x = 0:0.01:2*pi; y = x;
[X,Y] = meshgrid(x,y);
Z = cos(X+Y)+sin(X)+cos(Y);
mesh(x,y,Z)


%%
%
% Adems, podemos tener una idea aproximada de los puntos crticos de la
% funcin g dibujando sus curvas de nivel. As, estimamos que  g tiene un
% mximo alrededor del punto (1, 5.5), un mnimo alrededor de (5.5, 3.5) y
% un punto de silla sobre (3.2, 1.5).

contour(x,y,Z)


%%
%
% b) Los puntos que buscamos estn entre aquellos que anulan al gradiente
% de la funcin g. As pues, el sistema a resolver es
%
%               - sen(x+y) + cos(x) = 0 
%               - sen(x+y) - sen(y) = 0


%%
%
% c) Construimos el archivo funcion.m en el que definimos la funcin que
% describe el sistema znterior y su matriz jacobiana asociada.
%
% function [F,dF] = funcion(x)
%
% F(1) = -sin(x(1)+x(2))+cos(x(1));
% F(2) = -sin(x(1)+x(2))-sin(x(2));%
%
% dF(1,1) = -cos(x(1)+x(2))-sin(x(1));
% dF(1,2) = -cos(x(1)+x(2));
% dF(2,1) = dF(1,2);
% dF(2,2) = -cos(x(1)+x(2))-cos(x(2));


%%
%
% Llamamos a Newton.m utilizando como aproximaciones iniciales los puntos
% observados en el apartado a), calculando en cada caso las tasas de
% convergencia lineal y cuadrtica.

  [sol1,incr1,iter1,A1] = Newton('funcion',[1 5.5],1e-7,100)
  for i=1:iter1-1
      conv1(i)=norm(A1(i+1,:)-A1(i,:));
  end
  tcl1=conv1(2:end)./conv1(1:end-1);
  tcc1=conv1(2:end)./conv1(1:end-1).^2;
  subplot(2,1,1),plot(tcl1),title('tasa convergencia lineal')
  subplot(2,1,2),plot(tcc1),title('tasa convergencia cuadrtica')


%%
%
% En los dos primeros casos, las tasas de convergencia lineal y cuadrtica
% se estabilizan. 

  [sol2,incr2,iter2,A2] = Newton('funcion',[5.5,3.5],1e-7,100)
  for i=1:iter2-1
      conv2(i)=norm(A2(i+1,:)-A2(i,:));
  end
  tcl2=conv2(2:end)./conv2(1:end-1);
  tcc2=conv2(2:end)./conv2(1:end-1).^2;
  subplot(2,1,1),plot(tcl2),title('tasa convergencia lineal')
  subplot(2,1,2),plot(tcc2),title('tasa convergencia cuadrtica')


%%
%
% Sin embargo, en el tercer caso, nicamente se estabiliza la tasa de
% convergencia lineal.

  [sol3,incr3,iter3,A3] = Newton('funcion',[3.2,1.5],1e-7,100)
  for i=1:iter3-1
      conv3(i)=norm(A3(i+1,:)-A3(i,:));
  end
  tcl3=conv3(2:end)./conv3(1:end-1);
  tcc3=conv3(2:end)./conv3(1:end-1).^2;
  subplot(2,1,1),plot(tcl3),title('tasa convergencia lineal')
  subplot(2,1,2),plot(tcc3),title('tasa convergencia cuadrtica')


%%
%
% d) La matriz jacobiana obtenida en el apartado c) es a su vez la matriz
% hessiana de la funcin g(x,y). 
%
% Para determinar el tipo de punto crtico analizamos el carcter de la
% matriz hessiana evaluada en cada punto.
%
% Para el primer punto, podemos concluir que es un mximo relativo de la
% funcin g(x,y).

[F1,H1] = funcion(sol1)
sign(H1(1,1))
sign(det(H1))

%%
%
% Respecto al segundo punto, la hessiana es una matriz
% definida positiva y por tanto el punto (5.235988, 3.665191) es un
% mnimo relativo de la funcin g(x,y).

[F2,H2] = funcion(sol2)
sign(H2(1,1))
sign(det(H2))


%%
%
% Finalmente, la matriz hessiana evaluada en el tercer punto es una matriz indefinida
% y en consecuencia el punto (3.141593, 1.570796) es un punto
% de silla de la funcin g(x,y).

[F3,H3] = funcion(sol3)
sign(H3(1,1))
sign(det(H3))


%%
% FIN del Problema 8.9