%% 8.10. Optimizacin de una funcin con restricciones
% 
% El objetivo del problema es determinar el mximo de una funcin
% de varias variables, sujeto a restricciones. Dicho mximo se aproximar
% mediante los mtodos de Newton y Broyden.
%
% Tema 8, Problema 10
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson

%% 
%
% a) La funcin lagrangiana cuyo mximo buscamos es:
%
%           f(x,y,z,l) = xyz + l(xy + xz + yz - 1)
%
% Por tanto, tenemos que resolver el sistema no lineal:
%
%           yz + l(y + z) = 0
%           xz + l(x + z) = 0 
%           xy + l(x + y) = 0
%           xy + xz + yz - 1 = 0
%
% Observamos que las soluciones del sistema han de verificar x = y = z, 
% lo que aprovecharemos a la hora de tomar aproximaciones iniciales.

%%
%
% b) Construimos un fichero funcion.m en el que definimos la funcin
% vectorial que describe el sistema y su matriz jacobiana.
%
% function [F,dF] = funcion(x)
%
% F(1) = x(2)*x(3)+x(4)*(x(2)+x(3));
% F(2) = x(1)*x(3)+x(4)*(x(1)+x(3));
% F(3) = x(1)*x(2)+x(4)*(x(1)+x(2));
% F(4) = x(1)*x(2)+x(1)*x(3)+x(2)*x(3)-1;
% if nargout==2
%    dF(1,:) = [0, x(3)+x(4), x(2)+x(4), x(2)+x(3)];
%    dF(2,:) = [x(3)+x(4), 0, x(1)+x(4), x(1)+x(3)];
%    dF(3,:) = [x(2)+x(4), x(1)+x(4), 0, x(1)+x(2)];
%    dF(4,:) = [x(2)+x(3), x(1)+x(3), x(1)+x(2), 0];
% end


%%
%
% Llamamos a la funcin Newton.m con diferentes aproximaciones iniciales,
% representando en cada caso las tasas de convergencia lineal y cuadrtica.

  [sol1,incr1,iter1,A1] = Newton('funcion',[1 1 1 1],1e-7,100)
  for i=1:iter1-1
      conv1(i)=norm(A1(i+1,:)-A1(i,:));
  end
  tcl1=conv1(2:end)./conv1(1:end-1);
  tcc1=conv1(2:end)./conv1(1:end-1).^2;
  subplot(2,1,1),plot(tcl1),title('tasa convergencia lineal')
  subplot(2,1,2),plot(tcc1),title('tasa convergencia cuadrtica')


%%
%
% Notamos que en prcticamente todos los casos, tanto la tasa de
% convergencia lineal como cuadrtica se estabilizan.

  [sol2,incr2,iter2,A2] = Newton('funcion',[10 10 10 10],1e-7,100)
  for i=1:iter2-1
      conv2(i)=norm(A2(i+1,:)-A2(i,:));
  end
  tcl2=conv2(2:end)./conv2(1:end-1);
  tcc2=conv2(2:end)./conv2(1:end-1).^2;
  subplot(2,1,1),plot(tcl2),title('tasa convergencia lineal')
  subplot(2,1,2),plot(tcc2),title('tasa convergencia cuadrtica')


%%
%
% Para la aproximacin inicial (1,1,1,-1) el mtodo no es convergente ya
% que la matriz jacobiana en ese punto es singular.

  [sol3,incr3,iter3,A3] = Newton('funcion',[1,1,1,-1],1e-7,100)
  for i=1:iter3-1
      conv3(i)=norm(A3(i+1,:)-A3(i,:));
  end
  tcl3=conv3(2:end)./conv3(1:end-1);
  tcc3=conv3(2:end)./conv3(1:end-1).^2;
  subplot(2,1,1),plot(tcl3),title('tasa convergencia lineal')
  subplot(2,1,2),plot(tcc3),title('tasa convergencia cuadrtica')


%%
%
% c) Vamos a resolver el sistema de ecuaciones obtenido en el apartado a)
% mediante el mtodo de Broyden. 

  [sol4,iter4,A4] = Broyden('funcion',[1 1 1 1],1e-7,100)
  for i=1:iter4-1
      conv4(i)=norm(A4(i+1,:)-A4(i,:));
  end
  tcl4=conv4(2:end)./conv4(1:end-1);
  tcc4=conv4(2:end)./conv4(1:end-1).^2;
  subplot(2,1,1),plot(tcl4),title('tasa convergencia lineal')
  subplot(2,1,2),plot(tcc4),title('tasa convergencia cuadrtica')


%%
%
% Los resultados que proporciona el mtodo de Broyden son anlogos a los
% del mtodo de Newton, aunque se incrementa el n\'umero de iteraciones.

  [sol5,iter5,A5] = Broyden('funcion',[10 10 10 10],1e-7,100)
  for i=1:iter5-1
      conv5(i)=norm(A5(i+1,:)-A5(i,:));
  end
  tcl5=conv5(2:end)./conv5(1:end-1);
  tcc5=conv5(2:end)./conv5(1:end-1).^2;
  subplot(2,1,1),plot(tcl5),title('tasa convergencia lineal')
  subplot(2,1,2),plot(tcc5),title('tasa convergencia cuadrtica')


%%
%
% d) Vamos a disear una variante del mtodo de Broyden que no utilice la
% matriz jacobiana ni siquiera para la primera iteracin. Aproximaremos
% dicha matriz jacobiana por una matriz diagonal, utilizando en cada
% componente una aproximacin de la derivada parcial correspondiente por
% diferencias divididas, partiendo de dos estimaciones iniciales.
% Almacenamos esta variante del mtodo de Broyden en el fichero Broyden2.m.


%%
%
% function [x, iter,M] = Broyden2(funcion, x0, x1, tol, maxiter)
% x0 = x0(:);   x = x1(:);   s = x-x0;      M = [x0';x'];
% fx = feval(funcion,x0);    v = fx(:);
% fx = feval(funcion,x);     w = fx(:);     y = w-v;
% A = diag(s./y);
% incr = norm(s); iter = 1;
% while incr > tol & iter < maxiter
%     w = v;
%     fx = feval(funcion,x);    v = fx(:);  y = v-w;
%     z = -A*y;     p = -s'*z;  q = s'*A;
%     R = (s+z)*q/p;
%     A = A+R;      s = -A*v;
%     x = x+s;      M = [M;x'];
%     incr = norm(s);       iter = iter+1;
% end

%%
%
% Por otra parte, generamos el archivo funcion2.m con el sistema a resolver.
%
% function F = funcion2(x)
% F(1) = x(2)*x(3)+x(4)*(x(2)+x(3));
% F(2) = x(1)*x(3)+x(4)*(x(1)+x(3));
% F(3) = x(1)*x(2)+x(4)*(x(1)+x(2));
% F(4) = x(1)*x(2)+x(1)*x(3)+x(2)*x(3)-1;
%
% y ejecutamos la funcin Broyden2.m.

x0 = [2 2 2 2];
x1 = [1 1 1 1];
[sol6, iter6, A6] = Broyden2('funcion2',x0, x1, 1e-7, 100)
for i=1:iter6-1
    conv6(i)=norm(A6(i+1,:)-A6(i,:));
end
tcl6=conv6(2:end)./conv6(1:end-1);
tcc6=conv6(2:end)./conv6(1:end-1).^2;
subplot(2,1,1),plot(tcl6),title('tasa convergencia lineal')
subplot(2,1,2),plot(tcc6),title('tasa convergencia cuadrtica')



%%
%
% Cuando las aproximaciones iniciales las tomamos cerca de la solucin, el
% nmero de iteraciones es muy aceptable a pesar de la pobre aproximacin
% que estamos haciendo de la matriz jacobiana. 

x0 = [2 2 2 2];
x1 = [1 1 1 -1];
[sol7, iter7, A7] = Broyden2('funcion2',x0, x1, 1e-7, 100)
for i=1:iter7-1
      conv7(i)=norm(A7(i+1,:)-A7(i,:));
  end
  tcl7=conv7(2:end)./conv7(1:end-1);
  tcc7=conv7(2:end)./conv7(1:end-1).^2;
  subplot(2,1,1),plot(tcl7),title('tasa convergencia lineal')
  subplot(2,1,2),plot(tcc7),title('tasa convergencia cuadrtica')

%%
% FIN del Problema 8.10