%% 9.1. Potencial electroesttico entre dos esferas
%
% El potencial electroesttico entre dos esferas concntricas cargadas con
% potenciales distintos viene descrito por una ecuacin diferencial de 
% segundo orden con coeficientes variables y condiciones de contorno tipo 
% Dirichlet. Se aproxima la solucin del problema de frontera mediante 
% disparo lineal y diferencias finitas.
%
% Tema 9. Problema 1
% Problemas resueltos de mtodos numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson

%%
%
% a) Mtodo de disparo lineal
% 
% Este mtodo requiere resolver los dos problemas de valor inicial
%       u''=-(2/r)u',   u(2)=110,  u'(2)=0
% y
%       u''=-(2/r)u',   u(2)=0,    u'(2)=1
%
% Transformamos ambos problemas en problemas de valor inicial de primer
% orden y almacenamos en archivos .m las funciones que describen dichos
% problemas
%
% function F = F1(r,u)
% F = zeros(2,1);
% F(1) = u(2);
% F(2) = -(2/r)*u(2);
% 
% Llamamos a la funcin dispalin.m y obtenemos valores aproximados de la
% solucin del problema de frontera y de su derivada en los puntos
% elegidos, en particular en r = 3.

[x,Y] = dispalin('F1','F1',2,4,110,0,40)
  
%%
% b) Mtodo de diferencias finitas
%
% Utilizando diferencias finitas centrales transformamos el problema de
% frontera en el sistema de ecuaciones lineales:
% (1/h+1/r(i))u(i+1)-(2/h)u(i)+(1/h-1/r(i))u(i-1)=0,  i=1,2,...,9
% donde
% h=0.2, r(i)=2+ih, i=0,1,...,10, y u(i) representa el valor aproximado de
% la solucin u en r(i).

%%
% Resolvemos el sistema anterior por Gauss, mediante la barra invertida de
% MATLAB
 
h = 2/10;
r = 2:h:4;                         % nodos
d = -10*ones(9,1);                 % diagonal principal
p = (1/h)+(1./r(2:9));             % superdiagonal
q = (1/h)-(1./r(3:10));            % subdiagonal
A = diag(d)+diag(p,1)+diag(q,-1);  % matriz de coeficientes
b = [-500 0 0 0 0 0 0 0 0]';       % trminos independientes
u = A \ b

%%
% La funcin u(r)=110(4-r)/r es la solucin exacta del problema. Su
% evaluacin en r = 3 nos d u(3)=110/3, con lo que el error exacto 
% cometido por el mtodo de disparo en ese punto es 

errordisp=abs(110/3-Y(21,1))

%%
% El error cometido por diferencias finitas es

errordif=abs(110/3-u(5))

%%
%
% FIN del Problema 9.1

