%% 9.2. Problema de frontera con condiciones de Direchlet
%
% Dado un problema de frontera descrito por una ecuacin diferencial de
% segundo orden, no lineal, con condiciones de Dirichlet, aproximamos su
% solucin mediante el mtodo de disparo, utilizando para la obtencin de
% los distintos valores del parmetro t, el mtodo de la secante y el de
% Newton.
%
% Tema 9. Problema 2
% Problemas resueltos de mtodos numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson

%%
% a) Mtodo de disparo no lineal utilizando secante.
%
% Los diferentes problemas de valor inicial que debemos resolver para ir 
% acercndonos a la solucin del problema de frontera son:
%       y'' = (1/x^3)*(y-x*y')^2,     y(1)=0, y'(1)=t
% donde el parmetro t va variando hasta conseguir un valor tal
% que la solucin del problema, para ese valor de t, y(x,t), satisfaga 
% |y(2,t)-2| <= tol.
%
% Transformamos el problema en uno de valor inicial de primer orden, cuya
% funcin almacenamos en el fichero F1.m
%
% function F = F1(x,y)
% F = zeros(2,1);
% F(1) = y(2);
% F(2) = (1/x.^3)*(y(1)-x*y(2)).^2;

%%
% Llamamos a la funcin disecant.m, con los parmetros de entrada indicados
% en el problema

[x,Y,t,iter,incr] = disecant('F1',1,2,0,2,10,100,1e-7)

%%
% Obtenemos los valores aproximados de la solucin en los puntos
% elegidos, tras 6 iteraciones, un valor final del parmetro t=1.264241 y
% un valor de incr=3.762990e-11.
% Notemos que la eleccin de los valores iniciales del parmetro t juega,
% en ocasiones, un papel clave en la convergencia del mtodo. Por ejemplo, 
% si para este problema en la funcin disecant.m hubisemos tomado t0=1 y 
% t1=2 el proceso sera divergente.

%%
% b) Mtodo de disparo no lineal utilizando Newton.
%
% para utilizar el mtodo de Newton en la bsqueda del parmetro t, 
% debemos resolver en cada iteracin dos problemas de valor inicial,
% por un lado el del apartado anterior, que nos proporciona el numerador 
% y por otro
%       z'' = (2/x.^3)*(y-x*y')*z-(2/x.^2)*(y-x*y')*z', z(1)=0, z'(1)=1
% que nos proporciona el denominador.

%%
% Podemos unir ambos problemas de valor inicial en uno nico de primer
% orden, cuya funcin almacenamos en el archivo F2.m
%
% function F = F2(x,y)
% F(1:2,1) = [y(2); 1./x^3.*(y(1)-x.*y(2)).^2];
% F(3,1) = y(4);
% F(4,1) = (2./x.^3).*(y(1)-x.*y(2)).*y(3)-(2./x.^2).*(y(1)-x.*y(2)).*y(4);

%%
% Llamamos a la funcin disparoNewton.m 

[x,Y,t,iter,incr] = disparoNewton('F2',1,2,0,2,10,100,1e-7)

%% 
% Obtenemos valores aproximados de la solucin del problema de frontera 
% y de su derivada tras 11 iteraciones, un valor del parmetro t=1.264241 y
% una estimacin del error de incr=1.170296e-8.

%%
% Se puede comprobar que la solucin exacta del problema es
%       y = x*log(x/(-0.209012*x+1))-0.234472*x.
% Si evaluamos esta funcin en los nodos utilizados en el apartado b),
% podemos obtener con facilidad el error cometido en cada punto

x = 1:0.1:2;
sol = x.*log(x./(-0.2090116*x+1))-0.234472*x;
Error = abs(sol'-Y(:,1))

%%
% 
% FIN del Problema 9.2

