%% 9.3. Ecuacin de Euler
%
% Dado un problema de frontera descrito por una ecuacin diferencial de
% tercer orden, lineal, con coeficientes variables, aproximamos su
% solucin mediante el mtodo de disparo, utilizando para la obtencin de
% los distintos valores del parmetro t, el mtodo de la secante e 
% intentamos refinar los resultados mediante la tcnica de extrapolacin 
% de Richardson.
%
% Tema 9. Problema 3
% Problemas resueltos de mtodos numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson

%%
% a) Aproximacin por el mtodo de disparo.
% 
% Los diferentes problemas de valor inicial que debemos resolver
% para ir acercndonos a la solucin del problema de frontera, son: 
%       y''' =(1/x^3)*(4*x^2*y''-8*x*y'+8*y+4*log(x),
%       y(1)=0,  y'(1)=1,  y''(1)=t,
% donde el parmetro t va variando hasta conseguir un valor tal
% que la solucin del problema de valor inicial, para ese valor de
% t, y(x,t), satisfaga |y(2,t)-1| <= 1e-7.

%%
% Transformamos el problema anterior en un problema de valor inicial de
% primer orden, cuya funcin almacenamos en el archivo F1.m
%
% function F = F1(x,y)
% F = zeros(3,1);
% F(1) = y(2);
% F(2) = y(3);
% F(3) = (1/x.^3)*(4*x.^2*y(3)-8*x*y(2)+8*y(1)+4*log(x));

%%
% Llamamos a la funcin disecant03.m, con los parmetros de entrada
% descritos en el problema

[x,Yd,t,iter,incr] = disecant03('F1',1,2,0,1,1,10,100,1e-7)

%%
% Obtenemos los valores aproximados de la solucin del problema de
% frontera, para un valor del parmetro t=1.166180 y una estimacin del
% error de incr=2.220446e-16.

%%
% b) Refinamiento de los resultados por extrapolacin de Richardson.
%
% Si volvemos a aplicar el mtodo de disparo para h=1/20, h=1/40, ... vamos
% obteniendo diferentes aproximaciones de la solucin en los puntos x=1.1,
% x=1.2, ..., x=2. 
% Aplicando la tcnica de extrapolacin de Richardson podemos refinar 
% estos valores obteniendo, en principio, aproximaciones ms precisas de la
% solucin.
% Desafortunadamente, al llamar a la funcin disecant03 con 20 subintervalos

[x,Y,t,iter,incr] = disecant03('F1',1,2,0,1,1,20,100,1e-7)

%%
% Con 40 subintervalos

[x,Y,t,iter,incr] = disecant03('F1',1,2,0,1,1,40,100,1e-7)

%%
% Obtenemos los mismos resultados que se obtuvieron para 10 subintervalos, 
% en los puntos comunes. Por tanto, para este problema la tcnica de 
% extrapolacin de Richardson no consigue refinar los resultados iniciales.

%%
% c) Es sencillo comprobar que la solucin exacta del problema de frontera 
% es:
%       y = 0.055393*x+0.91691*x.^2-0.097303*x.^4-0.5*log(x)-7/8.
% Si evaluamos esta funcin en los nodos x(k)=1+0.1k, k=0,1,...,10,
% podemos obtener con facilidad el error cometido en cada punto

x = 1:0.1:2;
sol = 0.055393*x+0.91691*x.^2-0.097303*x.^4-0.5*log(x)-7/8;
Error = abs(sol'-Yd(:,1))

%%
% La grfica de la solucin exacta y de la solucin aproximada la obtenemos
% mediante las siguientes sentencias (observamos que ambas curvas se
% superponen).

plot(x,sol,x,Yd(:,1))
grid on
title('Solucin exacta y aproximada')

%%
% 
% FIN del Problema 9.3





