%% 9.4. Problema de frontera con condiciones mixtas
%
% Dado un problema de frontera descrito por una ecuacin diferencial de
% segundo orden, lineal, con coeficientes variables y condiciones de 
% contorno mixtas, aproximamos su solucin mediante el mtodo de disparo 
% no lineal, utilizando para la obtencin de los distintos valores del 
% parmetro t, el mtodo de la secante. Tambin aproximamos la solucin 
% por diferencias finitas y comparamos resultados.
%
% Tema 9. Problema 4
% Problemas resueltos de mtodos numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson

%%
% a) Solucin aproximada por el mtodo de disparo.
%
% Para aplicar el mtodo de disparo, debemos resolver los siguientes
% problemas de valor inicial:
%       u'' =(-1/r)*u'-4,  u(1)=log(1/3),  u'(1)=t,
% donde el parmetro t va variando hasta conseguir un valor tal
% que la solucin del problema de valor inicial, para ese valor de
% t, u(r,t), satisfaga
%       |u(3,t)-u'(3,t)-0.5*(log(3)-7)| <= 1e-7.

%%
% Transformamos el problema en un problema de valor inicial de primer
% orden, y almacenamos la funcin que describe dicho problema en el archivo
% F1.m
%
% function F = F1(r,u)
% F = zeros(2,1);
% F(1) = u(2);
% F(2) = (-1/r)*u(2)-4; 

%%
% Llamamos a la funcin disecant04.m con los parmetros de entrada
% indicados en el problema

alfa=log(1/3)-1;
beta=0.5*(log(3)-7);
[r,U,t,iter,incr] = disecant04('F1',1,3,alfa,beta,10,100,1e-7)

%%
% Obtenemos los valores aproximados de la solucin del problema de
% frontera, para un valor del parmetro t=-0.5 y una estimacin del
% error de incr=6.217249e-15.

%%
% b) Solucin aproximada por diferencias finitas.
%
% Mediante diferencias finitas centrales transformamos el problema de
% frontera en el sistema de ecuaciones lineales:
%
% (1-h/2r(i))u(i-1)-2u(i)+(1+h/2r(i))u(i+1)=-4h^2,  i=1,2,...,10
%
% donde
% h=0.2, r(i)=1+ih, i=0,1,...,10, y u(i) representa el valor aproximado de
% la solucin u en r(i).
% Utilizando las condiciones de contorno, la primera ecuacin resulta
%       -2u(1)+(1+h/2r(1))u(2)=-4h^2-(1-h/2r(1))alfa
% mientras que la ltima ecuacin tiene la forma
%       2u(9)+2(h(1+h/2r(10))-1)u(10)=-4h^2+2h beta(1+h/2r(10))

%%
% Resolvemos el sistema anterior utilizando el algoritmo de Crout,
% implementado en la funcin Crout.m

alfa = log(1/3)-1;  
beta = 0.5*(log(3)-7);
h=0.2;
r = 1:h:3;
b = 1+h./(2*r(2:10));                        % superdiagonal de A
c = 1-h./(2*r(3:11));   c(9) = 2;            % subdiagonal de A
a = -2*ones(10,1);                           % diagonal principal de A
a(10) = 2*(h*(1+h/(2*r(end)))-1);
d = -4*h^2*ones(10,1);                       % trminos independientes
d(1) = d(1)-(1-h/(2*r(2)))*alfa;
d(10) = d(10)+2*h*beta*(1+h/(2*r(end)));
u = Crout(a,b,c,d);
u = [alfa;u']

%%
% c) Evaluando la solucin exacta en los nodos r(k)=1+0.2k, k=0,1,...,10,
% podemos obtener el error exacto cometido en cada punto, por el mtodo de
% disparo

r = 1:0.2:3;
sol = log(r/3)+0.5*log(r)-r.^2;
ErrorDisp = abs(sol'-U(:,1))

%%
% Anlogamente, el error cometido por el mtodo de diferencias finitas
% resulta

ErrorDiff = abs(sol'-u)

%%
% 
% FIN del Problema 9.4




