%% 9.5. Temperatura en un anillo circular
%
% La temperatura en un anillo circular viene descrita por un problema de 
% frontera con una ecuacin diferencial de segundo orden, lineal, con 
% coeficientes variables y condiciones de contorno naturales. 
% Aproximamos su solucin mediante el mtodo de disparo no lineal, 
% utilizando para la obtencin de los distintos valores del parmetro t, 
% el mtodo de Newton. Tambin aproximamos la solucin por diferencias 
% finitas y comparamos resultados.
%
% Tema 9. Problema 5
% Problemas resueltos de mtodos numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson

%%
% a) Solucin aproximada por el mtodo de disparo.
%
% Tenemos que resolver los siguientes problemas de valor inicial:
%       u'' = (-1/r)u',  u(1)=t,  u'(1)=alfa-t,
% donde el parmetro t va variando hasta conseguir un valor tal, que la
% solucin del problema para ese valor de t, u(r,t), satisfaga
%       |u(3,t)+u'(3,t)-beta| <= 1e-7,
% siendo alfa = 1-1/(2log(3)) y beta = 0.5-1/(6log(3)).

%%
% Por otra parte, para poder aplicar la frmula iterativa de Newton,
% tenemos que resolver tambin el problema de valor inicial
%       z'' = (-1/r)z',  z(1)=1,  z'(1)=-1.
% Como en cada iteracin necesitamos resolver los dos problemas de valor
% inicial, vamos a unirlos en uno que lo convertimos en un problema de
% valor inicial de primer orden, cuya funcin almacenamos en el archivo
% F1.m
%
% function F = F1(r,u)
% F = [ u(2); -1./r*u(2); u(4); -1./r*u(4)];

%%
% Llamamos a la funcin disparoNewton05 con los parmetros de entrada
% fijados por el problema

alfa=1-1/(2*log(3));
beta=0.5-1/(6*log(3));
[r,u,ud,t,iter] = disparoNewton05('F1',1,3,alfa,beta,10,100,1e-7)

%%
% Obtenemos valores aproximados de la solucin y de la derivada, para un
% valor del parmetro de t=1.0000000105 y un valor de la cota de error
% incr=2.141352e-8.

%%
% b) Solucin aproximada por diferencias finitas.
%
% Aplicando diferencias centrales de orden 2, transformamos el problema de
% frontera en el sistema lineal de 11 ecuaciones con 11 incgnitas
%
%       (r(i)-h/2)u(i-1)-2r(i)u(i)+(r(i)+h/2)u(i+1)=0, 
%        i=0,1,...,10
% donde
% h=0.2, r(i)=1+ih, i=0,1,...,10, y u(i) representa el valor aproximado de
% la solucin u en r(i).
% Utilizando las condiciones de contorno, la primera ecuacin resulta
%       (2hr(0)-h^2-2r(0))u(0)+2r(0)u(1)=2h alfa(r(0)-h/2)
% mientras que la ltima ecuacin toma la forma
%       2r(10)u(9)-(2r(10)+2hr(10)+h^2)u(10)=-2h beta(r(10)+h/2)

%%
% Resolvemos el sistema anterior utilizando el algoritmo de Crout,
% implementado en la funcin Crout.m

alfa = 1-1/(2*log(3));
beta = 0.5-1/(6*log(3));
h = 2/10;
r = 1:h:3;
b = r(1:10)+h/2;    b(1) = 2*r(1);          % superdiagonal de A
c = r(2:11)-h/2;    c(10) = 2*r(11);        % subdiagonal de A
a = -2*r(1:11);                             % diagonal principal de A
a(1) = 2*h*r(1)-h^2-2*r(1);
a(11) = -h^2-2*h*r(11)-2*r(11);
d = zeros(11,1);                            % trminos independientes
d(1) = 2*h*alfa*(r(1)-h/2);
d(11) = -2*h*beta*(r(11)+h/2);
udf = Crout(a,b,c,d)                        % solucin

%%
% c) Evaluamos la solucin exacta del problema en los nodos utilizados en las
% aproximaciones anteriores para representar tanto una como otras

r = 1:0.2:3;
sol = (-1/log(3))*(log(r/3)-0.5*log(r));
plot(r,sol,r,u,r,udf)

%%
% Las grficas de la solucin exacta y la aproximacin por el mtodo de
% disparo se superponen, lo que nos d una idea sobre la eficacia de esa
% aproximacin
%
%
% 
% FIN del Problema 9.5


