%% 9.6. Un problema de frontera difcil
%
% Dado un problema de frontera descrito por una ecuacin diferencial de
% segundo orden, lineal, con coeficientes variables y condiciones de 
% contorno mixtas, aproximamos su solucin mediante el mtodo de disparo 
% no lineal, utilizando para la obtencin de los distintos valores del 
% parmetro t, el mtodo de la secante. Tambin aproximamos la solucin 
% por diferencias finitas y comparamos resultados.
%
% Tema 9. Problema 6
% Problemas resueltos de mtodos numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson

%%
% a) Solucin aproximada por el mtodo de disparo.
%
% En primer lugar, simplificamos el sistema que forman las condiciones de
% contorno, obteniendo el sistema equivalente:
%       2y(0)+2y(1)=7
%       2y'(0)+2y'(1)=1
% Introducimos dos parmetros, t y r, y debemos resolver los problemas de
% valor inicial
%       y''-xy'+x^2y=x^3,  y(0)=t, y'(0)=r,
% donde los parmetros t y r van variando hasta conseguir un valor tal, que la
% solucin del problema para ese valor de t y r, y(x,t,r), satisfaga
%       norm[2t+2y(1,t,r)-7,2r+2y'(1,t,r)-1] <= 1e-8,

%%
% Transformamos el problema de valor inicial en uno de primer orden y
% almacenamos en un archivo F1.m la funcin que describe el nuevo problema
%
% function F = F1(x,y)
% F = [ y(2); x.*y(2)-(x.^2).*y(1)+x.^3 ];
%
% Llamamos a la funcin disparo906.m

[x,Y,t,iter,incr] = disparo906('F1',0,1,10,100,1e-8)

%%
% Obtenemos la solucin aproximada del problema y su derivada, tras 20 
% iteraciones y con una estimacin del error de incr=1.08e-9.

%%
% b) Solucin aproximada por el mtodo de diferencias finitas.
%
% Aplicando diferencias finitas de segundo orden, transformamos el 
% problema  de contorno en un sistema lineal de 11 ecuaciones y 13 
% incgnitas:
%
%   (1+x(i)h/2)y(i-1)+(-2+h^2x(i)^2)y(i)+(1-x(i)/2)y(i+1)-h^2x(i)^3=0,
%   i=0,1,...,10,
%
% donde
% h=0.1, x(i)=ih, i=0,1,...,10, e y(i) representa el valor aproximado de
% la solucin y en x(i).
%
% Transformamos las condiciones de contorno, mediante diferencias finitas
% de segundo orden, y pasan a ser la primera ecuacin
%       2h(y(0)+y(10))+y(1)+y(11)-(y(-1)+y(9))=8h,
% y ltima ecuacin del sistema anterior
%       2h(y(0)+y(10))+y(-1)+y(9)-(y(1)+y(11))=6h.

%%
% Resolvemos el sistema anterior mediante la barra invertida de MATLAB,
% para lo cual ejecutamos las siguientes sentencias:

n = 10;
h = 1/n;    
x = 0:h:1;
a = x.^2*h^2-2;    a = [-1 a -1];
b = 1-x*h/2;       b = [2*h,b];
c = 1+x*h/2;       c = [c,2*h];
A = diag(a)+diag(b,1)+diag(c,-1);
A(1,3) = 1;    A(1,11) = -1;     A(1,12) = 2*h;     A(1,13) = 1;
A(13,1) = 1;   A(13,2) = 2*h;    A(13,3) = -1;      A(13,11) = 1;
d = [8*h h^2*x.^3 6*h]';
sol = A\d;
y = sol(2:end-1)                                % solucin aproximada
z = (sol(3:end)-sol(1:end-2))/(2*h)             % derivada aproximada

%%
% c) Representamos las aproximaciones de la solucin obtenidas por ambos 
% mtodos y las aproximaciones de la derivada. Podemos observar una
% perfecta identificacin entre las aproximaciones de ambos mtodos.

x=0:0.1:1;
subplot(1,2,1);
plot(x,Y(:,1),'r',x,y,'r')
title('Aproximaciones de la solucin')
subplot(1,2,2)
plot(x,Y(:,2),'g',x,z,'g')
title('Aproximaciones de la derivada')

%%
% 
% FIN del Problema 9.6





