%% 9.7. Mtodo de Rayleigh-Ritz
%
% Un cambio de variable nos permite transformar problemas de frontera, con 
% condiciones de Dirichlet, en problemas con el mismo tipo de condiciones,
% pero homogeneas.
% Aplicamos el mtodo de Rayleigh-Ritz y el mtodo de disparo a un problema
% de este tipo. Comparamos las aproximaciones obtenidas con la solucin 
% exacta.
%
% Tema 9. Problema 7
% Problemas resueltos de mtodos numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson

%%
% b) Solucin aproximada por el mtodo de Rayleigh-Ritz.
%
% Consideremos el problema de frontera
%       -z''+z = (1-1/e)x-1,  z(0)=0,  z(1)=0,
% tomamos h=0.1 y la particin del intervalo definida por x(k)=kh, 
% k=0,1,...,10. Tomamos 10 subintervalos en las aproximaciones por Simpson
% de las integrales que aparecen en la matriz de coeficientes y en los
% trminos independientes y llamamos a la funcin Rayleigh_lineal.m

x=0:0.1:1;
c = Rayleigh_lineal('ones(size(x))','ones(size(x))',...
                    '(1-exp(-1))*x-1',0:0.1:1,10)

%%
% Obtenemos los coeficientes de la funcin
%       h(x)=c(1)phi(1)+c(2)phi(2)+...+c(9)phi(9)
% que aproxima la solucin del problema de frontera, siendo phi(k),
% k=1,2,...,9, elementos finitos lineales.

%%
% c) Solucin aproximada por el mtodo de disparo lineal
%
% Aplicamos el mtodo de disparo lineal al problema de contorno
%       -y''+y=x,  y(0)=1,  y(1)=1+1/e
% para lo cual llamamos a la funcin dispalin.m

[x,Y] = dispalin('F1','F2',0,1,1,1+1/exp(1),10)

%%
% d) El error cometido por el mtodo de disparo es del orden de 1e-8

x=0:0.1:1;
sol=x+exp(-x);
Error=abs(sol'-Y(:,1))

%%
% En la grfica no se aprecia la diferencia entre la solucin exacta y la
% aproximacin por el mtodo de disparo

plot(x,sol,x,Y(:,1))
title('Solucin exacta y aproximada por disparo')

%%
% 
% FIN del Problema 9.7
