%% 9.8. Elementos finitos trigonomtricos
%
% Dado un problema de frontera descrito por una ecuacin diferencial 
% lineal, de segundo orden, con condiciones de Dirichlet, obtenemos
% valores aproximados de la solucin mediante el mtodo de disparo,
% diferencias finitas y el mtodo de Rayleigh-Ritz con elementos finitos
% trigonomtricos. Comparamos las distintas aproximaciones con la solucin
% exacta del problema.
%
% Tema 9. Problema 8
% Problemas resueltos de mtodos numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson

%%
% 
% a) Solucin aproximada por el mtodo de disparo.
%
% Aplicamos el mtodo de disparo no lineal, utilizando el mtodo de Newton
% para la obtencin de los distintos valores del parmetro t. Por tanto,
% debemos resolver dos problemas de valor inicial. Uno de ellos
%       y'' = 4y-4x(1-e^(-2)),  y(0)=0,  y'(0)=t,
% donde el parmetro t va tomando valores hasta conseguir un valor tal, que
% la solucin del problema para ese valor, y(x,t), satisfaga 
%       |y(1,t)-0|<=1e-7. 
% El otro problema nos proporciona el denominador de la expresin de Newton
%       z'' = 4z,  z(0)=0,  z'(0)=1.

%%
% Transformamos los dos problemas de valor inicial en uno nico de primer
% orden y almacenamos la funcin que lo describe en el archivo F1.m
%
% function F=F1(x,y)
% F=zeros(4,1);
% F(1)= y(2);
% F(2)= 4*y(1)-4*x*(1-exp(-2));
% F(3)= y(4);
% F(4)=4*y(3);
%
% Llamamos a la funcin disparoNewton.m

[x,Yd,t,iter,incr] = disparoNewton('F1',0,1,0,0,10,100,1e-7)

%%
% Obtenemos la solucin aproximada del problema y su derivada, para un 
% valor del parmetro t=0.387853 y una tasa de error incr=1.457168e-16.

%%
% b) Solucin aproximada por el mtodo de diferencias finitas.
%
% Teniendo en cuenta que las funciones que describen la ecuacin
% diferencial son p(x)=0, q(x)=4 y r(x)=-4x(1-e^(-2)), llamamos a la
% funcin difinlin.m para obtener la solucin aproximada del problema por
% diferencias finitas.

[X,Z] = difinlin('zeros(9,1)','4*ones(9,1)','-4*x*(1-exp(-2))',0,1,0,0,10)

%%
% c) Tcnica de extrapolacin de Richardson.
%
% Podemos refinar las aproximaciones obtenidas por diferencias finitas,
% utilizando mayor cantidad de subintervalos y la tcnica de extrapolacin
% de Richardson. 
% Trabajamos con 10, 20 y 40 subintervalos y llamamos a la funcin
% Richardsondifinlin.m

[X,Y] = Richardsondifinlin('0*x','4+0*x','-4*x*(1-exp(-2))',0,1,0,0,10,3)

%%
% La ltima fila de la matriz Y corresponde a los valores refinados por la
% tcnica de Richardson.

%%
% d) Elementos finitos trigonomtricos.
%
% Utilizamos como elementos finitos las funciones phi_k(x)=sen(kpix),
% k=1,2,...,5. Aplicando la tcnica de Rayleigh-Ritz obtenemos la funcin
% aproximante
%       phi(x)=c(1)sen(pix)+c(2)sen(2pix)+...+c(5)sen5pix)
% donde el vector c es

c=[0.15876236,-0.02534419,0.00794539,-0.00345743,0.00271544]

%%
% e) Evaluamos la solucin exacta en los nodos que hemos estado manejando

x=0:0.1:1;
sol=(1/(1+exp(2)))*exp(-2*x)-(1/(1+exp(2)))*exp(2*x)+(1-1/exp(2))*x

%%
% El error que se comete en cada punto con la aproximacin del mtodo de
% disparo es

ErrorDisparo=abs(sol'-Yd(:,1))

%%
% Si llamamos F2 al archivo .m en el que definimos la funcin aproximante
% por elementos finitos, el error que se comete en cada punto con esta 
% aproximacin es

EF=feval('F2',x,c);
ErrorEF=abs(sol'-EF)

%%
% Haciendo lo propio con cada aproximacin, vamos observando el error que
% se comete con cada una de ellas.

%%
% Representamos, por ejemplo, la solucin exacta y la solucin aproximada
% por elementos finitos

plot(x,sol,x,EF)
title('Solucin exacta y aproximada por elementos finitos')

%%
% 
% FIN del Problema 9.8

