%% 9.9. Un nmero de subintervalos elevado
%
% Dado un problema de frontera descrito por una ecuacin diferencial 
% no lineal, de segundo orden, con condiciones de contorno mixtas, 
% obtenemos valores aproximados de la solucin mediante el mtodo de 
% disparo y de diferencias finitas. Comparamos las distintas aproximaciones 
% con la solucin exacta del problema.
%
% Tema 9. Problema 9
% Problemas resueltos de mtodos numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson

%%
% 
% a) Solucin aproximada por el mtodo de disparo.
%
% Transformamos el problemas de frontera en el siguiente problema de valor
% inicial:
%            y''-yy' = e^{-2x}-x+xe^{-x},  x en [0,1], 
%            y(0) = tk,  y'(0) = 1-tk
% 
% Para cada valor de tk tenemos un problema de valor inicial cuya solucin
% y(x,tk) satisface la ecuacin diferencial y la primera de las dos
% condiciones de contorno. Tomaremos distintos valores de tk hasta que se
% cumpla el test de convergencia: 
%           | y(1,tk)-1-1/e |< tol = 10^-8 

%%
%
% Elegimos los distintos valores de tk partiendo de dos valores iniciales
% t0 = 0 y t1 = 2 y aplicando el mtodo de la secante, observamos que las
% solucin aproximada y la exacta, al representarlas grficamente, se
% superponen.

[x,YD,t,iter,incr] = disecant09('f',0,1,1+exp(-1),1000,50,1e-8);iter,incr
solex = x+exp(-x);
plot(x,YD(:,1),'r',x, solex','b'), 
title('Solucin exacta y aproximada')

%%
%
% b) Solucin aproximada por el mtodo de diferencias finitas.
%
% Aplicando diferencias finitas de segundo orden, transformamos el problema
% de contorno en un sistema lineal de 1000 ecuaciones y 1001 incgnitas:
%
% (y(i+1)-2y(i)+y(i-1))/(h^2)-y(i)((y(i+1)-y(i-1))/(2h)=
%                               =e^(-2x(i))-x(i)+x(i)e^(-x(i)),
% i=0,1,...,999,
%
% donde h = 0.001, x(i) = ih, i = 0,1,...,1000, e y(i) representa el valor
% aproximado de la solucin y en x(i).


%%
%
% Notemos que en la primera ecuacin intervienen y(-1),y(0) e y(1). Para
% evitarlo, emplearemos la primera condicin de contorno con una
% aproximacin de segundo orden de la derivada primera, obteniendo el
% sistema no lineal 1000x1000: 
%
% (-2+2h-h^2)y(0)+h^2y(0)^2+2y(1) = 2h + h^2(e^(-2x(0))-x(0)+x(0)e^(-x(0))
% y(2)-2y(1)+y(0)-h/2 y(1)(y(2)-y(0)) = h^2 (e^(-2x(1))-x(1)+x(1)e^(-x(1))
%                                .........                     
% y(999)-2y(998)+y(997)-h/2 y(998)(y(999)-y(997)) = 
%                        =h^2(e^(-2x(998))-x(998)+x(998)e^(-x(998)))
% -2y(999)+y(998)-h/2y(999)(1+1/e)-y(998)) = 
%                        =-1-1/e+h^2(e^(-2x(999))-x(999)+x(999)e^(-x(999)))
% 

%%
%
% c) Modificamos la funcin difnolin.m para adaptarla al problema,

type difnolin09

%%
%
% De nuevo las grficas de la solucin exacta y de esta aproximacin se
% superponen.

[x,YDF,iter, incre] = difnolin09('f2',0,1,1+exp(-1),999,1e-8,50);iter,incre
plot(x,YDF,'r',x, solex','b'), 
title('Solucin exacta e YDF')

%%
% d) Respecto a la precisin de los resultados, observamos como el mtodo de
% disparo tiene globalmente menor error exacto que el de diferencias
% finitas

errorD = YD(:,1)-solex; norm(errorD),max(abs(errorD))
errorDF = YDF-solex';norm(errorDF),max(abs(errorDF))

%%
% 
% FIN del Problema 9.9
