%% 9.10. Problema de frontera de orden 3
%
% Dado un problema de frontera descrito por una ecuacin diferencial 
% lineal de tercer orden, con condiciones de contorno de tipo Dirichlet, 
% obtenemos valores aproximados de la solucin mediante el mtodo de 
% disparo y de diferencias finitas. Comparamos las distintas aproximaciones 
% con la solucin exacta del problema.
%
% Tema 9. Problema 10
% Problemas resueltos de mtodos numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson

%%
% 
% a) Solucin aproximada por el mtodo de disparo.
%
% Dado que slo una de las condiciones de contorno corresponde al extremo
% inferior del intervalo [1,2],para evitar tener que trabajar con dos
% parmetros y tener que estimar su valor resolviendo en cada paso un 
% sistema de ecuaciones, podemos utilizar lo que se denomina "disparo 
% hacia atrs".
% No es ms que el mtodo de disparo pero trabajando con el intervalo [b,a]
% en lugar de [a,b].
% 
% y''' = -3(1+x)/xy'' - 6/xy' + 4y + xe^(x), x en [2,1], 
% y(2) = 1,  y'(2) = 0,  y''(2) = t
%
% donde el parmetro t va variando hasta conseguir un valor tal
% que la solucin del problema satisfaga  |y(1,t)-0| < tol = 10^{-7}.

%% 
% 
% Para elegir los distintos valores del parmetro t vamos a utilizar el
% mtodo de Newton. Como en cada iteracin del mtodo de disparo
% necesitamos resolver dos problemas de valor inicial de orden 3, los
% unimos en un nico problema de orden 6, definido en la funcin F1.m.
 
type F1

%%
% 
% Vamos a modificar la funcin disparoNewton, construida en el
% problema 9.2, para adaptarla al problema que nos ocupa. Ponemos como
% parmetros de salida la aproximacin de la solucin y(x) y
% de su derivada, yd(x).
% 
type disparoNewton10

%%
% 
% Con los parmetros de entrada que proporciona el problema obtenemos la
% aproximacin y1, el valor adecuado del parmetro y la estimacin del
% error. Representamos grficamente la curva solucin.
 
 [x,y1,yd1,t,incr] = disparoNewton10('F1',1,2,0,1,0,10,100,1e-7)
 plot(x,y1)
 
%%
% 
% b) Solucin aproximada por el mtodo de diferencias finitas.
%
% A partir de la diferencia central de orden 2 para y''(x),
% podemos deducir la siguiente diferencia finita de orden 2 para y''':
% 
%           (y(x+2h)-2y(x+h)+2y(x-h)-y(x-2h))/(2h^3)
% 
% Tomamos h = 0.1 y los nodos xi = 1+ih, i=0,1,...,10. Utilizamos
% diferencias finitas de orden 2 para aproximar las derivadas en los nodos
% x(2),...,x(9). Para el nodo x(1) es necesario utilizar una aproximacin
% de orden 1 en la derivada tercera,
%
%           (y(x+3h)-3y(x+2h)+3y(x+h)-y(x))/h^3
% 


%%
% 
% A continuacin, discretizamos la ecuacin diferencial, denotando y(xi)=y(i),
% i = 0,1,...,10. Teniendo en cuenta que y(0) = 0 e y(10) = 1, y
% estimando que y(11) = y(9) mediante una aproximacin de orden 2 en el
% ltimo nodo, obtenemos un sistema lineal con incgnitas
% y(1),y(2),...,y(9). 


%%
%
% La matriz de coeficientes de este sistema es
% pentadiagonal, con posicin (1,4) no nula. Introducimos la matriz y
% los trminos independientes en Matlab y resolvemos directamente el sistema.
 
h = 1/10;
x = 1:h:2;
a = -(12*h*(1+x(2:10))+8*h^3*x(2:10));      % diagonal principal
a(1) = -2.3644;         a(9) = a(9)+x(10);
e = 2*x(3:10)+6*h*(1+x(3:10))-6*h^2;        % primera subdiagonal
f = x(4:10);                                % segunda subdiagonal
b = -2*x(2:9)+6*h*(1+x(2:9))+6*h^3;         % primera superdiagonal
b(1) = 3.96;
c = x(2:8);     c(1) = -3.3;                % segunda superdiagonal
A = diag(a)+diag(b,1)+diag(c,2)+diag(e,-1)+diag(f,-2); % Matriz 
A(1,4) = 1.1;
d = 2*h^3*(x(2:10).^2).*exp(x(2:10));       % trminos independientes
d(1) = d(1)/2;  d(8) = d(8)-x(9);  d(9) = d(9)-6*h^2-6*h*(1+x(10))+2*x(10);
d = d';
y2 = A\d;                                    % solucin
y2=[0;y2;1]

%%
% 
% c) Si exigimos que la solucin general satisfaga las condiciones de contorno,
% obtenemos C1 = -0.0557895, C2 = -71.182862 y C3 = 71.559516. Evaluada la
% solucin exacta, calculamos los errores cometidos por las
% aproximaciones obtenidas por el mtodo de disparo y el mtodo de diferencias
% finitas. Concluimos que las aproximaciones obtenidas
% por el mtodo de diferencias finitas son bastante pobres.

C1 = -0.0557895;
C2 = -71.182862;
C3 = 71.559516;
x=1:0.1:2;
y1=flipud(y1);
solex = C1*exp(x)./x+(C2./x+C3).*exp(-2*x)+(x.^2-2*x+2).*exp(x)/27
errordisparo = abs(y1-solex')
errordiferencias = abs(y2-solex')


%%
% 
% FIN del Problema 9.10
