%% 10.1. La cuerda de un instrumento musical
%
% Las vibraciones de una cuerda tensa, estirada entre dos puntos extremos 
% fijos, vienen descritas por una ecuacin en derivadas parciales 
% hiperblica. Se resuelve la ecuacin mediante diferencias finitas, mtodo
% explcito, se representa la posicin de la cuerda en diferentes instantes
% de tiempo y se compara el comportamiento peridico que se obtiene con la
% frecuencia real deducida de la frmula fsica correspondiente.
%
% Tema 10. Problema 1
% Problemas resueltos de mtodos numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson

%%
%
% a) Diferencias finitas: mtodo explcito.
% 
% La ecuacin a resolver es 
%       u_{tt}=3136*10^6*u_{xx}
% con las condiciones de contorno
%       u(0,t)=u(80,t)=0
% y las condiciones iniciales
%       u(x,0)=0.03*x,       0<=x<=20
%       u(x,0)=0.8-0.01*x,  20<=x<=80
%       u_t(x,0)=0
%
% Definimos la funcin idnticamente nula en un archivo g.m y en otro
% archivo f.m la solucin en el instante t=0.
%
% function y=f(x)
% y=0.03*x.*(x<=20)+(0.8-0.01*x).*(x>20);
%
% Llamamos a la funcin exponda.m, utilizando el paso temporal ms grande
% posible que garantize la convergencia. Para este problema ese paso
% resulta ser k=1/5600. 

U = exponda(8,0,80,5600,1,56000,'f','g','g','g')

%%
% La posicin de la cuerda en el instante t=1 la encontramos en la columna
% 5601 de la matriz U

U(:,5601)

%%
% b) Si ejecutamos plot(U) obtenemos 5601 curvas que se corresponden con 
% las posiciones de la cuerda  en los distintos instantes de tiempo.

plot(U)

%%
% Desde luego el movimiento es peridico y vamos a determinar cada cuntos 
% pasos de tiempo se repiten los valores de u. Para ello comparamos cada 
% columna con la inicial, mediante la norma eucldea y una tolerancia de 
% 10^(-12).

tol = 10^(-12);     
j = 1;      
incr = tol+1;
while incr>tol
   incr = norm(U(:,1)-U(:,j+1));
   j = j+1;
end

%%
% Obtenemos j=17, por lo que el periodo es 16*k. En consecuencia, la
% frecuencia ser 1/(16*k)=350 ciclos/s.
% Si utilizamos la frmula fsica para determinar la frecuencia obtenemos
% exactamente el mismo valor.
%
%
% FIN del Problema 10.1
