%% 10.2. La ecuacin de ondas con condiciones Dirichlet
%
% Ecuacin de ondas, con condiciones de contorno tipo Dirichlet y 
% condiciones iniciales. Se aproxima la solucin del problema mediante
% diferencias finitas, mtodos explcito e implcito, y se comparan las
% aproximaciones obtenidas con la solucin exacta.
%
% Tema 10. Problema 2
% Problemas resueltos de mtodos numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson

%%
%
% a) Diferencias finitas: mtodo explcito.
% 
% Definimos la funcin idnticamente nula en un archivo g.m, la solucin 
% en el instante t=0 en un archivo f.m y la solucin exacta del problema en
% un archivo sol.m.
%
% Llamamos a la funcin exponda.m, utilizando 10 subintervalos espaciales y
% 120 temporales (k=0.025).

Uex = exponda(10,0,1,120,3,1,'f','g','g','g')

%%
% La solucin en el instante t=3 aparece en la columna 121 de la matriz
% Uex.

Uex(:,121)

%%
% Si ejecutamos mesh(Uex) tendremos una representacin del comportamiento de
% la solucin aproximada obtenida.

mesh(Uex)

%%
% Para determinar el error exacto cometido en esta aproximacin, restamos 
% la columna 121 de Uex y la solucin exacta para t=3 y x=0:0.1:1.

x=0:0.1:1;
t=3;
u=feval('sol',x,t);
u=u(:);
Errex=abs(u-Uex(:,121))

%%
% Si repetimos los pasos anteriores tomando 20 subintervalos temporales
% (k=0.15), observamos que las aproximaciones son peores. La razn la
% podemos encontrar en que, en este caso, el parmetro de Courant vale 1.5,
% por lo que no tenemos asegurada la convergencia.
%

%%
% 
% b) Diferencias finitas: mtodo implcito.
%
% Llamamos a la funcin imponda.m, utilizando 10 subintervalos espaciales y
% 120 temporales (k=0.025).

Uim = imponda(10,0,1,120,3,1,'f','g','g','g')

%%
% La solucin aproximada en el instante t=3 la encontramos en la columna
% 121 de la matriz Uim.

Uim(:,121)

%%
% Al igual que en el caso explcito, podemos determinar el error real
% cometido, en el instante t=3, restando la columna 121 de Uim y el valor
% de la solucin exacta en t=3 y x=0:0.1:1.

x=0:0.1:1;
t=3;
u=feval('sol',x,t);
u=u(:);
Errim=abs(u-Uim(:,121))

%%
% La magnitud del error para ambas aproximaciones es similar.
%
% Podemos representar la solucin exacta y las dos aproximaciones
% obtenidas, mediante los mtodos explcito e implcito, para 120
% subintervalos temporales.

x=0:0.1:1;
t=3;
u=feval('sol',x,t);
plot(x,u,x,Uex(:,121),x,Uim(:,121))
grid on

%%
%
% FIN del Problema 10.2
