%% 10.3. Ecuacin parablica con condiciones Dirichlet
%
% Ecuacin en derivadas parciales parablica, con condiciones de contorno 
% tipo Dirichlet y condiciones iniciales. Se aproxima la solucin del 
% problema mediante diferencias finitas, mtodo explcito, y mediante
% el mtodo de Crank-Nicholson. Se comparan las aproximaciones obtenidas 
% con la solucin exacta.
%
% Tema 10. Problema 3
% Problemas resueltos de mtodos numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson

%%
%
% a) Diferencias finitas: mtodo explcito.
% 
% La funcin idnticamente nula aparece en las condiciones de contorno, por
% lo que la definimos en un archivo g.m. Por otra parte, definimos la 
% condicin inicial en el archivo f.m y la solucin exacta del problema en
% el archivo sol.m.
%
% Para este problema, el parmetro de Courant es 25*k/pi^2, donde k es el
% paso temporal. Para garantizar la convergencia necesitamos que dicho
% parmetro sea menor o igual que 1/2, por lo que el valor mayor que puede
% tomar k es pi^2/50.
%
% Llamamos a la funcin expcalor03.m

Uex = expcalor03(10,0,4,50,pi^2,4/pi^2,'f','g','g')

%%
% La solucin aproximada en el instante final t=pi^2 la encontramos en la
% columna 51 de la matriz Uex.

Uex(:,51)

%%
% Para determinar el error exacto cometido por esta aproximacin, basta con
% restar la ltima columna de Uex y la solucin exacta para t=pi^2 y 
% x=0:0.4:4.

x=0:0.4:4;
t=pi^2;
u=feval('sol',x,t);
u=u(:);
Errex=abs(u-Uex(:,51))

%%
% Si repetimos el procedimiento tomando 1000 y 5000 subintervalos
% temporales, podemos constatar que las nuevas aproximaciones no mejoran
% substancialmente las obtenidas con 50 subintervalos.

%%
% 
% b) Diferencias finitas: mtodo de Crank-Nicholson.
%
% Llamamos a la funcin nicalor.m con los parmetros de entrada fijados por
% el problema y 50 subintervalos temporales

Uni = nicalor(10,0,4,50,pi^2,4/pi^2,'f','g','g')

%%
% En la ltima columna de la matriz Uni aparecen los valores aproximados de
% la solucin en el instante final. 

Uni(:,51)

%%
% Al igual que en el mtodo explcito, calculamos el error cometido por 
% esta aproximacin.

x=0:0.4:4;
t=pi^2;
u=feval('sol',x,t);
u=u(:);
Errni=abs(u-Uni(:,51))

%%
% Una simple comprobacin nos permitir afirmar que las aproximaciones
% obtenidas por el mtodo de Crank-Nicholson con mucho ms precisas que las
% obtenidas por el mtodo explcito. Representamos la solucin exacta y las
% aproximaciones obtenidas en el instante t=pi^2. 

plot(x,u,x,Uex(:,51),x,Uni(:,51))
title('Solucin exacta y aproximaciones')

%%
%
% FIN del Problema 10.3

