function [U,iter] = Jacobi(a,b,c,d,nx,ny,f,f0,f1,g0,g1,tol,maxiter)
% Sintaxis: 
% [U,iter] = Jacobi(a,b,c,d,nx,ny,'f','f0','f1','g0','g1',tol,maxiter)
% a,b,c,d: extremos de los intervalos de variacin de x e y
% nx,ny: nmero de subintervalos para x e y
% f: archivo.m con el trmino independiente de la ecuacin de Poisson
% f0,f1: archivos.m con la condicin de frontera en los puntos 
% (x,c) y (x,d), respectivamente.
% g0,g1: archivos.m con la condicin de frontera en los puntos 
% (a,y) y (b,y), respectivamente.
%
h = (b-a)/nx;       
x = a:h:b;
k = (d-c)/ny;       
y = c:k:d;
alfa = k/h;   
alfa2 = alfa^2;   
beta = 2*(1+alfa2);
f0x = feval(f0,x);    
f1x = feval(f1,x);
g0y = feval(g0,y);    
g1y = feval(g1,y);
U = zeros(nx+1,ny+1);                         % matriz U inicial
U(:,1) = f0x(:);                                % condiciones contorno
U(:,ny+1) = f1x(:);
U(1,:) = g0y;
U(nx+1,:) = g1y;

% Mtodo de Jacobi
    incr = tol+1;       iter = 0;
    v = U;
    C = 2:nx; L = 1:nx-1; R = 3:nx+1;
    while incr>tol & iter<maxiter
        for j = 2:ny
            for i=2:nx
            v(i,j) = (alfa2*(U(i-1,j)+U(i+1,j))+U(i,j-1)+U(i,j+1)-...
                     k^2*feval(f,x(i),y(j)))/beta;
            end
        end    
        incr = max(max(abs(U-v)));
        iter = iter+1;
        U = v;
    end

