%% 10.4. La ecuacin de Poisson
%
% Ecuacin de Poisson, con condiciones de contorno tipo Dirichlet. 
% Se aproxima la solucin del problema mediante los
% mtodos de Jacobi y sobrerrelajacin. Se comparan los resultados
% obtenidos con la solucin exacta.
%
% Tema 10. Problema 4
% Problemas resueltos de mtodos numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson

%%
% 
% a) Aplicando diferencias finitas centrales de orden 2, la ecuacin de
% Poisson se transforma en el sistema lineal
% 
% alpha^2(u(i-1,j)+u(i+1,j))+u(i,j-1)+u(i,j+1)-2(1+alpha^2)u(i,j) = 
%                               =k^2 f(xi,yj),
% i,j=1,2,...,9
% 
% donde alpha = k/h, xi = 0+ih, i = 0,1,...,10, e yj = 0+jk, j = 0,1,...,10,
% son los nodos elegidos en x e y respectivamente, u(i,j) = u(xi,yj) y
% la funcin del trmino independiente de la ecuacin diferencial es  
% f(x,y) = -(cos(x+y)+cos(x-y)). Los valores de u para i=0, i=10, j=0, j=10
% vienen dados por las condiciones de contorno del problema. 


%%
%
% b) Para resolver el sistema  por cualquier mtodo iterativo despejamos
% u(i,j) de la expresin anterior
% 
% 2(1+alpha^2)u(i,j) = alpha^2(u(i-1,j)+u(i+1,j))+u(i,j-1)+u(i,j+1)-
%                      -k^2 f(xi,yj)
% 
% Construimos los archivos .m necesarios para definir las funciones que
% describen el trmino independiente y las condiciones de contorno.
% Modificamos la parte de la funcin Laplace correspondiente al mtodo de
% Jacobi para adaptarla a ecuaciones diferenciales elpticas con trmino
% independiente no nulo y la almacenamos como Jacobi.m.

[UJ,iter] = Jacobi(0,pi,0,pi/2,10,10,'f','f0','f1','g0','g1',1e-10,1000)
mesh(UJ)


%%
%
% c) Aplicamos el mtodo de sobrerrelajacin para distintos valores del
% parmetro w. Para ello llamamos a la funcin Sor con los parmetros
% de entrada utilizados en el apartado anterior, aumentando el nmero
% mximo de iteraciones. En todos los casos la solucin obtenida
% coincide con la proporcionada por el mtodo de Jacobi.

k = 1;
for w = 0.2:0.2:1.8
[U,iter(k)] = Sor(0,pi,0,pi/2,10,10,'f','f0','f1','g0','g1',1e-10,2000,w);
k = k+1;
end
M = [0.2:0.2:1.8;iter]

%%
%
% Los datos obtenidos  muestran que el valor ptimo para el parmetro
% w se encuentra en el intervalo ]1.4, 1.8[. Repetimos el proceso tomando 
% valores en este intervalo y obtenemos los siguientes resultados:

k = 1;
for w = 1.42:0.02:1.6
[U,iter(k)] = Sor(0,pi,0,pi/2,10,10,'f','f0','f1','g0','g1',1e-10,2000,w);
k = k+1;
end
M = [1.42:0.02:1.6;iter]


%%
%
% Estos resultados nos confirman que el valor ptimo del parmetro del
% mtodo de sobrerrelajacin est alrededor de 1.54. Si evaluamos la
% expresin del enunciado del problema obtenemos 


wopt = 4/(2+sqrt(4-(2*cos(pi/10))^2))


%%
%
% d) Podemos construir y representar una matriz S, del mismo tipo que la U
% del apartado a), que contenga los valores exactos de la solucin en los
% distintos puntos del mallado:

x = 0:pi/10:pi;       
y = 0:pi/20:pi/2;
for i = 1:11
   for j = 1:11
    S(i,j) = cos(x(i))*cos(y(j));
   end
end
mesh(S)

%%
%
% Calculamos la matriz del error y el error mximo para la aproximacin
% U obtenida por cualquier mtodo. 

EJ = abs(S-UJ)
EmaxJ = max(max(EJ))
[US,iter] = Sor(0,pi,0,pi/2,10,10,'f','f0','f1','g0','g1',1e-10,2000,wopt);
ES = abs(S-UJ)
EmaxS = max(max(ES))


%%
%
% FIN del Problema 10.4


