function [U,iter] = ADI1005(a,b,c,d,h,k,f0,f1,tol,maxiter)
% Sintaxis: [U,iter] = ADI1005(a,b,c,d,h,k,'f0','f1',tol,maxiter)
% a,b: extremos de variacin de la x
% c,d: extremos de variacin de la y
% h,k: pasos para la x e y
% f0,f1: archivos .m con las condiciones de Dirichlet
% tol: tolerancia
% maxiter: nmero mximo de iteraciones
x = a:h:b;
nx = (b-a)/h;  
ny = (d-c)/k;
alfa = k/h;     
alfa2 = alfa^2;    
beta = 2*(1+alfa2);
f0x = feval(f0,x);  
f1x = feval(f1,x);
g0 = f0x(1):-k/2:f1x(1);                            % estimada
g1 = f0x(nx+1):-k/2:f1x(nx+1);                      % estimada
u0 = (sum(f0x+f1x)/(nx+1)+sum(g0+g1)/(ny+1))/4;     % inicializacin
U = u0*ones(nx+1,ny+1);
U(1,:) = g0;      U(nx+1,:) = g1;
U(:,1) = f0x(:);  U(:,ny+1) = f1x(:);

% diagonales del bloque columna
d0c = beta*ones(1,nx-1);    d0c = [beta-h*alfa2 d0c beta-2*h*alfa2];
d1c = -alfa2*ones(1,nx-1);  d1c = [-2*alfa2 d1c];
d2c = -alfa2*ones(1,nx-1);  d2c = [d2c -2*alfa2];

% diagonales del bloque fila
d0fila1 = (beta-h*alfa2)*ones(1,ny-1);
d1fila1 = -ones(1,ny-2);
d0f = beta*ones(1,ny-1);
d1f = -alfa2*ones(1,ny-2);
d0filan1 = (beta-2*h*alfa2)*ones(1,ny-1);
d1filan1 = -ones(1,ny-2);
incr = tol+1;  iter = 0;
while incr>tol & iter<maxiter
    incr = 0;
    for j = 2:ny                    % bloque columna
        b = U(:,j-1)+U(:,j+1);
        v = Crout(d0c,d1c,d2c,b);
        incr = max(incr,norm(v'-U(:,j)));
        U(:,j) = v';
     end
     b = 2*alfa2*U(2,2:ny);         % bloque fila
     b(1) = b(1)+U(1,1);   b(ny-1) = b(ny-1)+U(1,ny+1);
     v = Crout(d0fila1,d1fila1,d1fila1,b');
     U(1,2:ny) = v;
     for i = 2:nx
        b = U(i-1,2:ny)+U(i+1,2:ny);
        b(1) = b(1)+alfa2*f0x(i); b(ny-1) = b(ny-1)+alfa2* f1x(i);
        v = Crout(d0f,d1f,d1f,b);
        incr = max(incr,norm(v-U(i,2:ny)));
        U(i,2:ny) = v;
     end
     b = 2*alfa2*U(nx,2:ny);
     b(1) = b(1)+U(nx+1,1);  b(ny-1) = b(ny-1)+U(nx+1,ny+1);
     v = Crout(d0filan1,d1filan1,d1filan1,b);
     U(nx+1,2:ny) = v;
     iter = iter+1;
end
