function [U,iter] = GS1005(a,b,c,d,h,k,f0,f1,tol,maxiter)
% Sintaxis: [U,iter] = GS1005(a,b,c,d,h,k,'f0','f1',tol,maxiter)
% a,b: extremos de variacin de la x
% c,d: extremos de variacin de la y
% h,k: pasos para la x e y
% f0,f1: archivos .m con las condiciones de Dirichlet
% tol: tolerancia
% maxiter: nmero mximo de iteraciones
x = a:h:b;
nx = (b-a)/h;  
ny = (d-c)/k;
alfa = k/h;         
alfa2 = alfa^2;
beta = 2*(1+alfa2); 
beta2 = beta-2*h*alfa2;
f0x = feval(f0,x);   
f1x = feval(f1,x);
g0 = f0x(1):-k/2:f1x(1);                            % estimada
g1 = f0x(nx+1):-k/2:f1x(nx+1);                      % estimada
u0 = (sum(f0x+f1x)/(nx+1)+sum(g0+g1)/(ny+1))/4;     % inicializacin
U = u0*ones(nx+1,ny+1);
U(1,:) = g0;      
U(nx+1,:) = g1;
U(:,1) = f0x(:);  
U(:,ny+1) = f1x(:);
incr = tol+1;  
iter = 0;
while incr>tol & iter<maxiter
  incr = 0;
  for i = 2:nx
   for j = 2:ny
     U(1,j) = (2*alfa2*U(2,j)+U(1,j-1)+U(1,j+1))/(beta-h*alfa2);
     v = (alfa2*(U(i-1,j)+U(i+1,j))+U(i,j-1)+U(i,j+1))/beta;
     U(nx+1,j) = (2*alfa2*U(nx,j)+U(nx+1,j-1)+U(nx+1,j+1))/beta2;
     incr = max(incr,abs(U(i,j)-v));
     U(i,j) = v;
   end
 end
 iter = iter+1;
end
