function [U,iter] = Laplace(a,b,c,d,nx,ny,f0,f1,g0,g1,tol,maxiter)
% Sintaxis: 
% [U,iter] = Laplace(a,b,c,d,nx,ny,'f0','f1','g0','g1',tol,maxiter)
% a,b,c,d: extremos de los intervalos de variacin de x e y
% nx,ny: nmero de subintervalos para x e y
% f0,f1: archivos.m con la condicin de frontera en los puntos 
% (x,c) y (x,d), respectivamente.
% g0,g1: archivos.m con la condicin de frontera en los puntos 
% (a,y) y (b,y), respectivamente.
%
h = (b-a)/nx;       
x = a:h:b;
k = (d-c)/ny;       
y = c:k:d;
alfa = k/h;   
alfa2 = alfa^2;   
beta = 2*(1+alfa2);
f0x = feval(f0,x);    
f1x = feval(f1,x);
g0y = feval(g0,y);    
g1y = feval(g1,y);
u0 = (sum(f0+f1)/(nx+1)+sum(g0+g1)/(ny+1))/4;   % matriz U inicial
U = u0*ones(nx+1,ny+1);
U(:,1) = f0x(:);                                % condiciones contorno
U(:,ny+1) = f1x(:);
U(1,:) = g0y;
U(nx+1,:) = g1y;

k = menu('Elige el metodo: ','Jacobi','Gauss-Seidel','Sobrerrelajacion',...
        'Implicito de Direcciones Alternadas (ADI)');

% Mtodo de Jacobi
if k == 1
    incr = tol+1;       iter = 0;
    v = U;
    C = 2:nx; L = 1:nx-1; R = 3:nx+1;
    while incr>tol & iter<maxiter
        for j = 2:ny
            v(C,j) = (alfa2*(U(L,j)+U(R,j))+U(C,j-1)+U(C,j+1))/beta;
        end
        incr = max(max(abs(U-v)));
        iter = iter+1;
        U = v;
    end
end

% Mtodo de Gauss-Seidel
if k == 2
    incr = tol+1;       iter = 0;
    while incr>tol & iter<maxiter
    incr = 0;
        for i = 2:nx
            for j = 2:ny
                v = (alfa2*(U(i-1,j)+U(i+1,j))+U(i,j-1)+U(i,j+1))/beta;
                incr = max(incr,abs(U(i,j)-v));
                U(i,j) = v;
            end
        end
        iter = iter+1;
    end
end

% Mtodo de Sobrerrelajacin
if k == 3
    w = input('Factor de relajacin 0<w<2: ')
    incr = tol+1;       iter = 0;
    while incr>tol & iter<maxiter
    incr = 0;
      for i = 2:nx
        for j = 2:ny
          v = (1-w)*U(i,j)+w*(alfa2*(U(i-1,j)+U(i+1,j))+U(i,j-1)+U(i,j+1))/beta;
          incr = max(incr,abs(U(i,j)-v));
          U(i,j) = v;
        end
      end
    iter = iter+1;
    end
end

% Algoritmo Implcito de Direcciones Alternadas
if k == 4
    d0f = beta*ones(1,nx-1);        % diagonales del sistema
    d1f = -alfa2*ones(1,nx-2);      % para bloques fila
    d0c = beta*ones(1,ny-1);        % diagonales del sistema
    d1c = -alfa2*ones(1,ny-2);      % para bloques columna
    incr = tol+1;
    iter = 0;
    while incr>tol & iter<maxiter
        incr = 0;                   % bloque fila
        for i = 2:nx
            b = U(i-1,2:ny)+U(i+1,2:ny);   % vector de tr. independientes
            b(1) = b(1)+alfa2*f0(i);
            b(ny-1) = b(n-1)+alfa2* f1(i);
            v = crout(d0c,d1c,d1c,b);
            incr = max(incr,norm(v-U(i,2:ny)));
            U(i,2:ny) = v;
        end
        for j = 2:ny                 % bloque columna
            b = U(2:nx,j-1)+U(2:nx,j+1);   % vector de tr. independientes
            b(1) = b(1)+alfa2*g0(j);
            b(nx-1) = b(m-1)+alfa2* g1(j);
            U(2:nx,j) = crout(d0f,d1f,d1f,b)';
        end
        iter = iter+1;
    end
end
