%% 10.5. La ecuacin de Laplace
%
% Ecuacin en derivadas parciales elptica, homognea, con condiciones de 
% contorno mixtas. Mediante diferencias finitas se transforma el problema
% en un sistema de ecuaciones lineales. Se resuelve dicho sistema mediante
% los mtodos de Gauss-Seidel y de direcciones alternadas, comparando los
% resultados obtenidos.
%
% Tema 10. Problema 5
% Problemas resueltos de mtodos numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson

%%
%
% a) Transformacin del problema en un sistema lineal.
%
% Elegimos los nodos donde vamos a aproximar la solucin de la ecuacin en
% derivadas parciales.
%       x(i)=-1+ih,     h=2/nx,     i=0,1,...,nx
%       y(j)=-1+jk,     k=2/ny,     i=0,1,...,ny
% Aplicando fiferencias finitas de orden 2, tanto a la ecuacin en
% derivadas parciales como a las condiciones de contorno naturales, y
% utilizando la notacin u(i,j) para denotar la solucin u en el punto
% (x(i),y(j)), i=0,1,...,nx; j=1,2,...,ny-1, obtenemos el sistema lineal:
%
%%
%   u(0,j)=(2a^2u(1,j)+u(0,j-1)+u(0,j+1))/(2(1+a^2)-ha^2), 
%   j=1,2,...,ny-1,
%   u(i,j)=(a^2(u(i+1,j)+u(i-1,j))+u(i,j-1)+u(i,j+1))/(2(1+a^2)),
%   i=1,2,...,nx-1; j=1,2,...,ny-1,
%   u(nx,j)=(2a^2u(nx-1,j)+u(nx,j+1)+u(nx,j-1))/(2(1+a^2)-2ha^2),
%   j=1,2,...ny-1,
% 
% donde a=k^2/h^2.
%
%%
%
% b) Resolucin del sistema por el mtodo de Gauss-Seidel.
%
% Modificamos el programa de Gauss-Seidel estandar, para adaptarlo a las
% particularidades de este problema, y lo almacenamos en la funcin
% GS1005.m. Definimos en el archivo f0.m la funcin identicamente 1 y en 
% f1.m la funcin identicamente nula. Ejecutamos la citada funcin 

[U,iter] = GS1005(-1,1,-1,1,0.25,0.25,'f0','f1',1e-5,1000)

%%
% Tras 170 iteraciones obtenemos una matriz U, de tamao 9x9, donde en cada
% columna aparecen valores aproximados de la solucin, para los distintos x
% utilizados y un y fijo. Por ejemplo, la solucin para todos los x e y=0
% se encuentra en la columna 5 de U.

U(:,5)

%%
% c) Resolucin del sistema por el mtodo de direcciones alternadas.
%
% Modificamos el mtodo de direcciones alternadas, programado en el archivo
% Laplace.m, para adaptarlo a las caractersticas de este problema.
% Construimos el archivo ADI1005.m que recoge estas modificaciones.
% Ejecutamos la funcin ADI1005 con los parmetros de entrada fijados por
% el problema:

[U,iter] = ADI1005(-1,1,-1,1,0.25,0.25,'f0','f1',1e-5,1000)

%%
% Con tan slo 61 iteraciones, el mtodo nos proporciona una matriz U, de
% tamao 9x9, cuya interpretacin es idntica a la realizada en el apartado
% b).
%
%
% FIN del Problema 10.5
