%% 10.6. Ecuacin parablica con condiciones mixtas
%
% Ecuacin en derivadas parciales parablica, con condiciones de 
% contorno mixtas y condicin inicial. Mediante diferencias finitas 
% transformamos el problema en un esquema en diferencias explcito, que nos
% proporcionar una solucin aproximada del problema.
% Aproximamos tambin la solucin del problema por el mtodo de
% Crank-Nicholson y comparamos resultados.
%
% Tema 10. Problema 6
% Problemas resueltos de mtodos numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson

%%
%
% a) Esquema en diferencias explcito.
%
% Tomamos los nodos espaciales y temporales:
%
%       x(i)=ih,    i=0,1,...,nx,   h=1/nx,
%       t(j)=jk,    j=0,1,...,nt,   k=1/nt,
%
% donde nx y nt es el nmero de subintervalos espaciales y temporales,
% respectivamente

%%
% Teniendo en cuenta que no conocemos la solucin para x=0 y x=1, lo que
% diferencia este problema del problema estndar, el esquema en diferencias
% explcito viene descrito por la ecuaciones:
%
%   u(0,j+1)=(1-2a)u(0,j)+2au(1,j)+k exp(-t(j)),             j=0,1,...,nt-1
%   u(i,j+1)=(1-2a)u(i,j)+a(u(i+1,j)+u(i-1,j))+k exp(-t(j)), i=1,2,...,nx-1
%                                                            j=0,1,...,nt-1
%   u(nx,j+1)=(1-2a-2ah)u(nx,j)+2au(nx-1,j)+k exp(-t(j)),    j=0,1,...,nt-1
%     

%%
% b) Solucin aproximada por el mtodo explcito.
%
% En este caso el valor del parmetro de Courant es 0.05, lo que nos
% asegura la convergencia del mtodo. El instante final es T=1, por lo que
% el nmero de subintervalos temporales que vamos a utilizar es 2000.
% Definimos en un archivo f.m la funcin que describe la condicin inicial
% y llamamos a la funcin expcalor06.

Uex = expcalor06(10,0,1,2000,1,1,'f')

%%
% La matriz Uex tiene 11 filas y 2001 columnas. Cada columna corresponde al
% valor aproximado de la solucin en un instante de tiempo. La ltima
% columna contiene la solucin en el instante t=1.

Uex(:,2001)

%%
% La grfica de la matriz Uex, en tres dimensiones, nos dar una idea del
% comportamiento de las aproximaciones obtenidas por el mtodo explcito.

mesh(Uex)

%%
% c) Solucin aproximada por el mtodo de Crank-Nicolson
%
% Al aplicar la idea de Crank-Nicholson a la ecuacin en derivadas
% parciales, teniendo en cuenta que las condiciones de contorno no son tipo
% Dirichlet, obtenemos un esquema en diferencias matricial
%
%       A u^(j+1)=B u^(j)+p_j,  j=0,1,...,nt-1
%
% donde u^(j) representa la solucin en el instante t=t_j.

%%
% Tanto A como B son matrices tridiagonales:
%           
%       A=tridiagonal(a1,a2,a3)
%       B=tridiagonal(b1,b2,b3)
%
% donde 
%               a1=[1+a,...,1+a,1+a+ha] diagonal principal
%               a2=[-a/2,...,-a/2,-a] subdiagonal
%               a3=[-a,-a/2,...,-a/2] superdiagonal
%               b1=[1-a,...,1-a,1-a-ha] diagonal principal
%               b2=-a2 subdiagonal
%               b3=-a3 superdiagonal
% con a=k/h^2 y
%
% p_j=(k/2)(exp(-t(j)+exp(-t(j+1)))[1,1,...,1]^T.

%%
% Llamamos a la funcin nicalor06, en la que hemos implementado el mtodo
% de Crank-Nicholson que acabamos de describir.

Uni = nicalor06(10,0,1,2000,1,1,'f')

%%
% La matriz Uni tiene 11 filas y 2001 columnas. Cada columna corresponde al
% valor aproximado de la solucin en un instante de tiempo. La ltima
% columna contiene la solucin en el instante t=1.

%%
% La grfica de la matriz Uni, en dos dimensiones, nos dar 2001 curvas 
% correspondientes a cada una de las columnas de la matriz.

plot(Uni)

%%
%
% FIN del Problema 10.6