%% 10.7. La ecuacin del telgrafo
%
% Consideramos la ecuacin en derivadas parciales del telgrafo, con
% condiciones de contorno de Dirichlet y condiciones iniciales. Mediante
% diferencias finitas transformamos el problema en un esquema en
% diferencias explcito, que nos proporcionar una solucin aproximada del
% problema. Aproximamos tambin la solucin del problema por el mtodo
% implcito y comparamos resultados.
%
% Tema 10. Problema 7
% Problemas resueltos de mtodos numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson

%%
%
% a) Esquema en diferencias explcito.
%
% Tomamos los nodos espaciales x(i)=ih, i=0,1,...,nx, h=1/nx y temporales
% t(j)=jk, j=0,1,...,nt, k=0.5/nt. Utilizamos diferencias finitas centrales
% para aproximar las derivadas en la ecuacin diferencial y denotamos por
% u(i,j) la funcin incgnita evaluada en un par de nodos u(x(i),t(j)).
% En este caso, el esquema en diferencias explcito viene descrito por la
% ecuaciones:
%
%  u(i,1) = k^2(2/k^2-2-2/h^2)/2 sin(pi x(i))
%                 +k^2/(2h^2)(sin(pix(i+1))+sin(pi x(i-1))), i=1,2,...,nx-1
%
%  (1/k^2+1/(2k))u(i,j+1)=(2/k^2-2-2/h^2)u(i,j)+(u(i+1,j) +
%                          +u(i-1,j))/h^2+(1/(2k)-1/k^2)u(i,j-1), 
%                                           i=1,2,...,nx-1  j=1,1,...,nt-1
%     

%%
%
% Adaptamos el archivo exponda.m a nuestro problema y lo almacenamos en la
% funcin exponda07.m. Obtenemos la solucin en el ltimo instante y la
% representacin grfica en los instantes t = 0.1, 0.2,..., 0.5.

Ue = exponda07(10,0,1,100,0.5,'f','g','l','r');
Ue(:,101)
plot(0:.1:1,[Ue(:,21),Ue(:,41),Ue(:,61),Ue(:,81),Ue(:,101)])
title('Curvas solucin mtodo explcito')

%%
%
% b) El esquema en diferencias implcito para este problema es:
%
% (1/k^2+1/(2k)+1/h^2)u(i,j+1)-1/(2h^2)(u(i+1,j+1)+u(i-1,j+1)) = 
%       = (2/k^2-2)u(i,j)+(1/(2k)-1/k^2)-1/h^2)u(i,j-1)+
%         +1/(2h^2)(u(i+1,j-1)+u(i-1,j-1)),  
%                                           i=1,2,...,nx-1 , j=1,2,...,nt-1
%


%%
% Al igual que con el mtodo explcito, modificamos el archivo imponda.m 
% para adaptarlo a este problema, almacenndolo en imponda07.m. 

Ui = imponda07(10,0,1,100,0.5,'f','g','l','r');
Ui(:,101)
plot(0:.1:1,[Ui(:,21),Ui(:,41),Ui(:,61),Ui(:,81),Ui(:,101)])
title('Curvas solucin mtodo implcito')

%%
%
% Los resultados obtenidos por ambos mtodos para el instante final son 
% bastante parecidos. 

max(abs(Ue(:,101)-Ui(:,101)))

%%
%
% FIN del Problema 10.7