%% 10.8. Cuerda vibrante en un medio elstico
%
% Ecuacin de ondas que describe la vibracin de una cuerda en un medio 
% elstico, con extremos fijos y velocidad inicial nula. Aproximamos la
% solucin del problema mediante el mtodo explcito y el mtodo implcito.
% Comparamos resultados y representamos las soluciones aproximadas en 
% algunos instantes de tiempo.
%
%
% Tema 10. Problema 8
% Problemas resueltos de mtodos numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson

%%
%
% a) Esquema en diferencias explcito.
%
% Tomamos los nodos espaciales y temporales
%           x(i)=ih,    h=1/nx, i=0,1,...,nx
%           t(j)=jk,    k=1/nt, j=0,1,...,nt
% donde nx y nt son el nmero de subintervalos espaciales y temporales,
% respectivamente.
% El esquema en diferencias explcito para este problema tiene la
% expresin:
%
%   u(i,1)=(1-a^2-k^2/2)u(i,0)+(a^2/2)(u(i+1,0)+u(i-1,0)), 
%           i=1,2,...,nx-1
%   u(i,j+1)=(2-2a^2-k^2)u(i,j)+a^2(u(i+1,j)+u(i-1,j))-u(i,j-1),
%           i=1,2,...,nx-1, j=1,2,...,nt-1

%%
% Implementamos el esquema descrito en la funcin exponda08. Construimos 
% los archivos f.m y g.m para las funciones que describen las condiciones 
% iniciales y los archivos l.m y r.m para las funciones de las condiciones
% de contorno.
% Llamamos a la funcin exponda08 con los parmetros de entrada dados por 
% el problema:

Uex = exponda08(10,0,1,2000,1,1,'f','g','l','r')

%%
% Hemos obtenido una matriz de 11 filas y 2001 columnas. En cada columna
% aparece la solucin aproximada del problema en un instante fijo. Por
% ejemplo, la solucin en t=1 se encuentra en la ltima columna, la
% solucin para t=0.5, se encuentra en la columna 1001, etc.

%%
% Para representar la solucin en los instantes t=0, t=0.25, t=0.5, t=0.75
% y t=1, basta representar las columnas correspondientes a esos instantes:

x=0:0.1:1;
plot(x,Uex(:,1),x,Uex(:,501),x,Uex(:,1001),x,Uex(:,1501),x,Uex(:,2001))

%%
% b) Esquema en diferencias implcito.
%
% El esquema implcito para este problema tiene la expresin matricial:
%
%       Au^(j+1)=Bu^(j-1)+(k^2-2)u^(j),  j=1,2,...,nt-1
% 
% donde u^(j) representa la solucin en el instante t=t_j.
%
% Las matrices A y B son ambas tridiagonales:
%       A=tridiagonal(d0,d1,d1),
%       B=tridiagonal(-d0,-d1,-d1),
% donde
%       d0=[-1-a^2,...,-1-a^2], diagonal principal
%       d1=[a^2/2,...,a^2/2], sub y superdiagonal
% siendo a=k/h.

%%
% Implementamos el esquema descrito en la funcin imponda08. Llamamos a
% esta funcin con los parmetros de entrada indicados en el problema:

Uim = imponda08(10,0,1,2000,1,1,'f','g','l','r')

%%
% Hemos obtenido una matriz en la que cada columna representa la solucin
% en un instante de tiempo. La solucin en el instante final t=1, se
% encuentra en la ltima columna de la matriz

Uim(:,2001)

%%
%
% FIN del Problema 10.8
