%% 10.9. Ecuacin hiperblica complicada
%
% Ecuacin hiperblica con condiciones de contorno mixtas. Aproximamos la
% solucin del problema mediante un esuqema en diferencias implcito.
% Representamos las soluciones aproximadas en todos los instantes de
% tiempo.
%
%
% Tema 10. Problema 9
% Problemas resueltos de mtodos numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson

%%
%
% a) Los nodos espaciales y temporales son:
%           x(i)=ih,    h=1/nx, i=0,1,...,nx
%           t(j)=jk,    k=1/nt, j=0,1,...,nt
% donde nx y nt son el nmero de subintervalos espaciales y temporales,
% respectivamente. Utilizamos la notacin habitual u(i,j) = u(x(i),t(j)).
%
% A partir de las diferencias finitas centrales para las parciales
% primeras, es posible deducir para la derivada parcial segunda cruzada la
% aproximacin de orden O(k^2+h^2):
%
%           (u(x+h,t+k)-u(x-h,t+k)-u(x+h,t-k)+u(x-h,t-k))/(4hk)
%
%


%%
%
% El esquema en diferencias explcito para este problema es:
%
% b(i) u(i+1,j+1)-a u(i,j+1)+c(i) u(i-1,j+1) = -6/k^2 u(i,j)-c(i) u(i+1,j-1)+
%       +a u(i,j-1)-b(i) u(i-1,j-1)+cos(pi x(i)), i=1,2,...,nx, j= 1,2,...,nt-1
%
% donde a=1/h^2+3/k^2, b(i)=1/(2h^2)+x(i)/(2hk) y c(i)=1/(2h^2)-x(i)/(2hk).
%
% Debemos prestar un poco de atencin a la ltima ecuacin de este sistema
%
% b(nx)+c(nx) u(nx+1,1)-2a u(nx,1)+(b(nx)+c(nx)) u(nx-1,1) =
%           -6/k^2 u(nx,0)+cos(pi x(nx))
%
% u(nx+1,1) no es una incgnita de nuestro problema. Si aproximamos la
% segunda condicin de contorno por una diferencia central, tenemos
%
%   u(nx+1,j) = u(nx-1,j)+4ht(j)
%
% Al sustituir u(nx+1,j) en la ltima ecuacin, completamos el sistema.

%%
%
% El sistema se expresa matricialmente de la forma:
%
% A u(:,j+1) = -6/k^2 u(:,j)+B u(:,j-1)+cos(pi x)-p(j) j=1,2,...,nt-1
%
% donde la matriz de coeficientes es una matriz tridiagonal en la que
%
% dp = -a*ones(1,nx)
% ds = b(1:nx-1)
% di = [c(2:nx-1) b(nx)+c(nx)]
%
% son las diagonales principal, superior e inferior respectivamente.

%%
%
% y los elementos que aparecen en los trminos independientes son, por una
% parte, la matriz B de la que
%
% dp = a*ones(1,nx)
% ds = -c(1:nx-1)
% di = [b(2:nx-1) b(nx)+c(nx)]
%
% son las diagonales principal, superior e inferior respectivamente,
% y p_j = (0,0,...,0,4h(b(nx)t(j+1)+c(nx)t(j-1)))^T.

%%
%
% b) Generamos la funcin imponda09.m en la que introducimos las
% particularidades del problema. Se muestra por una parte la solucin
% aproximada en el instante t(1) y en el instante final. Adems, se
% proporciona la representacin grfica de las curvas solucin en cada
% instante.


U = imponda09(10,0,1,200,1,'f','l')
x=0:.1:1;
plot(x,U)

%%
%
% FIN del Problema 10.9