%% 10.10. Ecuacin parablica no estndar
%
% Ecuacin en derivadas parciales parablica, con condiciones de contorno
% tipo Dirichlet y condicin inicial. Aproximamos la solucin del problema
% mediante diferencias finitas, mtodos explcito e implcito, comparando 
% resultados. 
% Describimos el esquema en diferencias que resulta al aplicar la idea del
% mtodo de Crank-Nicholson a este problema.
%
%
% Tema 10. Problema 10
% Problemas resueltos de mtodos numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson

%%
%
% a) Esquema en diferencias explcito.
%
% Tomamos los nodos espaciales y temporales
%           x(i)=ih,    h=1/nx, i=0,1,...,nx
%           t(j)=jk,    k=1/nt, j=0,1,...,nt
% donde nx y nt son el nmero de subintervalos espaciales y temporales,
% respectivamente.
% El esquema en diferencias explcito para este problema tiene la
% expresin:
%
%   u(i,j+1)=((k/h^2)+kx(i)/(2h))u(i+1,j)+((-2k/h^2)+k+1)u(i,j)+
%            ((k/h^2)-kx(i)/(2h))u(i-1,j),
%   i=1,2,...,nx-1, j=0,1,...,nt-1

%%
% Implementamos este esquema en el archivo expcalor10.m. Construimos el
% archivo f.m con la funcin que describe la condicin inicial, y los
% archivos c1.m y c2.m con las funciones de las condiciones de contorno.
% Llamamos a la funcin expcalor10, con los parmetros de entrada
% establecidos por el problema:

Uex = expcalor10(10,0,1,1000,0.5,'f','c1','c2')

%%
% En cada columna de la matriz Uex tenemos el valor aproximado de la
% solucin en un instante de tiempo. La solucin en el instante final est
% en la ltima columna.

Uex(:,end)

%%
%
% b) Esquema en diferencias implcito.
%
% El esquema en diferencias implcito para este problema tiene la
% expresin:
%
%   -((k/h^2)+kx(i)/(2h))u(i+1,j)+((2k/h^2)-k+1)u(i,j)-
%            -((k/h^2)-kx(i)/(2h))u(i-1,j)= u(i,j-1),
%   i=1,2,...,nx-1, j=1,2,...,nt
%
% Implementamos este esquema en el archivo impcalor10.m. Llamamos a esta
% funcin con los parmetros de entrada descritos por el problema.

Uim = impcalor10(10,0,1,1000,0.5,'f','c1','c2')

%%
% En cada columna de la matriz Uim tenemos el valor aproximado de la
% solucin en un instante de tiempo. La solucin en el instante final est
% en la ltima columna.

Uim(:,end)


%%
% Podemos hacer la grfica de la solucin en determinados instantes sin ms
% que representar las columnas correspondientes de la matriz Uim. Por
% ejemplo, la solucin en los instantes t=0, t=0.1, t=0.2, t=0.3, t=0.4 y
% t=0.5 se encuentra en las columnas 1, 201, 401, 601, 801 y 1001,
% respectivamente. Representamos estas columnas:

x=0:0.1:1;
C=[1 201 401 601 801 1001];
plot(x,Uim(:,C))

%%
%
% FIN del Problema 10.10